//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: BaseObject.java,v 1.7 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: BaseObject.java,v $
//  Revision 1.7  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/12/04 21:52:54  cvs
//  BaseObject class now implements Serializable.
//  -- Cindy
//
//  Revision 1.5  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.4  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.3  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.2  2000/08/09 04:29:51  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/08 22:50:55  cvs
//  The BaseObject.java the super class of all objects in the object
//  model.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.io.Serializable;

/** This represents the <B>BaseObject</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public abstract class BaseObject implements Serializable{

	private BaseObject parent;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public BaseObject() {
	    parent = null;
	}

	/** Another Constructor
	 * @param parentIn The BaseObject parent 
	 * @return Not applicable
	 */
	public BaseObject(BaseObject parentIn) {
		parent = parentIn;
	}

	/** To retrieve the parent
	 * @return BaseObject representing the parent of this object
	 */
	public BaseObject getParent() {
		return parent;
	}

    /** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public abstract String toString();

	/**Abstract method declared to look up for a label
	 * @param Label
	 * @return LabelledItem
	 */
	public abstract LabelledItem labelLookup(Label labelIn);
	
	//Added by Srinivas on Dec 2nd, 2000
	/* This function is to get the type from the parent
	 * @param String
	 * @return Type
	 */
	public Type getType(String str){
		if (getParent() != null)
			return getParent().getType(str);
		else 
			return null;
	}

	//Added by srinivas on Dec 2nd, 2000
	/* This function is to check if the type already exsists
	 * @param String
	 * @param Type
	 * @return Type
	 */
	public Type alreadyDeclared(String str, Type type){
		if (getParent() != null)
			return getParent().alreadyDeclared(str,type);
		else 
			return null;
	}

}
		

