//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Expression.java,v 1.11 2001/03/12 23:56:46 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Expression.java,v $
//  Revision 1.11  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.9  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.8  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.7  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.6  2000/09/18 20:43:10  cvs
//  Formatted file.
//  -- Cindy
//
//  Revision 1.5  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.4  2000/09/16 19:48:15  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.3  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.2  2000/08/10 01:04:12  cvs
//  All object in ROM has a parent, therefore implemented the constructors
//  to take in a BaseObject parent.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:07  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.2  2000/08/09 04:29:51  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 00:51:14  cvs
//  Skeletons for several added classes. -David
//
//
//=============================================================================
package ROM;
import java.io.*;
/** This represents the <B>Expression</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public abstract class Expression 
extends BaseObject {

	//changed the constructor so that the param is an Object of type BaseObject
	/** Primary constructor
	 * @param parentIn BaseObject that represents this Value's parent.
	 * @return Not applicable
	 */
	public Expression(BaseObject parent) {
		super(parent);
	}

	//Added by Srinivas on Dec 2nd, 2000
	/* This function is to get the type from the parent
	 * @param String
	 * @return Type
	 */
	public Type getType(String str){
		if (getParent() != null)
			return getParent().getType(str);
		else
			return null;
	}
	
	//Added by srinivas on Dec 2nd, 2000
	/* This function is to check if the type already exsists
	 * @param String
	 * @param Type
	 * @return Type
	 */
	public Type alreadyDeclared(String str, Type type){
		if (getParent() != null)
			return getParent().alreadyDeclared(str,type);
		else 
			return null;
	}

	/** labelLookup
	 *
	 * @param labelIn - label to search for
	 * @return LabelledItem -   found in this scope
	 */
	public LabelledItem labelLookup(Label labelIn) {
		return getParent().labelLookup(labelIn);
	}

	public abstract void publishROM(PrintWriter pw);

	// added by srinivas on Dec 3rd, 2000
	/** To determine if expression is pre or post condition
	 * @return The boolean value, true if it is a post condition and
	 * false otherwise 
	 */
	public boolean isTickExpression() {
		try {
			String ret_str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw, true);
			this.publishROM(pw);
			pw.flush();
			pw.close();
			ret_str = sw.toString();
			sw.close();
			if((ret_str.indexOf('\'')>0) ||(ret_str.indexOf('@')> 0))
			    return true;
			else
			    return false;
		}
		catch (Exception e) {
		    System.out.println(e);
		    return false;
		}
	}
    
	
	
}

		


