//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Facet.java,v 1.16 2001/10/09 23:39:09 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Facet.java,v $
//  Revision 1.16  2001/10/09 23:39:09  cvs
//  Relabeling facet instantiation now works. A case was simply missing
//  in the labelLookup function - there was no lookup for labels in
//  already defined terms.
//  -- Cindy
//
//  Revision 1.15  2001/10/01 18:50:18  cvs
//  I added a couple of predefined operators that were missing in
//  universe of ROM. Tabulation is set to two. 
//  -- Cindy
//
//  Revision 1.14  2001/09/27 22:31:58  cvs
//  Had Some problems with the previous commit.Cooresponding changes
//  were made and now the parser supports
//  interactions,domains,enumerations. 
//   --Murthy
//
//  Revision 1.13  2001/08/28 23:30:26  cvs
//  Added the implementation of Domains and interactions
//     --Murthy
//
//  Revision 1.12  2001/03/26 19:01:42  cvs
//  Completed the implementation for the change in export clauses.
//  -- Cindy
//
//  Revision 1.11  2001/03/12 23:56:46  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.9  2001/02/07 20:52:20  cvs
//  There was an error causing an infinite loop when doing label lookup in
//  use clauses. Modifications were made in labelLookup methods to remove
//  error.
//  -- Cindy
//
//  Revision 1.8  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.7  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.4  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.3  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.2  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.1  2000/08/09 04:29:52  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.ListIterator;
import java.io.*;
import java.lang.String;
/** This represents the <B>Facet</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class Facet
  extends Expression {

  ItemList interfaceList;
  ItemList declaredItems;
  ItemReferenceList uses = null;
  ItemList terms;
  Label domain;
  LabelList exportList;
  int refCount = 0;
  TypeList types;

  /** The primary Constructor
   * @param parentIn Item representing the parent of this Facet
   * @return Not applicable
   */
  public Facet(Item parentIn) {
    super(parentIn);
    interfaceList = new ItemList();
    declaredItems = new ItemList();
    terms = new ItemList();
    exportList = new LabelList();
    types = new TypeList();
  }

  /** To add to the uses list
   * @param facetRef The FacetReference to be added
   * @return void
   */
  public void addUseClauseItem(FacetReference facetRef) {
    uses.addItemReference(facetRef);
  }

  /** To set the uses list
   * @param useListIn The ItemReferenceList 
   * @return void
   */
  public void setUseClauseList(ItemReferenceList useListIn) {
    uses = useListIn;
  }

  /** To get the uses list
   * @return The ItemReferenceList corresponding to use clauses
   */
  public ItemReferenceList getUseClause() {
    return uses;
  }

  /** To increment refCount
   * @return void
   */
  public void incrementRefCount() {
    refCount++;
  }

  /** To get the refCount
   * @return refCount
   */
  public int getRefCount() {
    return refCount;
  }

  /** To add an interface
   * @param interfaceItemIn The Item to be added
   * @return void
   */
  public void addInterfaceItem(Item interfaceItem) {
    interfaceList.addItem(interfaceItem);
  }

  /** To set the interface list
   * @param listIn, ItemList to be set
   * @return void
   */
  public void setInterfaceItems(ItemList listIn) {
    interfaceList = listIn;
  }

  /** To get the interface list
   * @return The ItemList containing the interfaces
   */
  public ItemList getInterfaceItems() {
    return interfaceList;
  }

  /** To add an item
   * @param declaredItemIn The declared item to be added
   * @return void
   */
  public void addItem(Item declaredItemIn) {
    declaredItems.addItem(declaredItemIn);
  }

  /** To set the item list
   * @param listIn, list of items to be set
   * @return void
   */
  public void setItems(ItemList listIn) {
    declaredItems = listIn;
  }

  /** To get the item list
   * @return The ItemList containing declared items
   */
  public ItemList getItems() {
    return declaredItems;
  }

  /** To set the type list
   * @param listIn, list of types to be set
   * @return void
   */
  public void setTypes(TypeList listIn) {
    types = listIn;
  }

  /** To get the type list
   * @return The TypeList containing declared type items
   */
  public TypeList getTypes() {
    return types;
  }
	
  /**To add a type to the type list
   * @param type The type to be added
   * @return void
   */
  public void addTypeElement(Type type){
    types.addItem(type);
  }

  /** To set the domain
   * @param domainIn The Label corresponding to the domain
   * @return void
   */
  public void setDomain(Label domainIn) {
    domain = domainIn;
  }

  /** To get the domain
   * @return domain
   */
  public Label getDomain() {
    return domain;
  }

  /** To add a term
   * @param termIn The term to be added
   * @return void
   */
  public void addTerm(Item termIn) {
    terms.addItem(termIn);
  }
	
  /** To set the term list
   * @param listIn, the term list to be set
   * @return void
   */
  public void setTerms(ItemList listIn) {
    terms = listIn;
  }

  /** To get the term list
   * @return SOM_ItemList containing terms
   */
  public ItemList getTerms() {
    return terms;
  }

  /** To add to the export list
   * @param exportIn The export label to be added
   * @return void
   */
  public void addExportLabel(Label exportIn) {
    exportList.addLabel(exportIn);
  }

  /** To set export list
   * @param listIn, list of export names to be set
   * @return void
   */
  public void setExportLabels(LabelList listIn) {
    exportList = listIn;
  }

  /** To get export list
   * @return exportList
   */
  public LabelList getExportLabels() {
    return exportList;
  }
	
  //Added by srinivas on Dec 1st, 2000
  /** To get the type corresponding to a name (String)
   * @param nameIn String corresponding to name of type
   * @return Type, type corresponding to name
   */
  public Type getType(String nameIn) {
    ListIterator declIter = getItems().listIterator();
    Type type =null;
    if (declIter != null){
      while (declIter.hasNext()){
	LabelledItem item = (LabelledItem)declIter.next();
	if (item.getLabel().toString().equals(nameIn)){
	  return item.getType();
	}
      }
    }
		
    if (getUseClause() != null) {
      ListIterator useIterator = getUseClause().listIterator();
      ItemReference useRef;
      LabelledItem toReturn;
      while (useIterator.hasNext()) {
	useRef = (ItemReference)useIterator.next();
	Facet curItem = 
	  (Facet)((Item)useRef.getItemReferred()).getValue();
	LabelList refExportList = curItem.getExportLabels();
	if (refExportList.containsLabelString("ALL") ||
	    refExportList.containsLabelString(nameIn)) {
	  type = curItem.getType(nameIn);
	}
	if (type != null)
	  return type;
      }
    }
    if (type == null)
      if (getParent() instanceof Universe){
	return null;
      }else
	return getParent().getType(nameIn);
    return type;
  }

  /** To get the string representation of the object
   * @return String representing the string of this object.
   *
   * SLP:	 Shouldn't it just get the itemString from its parent Item??
   */
  public String toString() {
    // To be completed !!
    return getParent().toString();
  }

  //Added by srinivas on Dec 1st, 2000
  /**To check if the array or set or sequence of the type already
   * exsists
   * @param type To know if it is a array, set or sequence
   * @param elementType To search for the element type
   * @return Type 
   */
  public Type alreadyDeclared(String type, Type elementType){
    Type result = types.alreadyDeclared(type,elementType);
    if (result == null){
      return getParent().alreadyDeclared(type,elementType);
    }else
      return result;
  }

  /** To look up a label
   * @param labelIn Label to be looked up
   * @return Item corresponding to label
   */
  public LabelledItem labelLookup(Label labelIn){
    Item curItem;
    LabelledItem curLabelledItem;
    ListIterator interIterator = interfaceList.listIterator();
    while (interIterator.hasNext()) {
      curItem = (Item)interIterator.next();	 
      if (curItem instanceof LabelledItem) {
	curLabelledItem = (LabelledItem)curItem; 
	if (curLabelledItem.getLabel().equals(labelIn))
	  return curLabelledItem;
      }
    }
    ListIterator itemIterator = declaredItems.listIterator();
    while (itemIterator.hasNext()) {
      curItem = (Item)itemIterator.next();	
      if (curItem instanceof LabelledItem) {
	curLabelledItem = (LabelledItem)curItem;
	if (curLabelledItem.getLabel().equals(labelIn)) {
	  return curLabelledItem;
	}
      }
    }
    // added by ckong October 9, 2001
    // Case when a previous term represents renaming of an instance of
    // a facet
    if (getTerms() != null) {
      for (ListIterator iter = getTerms().listIterator();iter.hasNext();) {
	LabelledItem term = (LabelledItem)iter.next();
	if (term.getLabel().equals(labelIn))
	  return term;
      }  
    }

    if (getUseClause() != null) {
      ListIterator useIterator = getUseClause().listIterator();
      ItemReference useRef;
      LabelledItem toReturn = null;
      while (useIterator.hasNext()) {
	useRef = (ItemReference)useIterator.next();
	curItem = useRef.getItemReferred();
	// Calling the labelLookup on the value of the
	// (package) item directly
	if (curItem instanceof LabelledItem) {
	  Facet refFacet = (Facet)((LabelledItem)curItem).getValue();
	  LabelList refExportList = refFacet.getExportLabels();
	  if (refExportList.containsLabelString("ALL") ||
	      refExportList.containsLabelString(labelIn.toString()))
	    toReturn = refFacet.labelLookup(labelIn);

	  if (toReturn != null) {
	    return toReturn;
	  }
	}
      }
    }
    return getParent().labelLookup(labelIn);
  }
	
  /*Added by Srinivas to print the ROM structure
   * @param PrintWriter
   * @return void
   */
  public void publishROM(PrintWriter pw){
    if (getUseClause() != null) {
      pw.println("THE USE ITEMS: ");
      ListIterator useIter = getUseClause().listIterator();
      while (useIter.hasNext()) {
	pw.print("package: ");
	((ItemReference)useIter.next()).publishROM(pw);
      }
    }
    
    if (getInterfaceItems() != null){
      ListIterator interItemIter = getInterfaceItems().listIterator();
      if (interItemIter != null){
	pw.println("THE INTERFACE ITEMS:");
	while (interItemIter.hasNext()){
	  Item item= (Item)interItemIter.next();
	  pw.println(item.toString());
	}
      }
    }
    if (getItems() !=null){
      ListIterator declItemIter = getItems().listIterator();
      if (declItemIter !=null){
	pw.println("THE DECLARED ITEMS:");
	while (declItemIter.hasNext()){
	  Item declItem = (Item)declItemIter.next();
	  if (declItem.getValue() instanceof Package){
	    pw.println("FOR INNER PACKAGE");
	    ((Package)declItem.getValue()).publishROM(pw);
	    pw.println("FINISHED INNER PACKAGE");
	  }else if (declItem.getValue() instanceof Domain) {
	    pw.println("FOR INNER DOMAIN");
	    ((Domain)declItem.getValue()).publishROM(pw);
	    pw.println("FINISHED INNER DOMAIN");
	  }else if (declItem.getValue() instanceof Component){
	    ((Component)declItem.getValue()).publishROM(pw);
	  }else if (declItem.getValue() instanceof Interaction) {
	    pw.println("FOR INNER INTERACTION");
	    ((Interaction)declItem.getValue()).publishROM(pw);
	    pw.println("FINISHED INNER INTERACTION");
	  }else if (declItem.getValue() instanceof Facet){
	    pw.println("FOR INNER FACET:");
	    ((Facet)declItem.getValue()).publishROM(pw);
	    pw.println("FINISHED INNER FACET:");
	  }else {
	    //pw.print(declItem.toString());
	    pw.println(declItem.getString());
	  }
	}
      }
    }
    if (getExportLabels() !=null){
      ListIterator exports = getExportLabels().listIterator();
      if (exports !=null){
	pw.println("THE EXPORT LIST IS:");
	while(exports.hasNext()){
	  Label label = (Label)exports.next();
	  pw.println(label.toString());
	}
      }
    }

    if (domain !=null){
      pw.println("THE DOMAIN IS:");
      pw.println(domain.toString());
    }
    if (getTerms() !=null){
      ListIterator termEnum = getTerms().listIterator();
      if (termEnum !=null){
	pw.println("THE TERMS ARE");
	while(termEnum.hasNext()){
	  pw.println();
	  Item item = (Item)termEnum.next();
	  pw.print(((LabelledItem)item).getLabel().toString());
	  pw.print(":");
	  Expression term = item.getValue();
	  if (term !=null){
	    term.publishROM(pw);
	  }
	  pw.print(";");
	  pw.println();
	}
      }
    }
  }

}
