//=============================================================================
//	authors: Steve Parker
//
//  $Id: Factory.java,v 1.9 2001/10/01 18:50:18 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Factory.java,v $
//  Revision 1.9  2001/10/01 18:50:18  cvs
//  I added a couple of predefined operators that were missing in universe of ROM. Tabulation is set to two.
//  -- Cindy
//
//  Revision 1.8  2001/08/28 23:30:26  cvs
//  Added the implementation of Domains and interactions
//     --Murthy
//
//  Revision 1.7  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.6  2000/12/04 19:39:39  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.5  2000/11/13 23:50:53  cvs
//  Changed the method newFunction so that it takes a BaseObject as a
//  parameter rather than a Item 
//  --srinivas
//
//  Revision 1.4  2000/10/25 20:50:06  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.3  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//
//=============================================================================

package ROM;

/** This represents the <B>Factory</B> object
 * @author <I>Steve Parker</I> 
 * <a href="mailto:sparker@averstar.com">sparker@averstar.com</a>
 */
public class Factory {


	/** The primary Constructor
	 * @return Not applicable
	 */
	public Factory() {
	}

	/**
	 * Instantiates a Universe
	 * @return Universe that gets created
	 */
	public Universe newUniverse() {
	    Universe thisUniverse = new Universe();
          
		/**
	     * Add predefined Items here (in Factory) rather than in the 
		 * constructor of Universe.
	     **/
		/* ????  What gets added here:  Integer, Real, Set Sequence, 
		   Array, etc
		   A predefinedItem is ...  ?   
		   Type is an Item with a bunch for a value
		*/
		// Null
		LabelledItem nullItem =
			newLabelledItem(thisUniverse, null, null, null, "null");
		Label nullLabel = newLabel(nullItem, "null");
		nullItem.setLabel(nullLabel);
		BunchValue nullVal = 
			new BunchValue(nullItem, thisUniverse.getType("universal"), null); 
		nullItem.setValue(nullVal);
		nullItem.setType(thisUniverse.getType("universal"));
		thisUniverse.setPredefinedItem(nullItem);
		
		return thisUniverse;

	}

	/**
	 * Instantiate an Item
	 *
	 * @param parentIn  BaseObject representing the parent.
	 * @param typeIn The type.
	 * @param stringIn String of source file corresponding to this.
	 * @param valueIn Expression representing the value, defaults to 
         * null (if no value to be set).
	 * @return Item that gets created. 
	 **/
	/*
	  public Item newItem(BaseObject parentIn, Type typeIn, 
						Expression valueIn, String stringIn) {
		if (valueIn != null) {
			return new Item(parentIn, typeIn, valueIn, stringIn);
		} else {
			return new Item(parentIn, typeIn, stringIn);
		}
	}
	*/

	/**
	 * Instantiate a LabelledItem
	 *
	 * @param parentIn  BaseObject representing the parent.
	 * @param typeIn The type.
	 * @param stringIn String of source file corresponding to this.
	 * @param valueIn Expression representing the value, defaults to 
	 * null (if no value to be set).
	 * @return LabelledItem that gets created. 
	 **/
	public LabelledItem newLabelledItem(BaseObject parentIn,
										Label labelIn, 
										Type typeIn, 
										Expression valueIn,
										String stringIn) {
		if (valueIn != null) {
			return new LabelledItem(parentIn, labelIn, typeIn, 
									valueIn, stringIn);
		} else {
			return new LabelledItem(parentIn, labelIn, typeIn, stringIn);
		}
	}

	/**
	 * Instantiate a ParameterItem
	 *
	 * @param parentIn  BaseObject representing the parent.
	 * @param typeIn The type.
	 * @param stringIn String of source file corresponding to this.
	 * @param valueIn Expression representing the value, defaults to 
	 * null (if no value to be set).
	 * @param modeIn Constant int used in representing mode
	 * @return LabelledItem that gets created. 
	 **/
	public ParameterItem newParameterItem (BaseObject parentIn,
										   Label labelIn, 
										   Type typeIn, 
										   Expression valueIn,
										   int modeIn,
										   String stringIn
										   ) {
		if (valueIn != null) {
			return new ParameterItem(parentIn, labelIn, typeIn, 
									valueIn, modeIn, stringIn);
		} else {
			return new ParameterItem
				(parentIn, labelIn, typeIn, modeIn, stringIn);
		}
	}

	/** 
	 * Instantiate a Label
	 * @param parentIn The parent of this label.
	 * @param nameIn The string representing name of the label.
	 * @return Label that gets created.
	 */
	public Label newLabel(Item parentIn, String nameIn) {
	    return new Label(parentIn, nameIn);
	}

	/** 
	 * Instantiate a Facet
	 * @param parentIn the parent of this Facet (Item or Facet)
	 * @return Facet that gets created.
	 */
	public Facet newFacet(Item parentIn) {
	  return new Facet(parentIn);
	}
       
                /**
         * Instantiate a Domain
         * @param parentIn the parent of this Facet (Item or Facet)
         * @return Facet that gets created.
         */
        public Domain newDomain(Item parentIn) {
          return new Domain(parentIn);
        }
	
  /**
         * Instantiate an Interaction 
         * @param parentIn the parent of this Facet (Item or Facet)
         * @return Facet that gets created.
         */
        //public Facet newInteraction(Item parentIn) {
          //return new Interaction(parentIn);
        //}
	
	public Interaction newInteraction(Item parentIn) {
	  return new Interaction(parentIn);
	}
 
	/**
	 * Instantiate a Component.
	 *
	 * @param parentIn Item representing parent of this Component. 
	 * @param assumptionIn Item representing the ASSUMPTION Facet.
	 * @param definitionIn Item representing the DEFINITION Facet.
	 * @param verificationIn Item representing the VERIFICATION Facet.
	 * @return Component that gets created.
	 */
	public Component newComponent(Item parentIn, 
								  Item assumptionIn, 
								  Item definitionIn,
		                          Item verificationIn) {
		return new Component(parentIn, assumptionIn, 
							 definitionIn, verificationIn);
	}

	/**
	 * Instantiate a Component.
	 *
	 * @param parentIn Item representing parent of this Component. 
	 * @param assumptionIn Item representing the ASSUMPTION Facet.
	 * @param definitionIn Item representing the DEFINITION Facet.
	 * @param verificationIn Item representing the VERIFICATION Facet.
	 * @return Component that gets created.
	 */
	public Component newComponent(Item parentIn) {
		return new Component(parentIn);
	}

	/**
	 * Instantiate an Interaction.
	 *
	 * @param parentIN Item representing parent of this Interaction.
	 * @return Interaction that gets created.
	 */
        //public Interaction newInteraction(Item parentIn) {
	//    return new Interaction(parentIn);
	//}
    
	/**
	 * Instatiate a Package.
	 *
	 * @param parentIn The parent of this Package.
	 * @return Package that gets created.
	 */
	public Package newPackage(Item parentIn) {
	    return new Package(parentIn);
	}

	/**
	 * Instantiate a Function.
	 *
	 * @param parentIn Item representing the parent of this Function.
	 * @return Function which is created.
	 */
	public Function newFunction(BaseObject parentIn) {
	  return new Function(parentIn);
	}

}
