//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: FunctionReference.java,v 1.11 2001/03/12 23:56:47 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: FunctionReference.java,v $
//  Revision 1.11  2001/03/12 23:56:47  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Also added some predefined operators to the
//  Universe. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.10  2001/02/01 22:34:30  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.9  2000/12/03 23:59:45  cvs
//  Added the new types of array, set and sequence. The methods
//  getType  and alreadyDeclared are also added to get a type that has
//  been previously declared. 
//  --srinivas
//
//  Revision 1.8  2000/11/27 23:46:52  cvs
//  Changed the publishROM method in  all the classes and added some
//  predefined operators to the Universe. 
//  --srinivas
//
//  Revision 1.7  2000/11/13 21:49:11  cvs
//  Added the function buildROM to all the classes.
//  --srinivas
//
//  Revision 1.6  2000/10/30 22:34:30  cvs
//  Added secondary constructors to some of the classes and some
//  accessor functions are also added 
//  --srinivas
//
//  Revision 1.5  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.4  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//  Revision 1.3  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.2  2000/08/30 20:39:11  cvs
//  Changed Expression class to inherit from Value and amended all
//  other classes affected by this change. -David 
//
//  Revision 1.1  2000/08/10 00:49:48  cvs
//  Implemented the new hierarchy for ItemReference and added some
//  accessor methods.
//  -- Cindy
//
//
//=============================================================================
package ROM;

import java.io.*;
import java.util.*;
/** This represents the <B>FunctionReference</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */

public class FunctionReference 
extends ParametizedReference {

	//changed the type of param from Item to BaseObject
	/** Primary constructor
	 * @param parent The BaseObject parent
	 * @param itemIn The function item being referred
	 * @return Not applicable
	 */
	public FunctionReference(BaseObject parent, Item itemIn) {
		super(parent, itemIn);
	}

	public LabelledItem labelLookup(Label labelIn){
		return getParent().labelLookup(labelIn);
	}

	public void publishROM(PrintWriter pw){
		if (itemReferred instanceof LabelledItem){
			String str =
				((LabelledItem)itemReferred).getLabel().toString();
			if (str.equals("if")){
				pw.print("if ");
				ListIterator itemIter =
					getParameters().listIterator();
				((Expression)itemIter.next()).publishROM(pw);
				pw.print(" then ");
				((Expression)itemIter.next()).publishROM(pw);
				pw.print(" else ");
				((Expression)itemIter.next()).publishROM(pw);
				pw.print(" endif ");
			}else{
				pw.print(str);
				ListIterator itemIter =
					getParameters().listIterator();
				if (itemIter != null){
					if (getParameters().size() != 0){
						pw.print("(");
						while (itemIter.hasNext()){
							((Expression)itemIter.next()).publishROM(pw);
							if (itemIter.hasNext())
								pw.print(",");
						}
						pw.print(")");
					}
				}
			}
		}
	}
}

		

