//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: Item.java,v 1.8 2001/05/30 16:31:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Item.java,v $
//  Revision 1.8  2001/05/30 16:31:58  cvs
//  Added some accessor methods to the classes. The methods are used by the Moml to Rosetta converter.
//
//  --Srinivas
//
//  Revision 1.7  2000/12/04 19:39:38  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  Class Item is now an abstract class.
//  -- Cindy
//
//  Revision 1.6  2000/11/17 23:14:13  cvs
//  Added the method labelLookup in some classes and changed the same
//  method in others 
//  --srinivas
//
//  Revision 1.5  2000/10/25 20:50:07  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.4  2000/09/16 19:48:16  cvs
//  Revamped ROM class hierarchy.  Committed partially skelatal
//  classes that successfully compile. 
//
//  Revision 1.3  2000/08/22 18:40:17  cvs
//  First Commit by Steve Parker, Averstar.  Basic Structure of
//  classes setup for ROM. 
//
//  Revision 1.3  2000/08/09 04:29:53  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//  Revision 1.2  2000/08/08 23:40:25  cvs
//  Revised version of Item class. -David
//
//  Revision 1.1  2000/08/08 22:50:55  cvs
//  This is the Item class. -David
// 
// 
//
//
//
//=============================================================================
package ROM;

/** This represents the <B>Item</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public abstract class Item 
extends BaseObject {

	private Type type;
	private Expression value = null;
	private String itemString;

	/** The primary constructor
	 * @param parent BaseObject representing the parent
	 * @param typeIn Type representing the type
	 * @param valueIn Value representing value of the item
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public Item(BaseObject parent, Type typeIn, Expression
					valueIn, String stringIn) { 
		super(parent);
		type = typeIn;
		value = valueIn;
		itemString = stringIn;
	}
    
	/** Another constructor
	 * @param parent BaseObject representing the parent
	 * @param typeIn Type representing the type
	 * @param stringIn String representation of item
	 * @return Not applicable
	 */
	public Item(BaseObject parent, Type typeIn, String stringIn){
		this(parent, typeIn, null, stringIn);
	}

	/** To retrieve the type
	 * @return Type representing the type of this object
	 */
	public Type getType() {
		return type;
	}

	/** To set the type
	 * @param typeIn The type
	 * @return void
	 */
	public void setType(Type typeIn) {
		type = typeIn;
	}

	/** To retrieve the value
	 * @return Value representing the value of this object
	 */
	public Expression getValue() {
		return value;
	}

	/** To set the expression value
	 * @param exprIn Expression representing value associated with
	 * item
	 * @return void
	 */
	public void setValue(Expression exprIn) {
		value = exprIn;
	}
		
    /** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String getString() {
		return itemString;
	}

	/** To set the string representation of the object
	 * @param stringIn String representation of the object
	 * @return void
	 */
	public void setString(String stringIn) {
		itemString = stringIn;
	}

}
	       









