//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: LabelList.java,v 1.2 2001/03/26 19:01:42 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: LabelList.java,v $
//  Revision 1.2  2001/03/26 19:01:42  cvs
//  Completed the implementation for the change in export clauses.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:11  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.ListIterator;

/** This represents the <B>LabelList</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class LabelList
extends List {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public LabelList() {
		super();
	}

	/** To add an label
	 * @param label The Label to add
	 * @return void
	 */
	public void addLabel (Label label) {
		addElement(label);
	}

	/** To remove an label
	 * @param label The BaseObject to remove
	 * @return void
	 */
	public void removeLabel (Label label) {
		removeElement(label);
	}

	/** To retrieve the first label in the list
	 * @return Label representing the first label
	 */
	public Label firstLabel() {
		return (Label)first();
	}

	/** To retrieve the last label in the list
	 * @return Label representing the last object in the list
	 */
	public Label lastLabel() {
		return (Label)last();
	}

	/** To check if list contains a specific label
	 * @param label Label present or not in list
	 * @return boolean, true if label present, false otherwise
	 */
	public boolean containsLabel(Label label) {
		return containsElement(label);
	}

	/** To check if list contains a label with same string
	 * @param stringIn String of label being searched
	 * @return boolean, true if label corresponding to string found
	 */
	public boolean containsLabelString(String stringIn) {
		ListIterator iterator = listIterator();
		Label curLabel;
		while (iterator.hasNext()) {
			curLabel = (Label)iterator.next();
			if (curLabel.toString().equals(stringIn))
				return true;
		}
		return false;
	}
}
		

