//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy ckong@ittc.ukans.edu
//           David schond@ittc.ukans.edu
//
//  $Id: List.java,v 1.2 2001/03/26 19:01:42 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: List.java,v $
//  Revision 1.2  2001/03/26 19:01:42  cvs
//  Completed the implementation for the change in export clauses.
//  -- Cindy
//
//  Revision 1.1  2000/08/09 23:03:11  cvs
//  Dropped the ROM_ prefix on all files; dumped getElt() method from
//  List.java, made Universe a subclass of BaseObject. --David 
//
//  Revision 1.2  2000/08/09 16:43:05  cvs
//  Added getElt() method to List class. Hope it's ok. -David
//
//  Revision 1.1  2000/08/09 04:29:54  cvs
//  The data structures for the files I have added are more or less
//  complete. For several of the classes no implementation has been given
//  yet. Their presence gives an idea of the inheritance hierarchy.
//  -- Cindy
//
//
//
//=============================================================================
package ROM;

import java.util.LinkedList;

/** This represents the <B>List</B> object
 * @author <I> Cindy Kong and David Schonberger </I>
 * (ckong@ittc.ukans.edu, schond@ittc.ukans.edu)
 */
public class List
extends LinkedList {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public List() {
		super();
	}

	/** To add an element 
	 * @param element The BaseObject to add
	 * @return void
	 */
	public void addElement (BaseObject element) {
		add(element);
	}

	/** To remove an element 
	 * @param element The BaseObject to remove
	 * @return void
	 */
	public void removeElement (BaseObject element) {
		remove(element);
	}

	/** To retrieve the first object in the list
	 * @return BaseObject representing the first object
	 */
	public BaseObject first() {
		return (BaseObject)getFirst();
	}

	/** To retrieve the last object in the list
	 * @return BaseObject representing the last object in the list
	 */
	public BaseObject last() {
		return (BaseObject)getLast();
	}

	/** To check if list contains a specific element
	 * @param element BaseObject present or not in list
	 * @return boolean, true if element present, false otherwise
	 */
	public boolean containsElement(BaseObject element) {
		return contains(element);
	}

	/** To check if list is empty
	 * @return boolean, true if list is empty, false otherwise
	 */
	public boolean isEmpty() {
		return (size() == 0);
	}

}
		

