//=============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Srinivas Akkipeddi (akki@ittc.ukans.edu)
//
//  $Id: SequenceType.java,v 1.1 2000/12/03 23:59:46 cvs Exp $
//
//
//=============================================================================

package ROM;

import java.util.HashSet;

/** This represents the <B>ArrayIntegerType</B> object
 * @author <I>Srinivas Akkipeddi</I>
 * <a href="mailto:akki@ittc.ukans.edu"></a>
 */

public class SequenceType extends Type {
	
	private static BunchValue elements;
	private static Type elementType;
	
	/** The primary Constructor
	 * @param parentIn The parent object
	 * @param type the type of the element
	 * @param elementsIn The initial values in the type
	 * @return Not applicable
	 */
	public SequenceType(BaseObject parentIn, BunchValue elementsIn,
						Type type) {
		super(parentIn);
		elementType = type;
		elements = elementsIn;
	}
	
	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To add an element to the type
	 * @param elementIn The object to be added
	 * @return void
	 */
	public void addOneElement(Object elementIn) {
		elements.getElementSet().add(elementIn);
	}

	/**To get the element type
	 * @param void
	 * @return type The type of the element
	 */
	public Type getElementType(){
		return elementType;
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "Sequence of " + elementType.toString();
	}
}

