//=============================================================================
//	authors: Steve Parker
//
//  $Id: UniversalType.java,v 1.4 2001/08/28 23:30:27 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: UniversalType.java,v $
//  Revision 1.4  2001/08/28 23:30:27  cvs
//  Added the implementation of Domains and interactions
//     --Murthy
//
//  Revision 1.3  2000/10/25 20:50:08  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//
//=============================================================================
package ROM;

import java.util.HashSet;

/** This represents the <B>UniversalType</B> object
 * @author <I>Steve Parker</I>
 * <a href="mailto:sparker@titan.com">sparker@titan.com</a>
 */
public class UniversalType extends Type {

	private static BunchValue elements;

	/** The primary Constructor
	 * @param parentIn The parent object
	 * @param elementsIn The bunch value obtained from all current types
	 * @return Not applicable
	 */
	public UniversalType(BaseObject parentIn, BunchValue elementsIn) {
		super(parentIn);
		elements = elementsIn;
	}
	
	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public static boolean isTypeOf(Object valIn) {
		return (ElementType.isTypeOf(valIn) ||
				CharacterType.isTypeOf(valIn) ||
				elements.elementPresent(valIn));
	}
	
	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	/** To add new elements to the Universal type
	 * @param bunchIn The bunch to union
	 * @return void
	 */
	public void addBunch(BunchValue bunchIn) {
		elements.bunchConcat(bunchIn);
	}

	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return "univ";
	}

}

