//=============================================================================
//	authors: Steve Parker
//
//  $Id: UserDefinedType.java,v 1.5 2001/08/28 23:30:27 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: UserDefinedType.java,v $
//  Revision 1.5  2001/08/28 23:30:27  cvs
//  Added the implementation of Domains and interactions
//     --Murthy
//
//  Revision 1.4  2001/03/06 02:15:14  cvs
//  Added some accessor methods that are used in DVTG to the ROM classes
//
//  --srinivas
//
//  Revision 1.3  2000/10/25 20:50:08  cvs
//  Implemented some of the methods for a type. Each type contains a
//  bunch(BunchValue object) that groups the elements that are of that
//  type. For the built-in types, the bunch is fixed. For composite types
//  or user defined types, the bunch gets elements added to it when a new
//  element of the type is defined by a user.
//  Added ParameterItem class, as a parameter item is a specific case of a
//  labelled item (there is a field for the mode).
//  Initialized the built-in types in Universe.
//  -- Cindy
//
//  Revision 1.2  2000/09/18 20:41:03  cvs
//  Added correct tabbing to files. Also added CVS log info to new files
//  added by Steve Parker (Averstar).
//  -- Cindy
//
//
//=============================================================================

package ROM;

import java.util.HashSet;

/** This represents the <B>UserDefinedType</B> object
 * @author <I>Steve Parker</I>
 * <a href="mailto:sparker@titan.com">sparker@titan.com</a>
 */
public class UserDefinedType extends Type {

	private static BunchValue elements;
	private Type superType;
	String name;
        private String bunchType;
 
	/** The primary Constructor
	 * @param parentIn The parent object
	 * @param nameIn Name of type
	 * @param elementsIn A bunch containing elements of this type
	 * @param superTypeIn The Type parent of this type
	 * @return Not applicable
	 */
	public UserDefinedType(BaseObject parentIn, String nameIn, 
						   BunchValue elementsIn, Type superTypeIn) {
		super(parentIn);
		name = nameIn;
		elements = elementsIn;
		superType = superTypeIn;
	        bunchType = "bunch("+ superTypeIn + ")";
	}


	/** To add an element in type
	 * @param elemIn Element to be added
	 * @return void
	 */
	public void addElement(Object elemIn) {
		elements.addAnElement(elemIn);
	}

	/** To set the whole set of elements
	 * @param elementsIn to be set
	 * @return void
	 */
	public void setElements(HashSet elementsIn) {
		elements.setElementSet(elementsIn);
	}

	/** To verify that some object has correct type
	 * @param valIn Object to be tested against type 
	 * @return boolean, true if object of correct type
	 */
	public boolean isTypeOf(Object valIn) {
		return elements.elementPresent(valIn);
	}

	/** To get the set of all possible values of type
	 * Note: It may not always be possible to implement this set (infinite)
	 * @return BunchValue containing values
	 */
	public BunchValue getElements() {
		return elements;
	}

	
	/**To get the super type
	 *@return Type
	 */
	public Type getSuperType(){
		return superType;
	}


	/** To get the string representation of the object
	 * @return String representing the string of this object
	 */
	public String toString() {
		return bunchType;
	}

}

