//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_AbstractExpression.java,v 1.15 2001/03/12 23:54:36 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_AbstractExpression.java,v $
//  Revision 1.15  2001/03/12 23:54:36  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.14  2000/08/01 22:48:44  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.13  2000/06/22 17:22:38  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.12  2000/05/18 18:03:40  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I 
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.12  2000/05/05 19:28:29  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.11  2000/04/11 03:17:51  cvs
//  The dummy M__Label and M__Value functions have been removed .
//  --srinivas
//
//  Revision 1.10  2000/04/06 19:13:17  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.9  2000/03/23 00:22:44  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.8  2000/02/10 00:37:07  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.7  2000/02/08 18:42:18  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.6  1999/11/17 18:31:54  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.5  1999/11/15 22:59:14  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/15 03:16:27  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:25  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/05 23:43:07  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:25  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:02  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.8  1999/10/01 15:23:19  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.7  1999/08/01 20:54:21  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.6  1999/07/28 01:59:34  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.5  1999/07/16 21:50:24  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.4  1999/07/14 23:23:16  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.3  1999/07/13 19:29:04  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//  Revision 1.2  1999/07/13 14:27:00  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.1  1999/06/30 22:19:49  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//
//=============================================================================
package SOM;

import SYM.SYM_UnresolvableException; // for javadoc

import util.ListEnumeration;

import java.io.PrintWriter;
import java.io.Serializable;

/** This represents the <B>AbstractExpression</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu) 
 */
public abstract class SOM_AbstractExpression 
extends SOM_BaseClass 
implements Serializable {

	/** The typelist is used for overloading resolution.... */
	private SOM_TreeNodeList typeList;

	/** The final type after overload resolution is done and unfolded */
	private SOM_TypeDeclaration resolvedType = null;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_AbstractExpression() {
		super();
	}
	
	/** The publishSOM routine for this class
	 * @return void
	 */
	public abstract void publishSOM(PrintWriter pw);

	
	/** The publishPVS routine for this class
	 * @return void
     */
	public abstract void publishPVS(PrintWriter pw);
		
	/** To set the type of this expression.
	 * @param typeListIn The SOM_TreeNodeList corresponding to all
	 * the declaration entries that match this object
	 * @return void
	 */
	protected void setTypeList(SOM_TreeNodeList typeListIn) {
		typeList = typeListIn;
	}

	/** To get the type of this Expression. 
	 * @return The SOM_TreeNodeList corresponding to the possible
	 * types of this expression
	 */
	public SOM_TreeNodeList getTypeList() {
		return typeList;
	}

	/** To set the final resolvedType of this Expression
	 * @param typeIn The SOM_TypeDeclaration corresponding to the final
	 * resolved type
	 * @return void
	 */
	protected void setResolvedType(SOM_TypeDeclaration typeIn) {
		//System.out.println("Setting following type: " +
		//				   typeIn.toString() +
		//				   " to object of class: " +
		//				   this.getClass().getName());
		resolvedType = typeIn;
	}

	/** To get the final resolvedType of this Expression
	 * Assumes that a call to setResolvedType was successful prior to a call
	 * to this function
	 * @return The SOM_TypeDeclaration corresponding to the final resolved type
	 */
	public SOM_TypeDeclaration getResolvedType() {
		if (resolvedType == null) 
			// This is here for debugging purposes - and can be removed after 
			// the parser is stable
			throw  new RuntimeException
				(getClassName() + 
				 ": Call to getResolvedType before call to setResolvedType");
		return resolvedType;
	}

	/** To check if one of the possible types for this expression is a specific
	 * type
	 * @param typeIn The SOM_TypeDeclaration which is the expected type
	 * @return boolean
	 */
	public boolean isOfType(SOM_TypeDeclaration typeIn) {
		ListEnumeration enum = getTypeList().elements();
		SOM_Declaration resultType;
		//System.out.println("In isOfType, typeIn is: " + typeIn.toString());
		while (enum.hasMoreElements()) {
			resultType = ((SOM_TreeNode)enum.nextElement()).getResultType();
			//System.out.println("In isOfType, resultType is: " 
			//+ resultType.toString());
			if (resultType instanceof SOM_TypeDeclaration) {
				if (((SOM_TypeDeclaration)resultType).isSubtypeOf(typeIn))
					return true;
			}
		}
		return false;	
	}

	/** To do the sem check. This should be overridden in each class that
	 * inherits from this class.
	 * @return void
	 */
	public abstract void semCheck();

	/** To unfold the overloading resolution algorithm. Assumes semCheck() 
	 * was called and it successfully set the typeList. 
	 * Normally this function should be called on toplevel terms when we want
	 * the top level term's final type to be resolved automatically. If we
	 * want it to resolve it to a particular type, the overloaded version 
	 * of this function should be used!
	 * This should be overridden in all non-abstract child classes
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if the type cannot be resolved
	 */
	public abstract void resolveFinalType();


	/** To unfold the overloading resolution algorithm. Assumes semCheck() 
	 * was called and it successfully set the typeList. Note that this is not
	 * visible to the external world. It will be used by this and derived
	 * classes only...
	 * NOTE: Now made visible to the external world. I realised that this
	 * function can be used to verify if the resulting type of an expression is
	 * typeIn or not! Hence it can be very useful...
	 * This should be overridden in all non-abstract child classes
	 * @param typeIn The SOM_TypeDeclaration which is expected to be the final
	 * resolved type
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if the type cannot be resolved
	 */
	public abstract void resolveFinalType(SOM_TypeDeclaration typeIn);

	/** To retreive the lowest level "names" (and their declarations) present
	 * in this expression
	 * Assumes that a prior call to resovleFinalType was successful
	 * @return 
	 */
	public abstract SOM_DeclarationList getNames();

	

}


