//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: SOM_ArrayDeclarationStatement.java,v 1.4 2001/11/02 16:19:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ArrayDeclarationStatement.java,v $
//  Revision 1.4  2001/11/02 16:19:28  cvs
//  Made changes to support facet as a type.
//    --Murthy
//
//  Revision 1.3  2001/08/28 23:27:00  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.2  2000/12/03 23:55:25  cvs
//  Added the M__Value method to the Literal classes.Previously they were returning null.The M__Type method for all the Declaration statement classes has been completed
//  --srinivas
//
//  Revision 1.1  2000/07/11 00:11:07  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//
//============================================================================
package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;
import java.util.*;

public class SOM_ArrayDeclarationStatement 
extends SOM_BaseClass {

	private SOM_Expression typeExpr = null;
	private SOM_Expression typeValue = null;
	
	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ArrayDeclarationStatement()
	{
		super();
	}

	/** To set the expression for the type
	 * @param typeExprIn The SOM_Expression representing the type
	 * @return void
	 */
	public void setType(SOM_Expression typeExprIn) {
		typeExpr = typeExprIn;
	}

	/** To get the expression for the type
	 * @return The SOM_Expression representing the type
	 */
	public SOM_Expression getType() {
		return typeExpr;
	}
 
	/** To set the expression for the values of the type
	 * @param typeValueIn The SOM_Expression for the values in type
	 * @return void
	 */
	public void setValue(SOM_Expression typeValueIn) {
		typeValue = typeValueIn;
	}

	/** To get the expression for the values of the type
	 * @return The SOM_Expression for the values in the type
	 */
	public SOM_Expression getValue() {
		return typeValue;
	}

	//Added by srinivas on Nov 29th, 2000
	/** The function is to get the Type of the array declaration
	 * @param parent
	 * @param root The Universe object to search for predefined types
	 * @param value The Expression object to add to bunchVlaue of type
	 * @return ROM.Type
	 */
	public Type M__Type(BaseObject parent,Universe root, Expression value){
		SOM_Expression expr = getType();
		Type type = null;
		if (expr != null){
			String str = expr.getResolvedType().toString();
			if (str.equals("integer")){
				type = root.getType("arrayInteger");
				if (value != null)
					((ArrayIntegerType)type).addOneElement(value);
			}
			else if (str.equals("real")){
				type = root.getType("arrayReal");
				if (value != null)
					((ArrayRealType)type).addOneElement(value);
			}else if (str.equals("bit")){
				type = root.getType("arrayBit");
				if (value != null)
					((ArrayBitType)type).addOneElement(value);
			}else if (str.equals("boolean")){
				type = root.getType("arrayBoolean");
				if (value != null)
					((ArrayBooleanType)type).addOneElement(value);
			}else{
				Type result = null;
				if (parent instanceof Facet){
					type = ((Facet)parent).getType(str);
					result =
						((Facet)parent).alreadyDeclared("array",type);
				}
				if (type == null){
					type = root.getType(str);
					result = root.alreadyDeclared("array",type);
				}
				if (result == null){
					BunchValue arrayBunch =
						new BunchValue(null, null, new HashSet());
					ArrayType array = new
						ArrayType(parent,arrayBunch,type);
					arrayBunch.setType(array);
					if (value != null)
						((ArrayType)array).addOneElement(value);
					if (parent instanceof Facet)
						((Facet)parent).addTypeElement(array);
					return (Type)array;
				}else {
					if (value != null)
						((ArrayType)result).addOneElement(value);
					return result;
				}
			}
		}
		return type;
	}
	
	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		pw.print(" array");
		pw.print("(");
		//Added by Murthy
		if (typeExpr != null) {
		typeExpr.publishSOM(pw);
		}
		pw.print(")");
	    if (typeValue != null){
			pw.print(" is");
			typeValue.publishSOM(pw);
	    }
		pw.print(";");
	}
}
