//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_BaseClass.java,v 1.6 2000/06/22 17:22:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_BaseClass.java,v $
//  Revision 1.6  2000/06/22 17:22:39  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.5  2000/03/03 18:38:27  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.4  2000/02/10 00:37:10  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:20  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.2  1999/11/05 23:43:08  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:29  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:20  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.10  1999/08/01 20:54:23  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.9  1999/07/14 23:23:29  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.8  1999/07/13 19:29:05  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//  Revision 1.7  1999/07/13 14:27:04  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.6  1999/07/01 23:27:27  rkamath
//  Now that "list" objects are no longer returned as parse-objects, the need
//  for the SOM_Unifier is nolonger extant. Hence the root of the OM is again
//  SOM_BaseClass, and there is no publishRosetta for the list.
//  Note that the list can now only store SOM_BaseClass objects and hence list
//  objects cannot be added to the parseOrderList. All such instances have a
//  wrapper derived from SOM_BaseClass (eg. SOM_IdentifierListing for
//  SOM_IdentifierList etc.)
//
//  Revision 1.5  1999/06/30 02:18:57  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//  Revision 1.4  1999/06/27 17:33:26  rkamath
//  Added some of the object model for the Expression grammar and a few changes to
//  the other files..
//
//  Revision 1.3  1999/06/25 15:10:18  rkamath
//  Modified the Makefile and changed the javadoc comments in the java files
//
//  Revision 1.2  1999/06/23 18:34:55  rkamath
//  A fresh new addition and modification of new files
//
//  Revision 1.1  1999/06/18 21:14:53  rkamath
//  The first commit for the SOM (SLDL Object Model). The DesignUnit is purely to
//  enable compilation and is yet incomplete...
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;
import java.lang.Object;
import java.util.StringTokenizer;
import java.io.*;

/** This is the <B>root</B> of the <I>Rosetta Object Model</I> Hierarchy
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
// made serializable by makarand

public abstract class SOM_BaseClass
extends Object 
implements Serializable {

	/** This is the list that will store the elements in the order they
	 *  were parsed. This is in ADDITION to the private data members
	 *  that each subclass will have. This list will hopefully help in
	 *  automating the publishRosetta function (and mebbe other stuff
	 *  too).
	 */
	private SOM_List parseOrderList = new SOM_List();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_BaseClass() {
		super();
	}

	/** To retrieve the full class name
	 * @return String containing the full class name
	 */
	public String getFullClassName() {
		return this.getClass().getName();
	}

	/** To retrieve the class name
	 * @return String containing the class name
	 */
	public String getClassName() {
		String toReturn = null;
		StringTokenizer tokenizer = 
			new StringTokenizer(this.getClass().getName(), ".");

		while (tokenizer.hasMoreTokens())
			toReturn = new String(tokenizer.nextToken());

		return toReturn;
	}

	/** To add SOM_BaseClass objects to the parseOrderList
	 * @param parsedObject The SOM_BaseClass that is to be added
	 * @return void
	 */
	public void addToParseOrderList(SOM_BaseClass parsedObject) {
	    parseOrderList.addElement(parsedObject);
	}

	/** The toString function which is used to hash objects into the
	 * Symbol Table. The classes that deal with the symbol table should
	 * override this function!
	 * @return String representation of this object
	 * @exception SOM.SOM_MethodNotSupportedException if this function
	 * is called on an object for which toString() has not be overridden
	 */
	public String toString() {
		throw new SOM_MethodNotSupportedException
			(getClassName() + " does not support toString()");
	}

	/** This is the Publish Rosetta function to publish from the object
	 * model.  This function will not be overridden in other subclasses,
	 * except for those that represent "terminal" objects. In other
	 * words this function achieves a pre-order traversal of the object
	 * model via a recursive call.
	 * @return void
	 */
	public final void publishRosetta(PrintWriter pw) {
	    ListEnumeration enum = parseOrderList.elements();
	    while (enum.hasMoreElements()) {
			SOM_BaseClass currElem = (SOM_BaseClass)enum.nextElement();
			if (currElem instanceof SOM_Terminal) {
				((SOM_Terminal)currElem).publishRosettaTerminal(pw);
			}
			else currElem.publishRosetta(pw);
		}
	}
}
