//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Declaration.java,v 1.15 2001/08/28 23:27:00 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Declaration.java,v $
//  Revision 1.15  2001/08/28 23:27:00  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.14  2001/07/03 14:53:57  cvs
//
//  Initial implementation of enumerations.
//  --Morel
//
//  Revision 1.13  2001/03/12 23:54:37  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.12  2000/07/24 23:41:01  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.11  2000/07/11 00:11:08  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.10  2000/06/22 17:22:39  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.9  2000/05/18 18:03:42  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.9  2000/05/05 19:28:30  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.8  2000/04/28 19:34:04  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.7  2000/04/06 19:13:18  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.6  2000/02/13 01:15:53  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.5  2000/02/10 00:37:12  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  1999/11/15 03:16:27  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:25  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/11 00:04:20  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//  Revision 1.1  1999/10/27 20:43:30  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:26  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.11  1999/10/01 15:23:21  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.10  1999/08/01 20:54:25  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.9  1999/07/28 01:59:35  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.8  1999/07/21 00:28:11  rkamath
//  Small changes and consistency corrections...
//
//  Revision 1.7  1999/07/16 21:50:25  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.6  1999/07/14 23:23:31  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.5  1999/07/13 19:29:06  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//  Revision 1.4  1999/07/13 14:27:08  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.3  1999/07/10 21:31:21  rkamath
//  The commit for today. Introduced a toString() function in
//  SOM_Declaration that will be used to hash "declarations" into the
//  symboltable 
//
//  Revision 1.2  1999/07/09 22:21:07  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//  Revision 1.1  1999/07/06 21:01:33  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//
//=============================================================================
package SOM;

import SYM.SYM_Symboltable;

import java.io.Serializable;
import java.lang.IllegalArgumentException;
import java.lang.IllegalStateException;
import java.util.NoSuchElementException;

/** This represents the <B>Declaration</B>
 * Every declaration should have a Type!!
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public abstract class SOM_Declaration
extends SOM_BaseClass 
implements Serializable {

	// Whenever you add a constant here, update the setKind(), 
	// getStringForKind(), and the isDistinguishable() function etc
	public static final int UNDEFINED = 1;

	public static final int TYPE = 11;

	public static final int VARIABLE = 21;

	public static final int FACET = 31;
	public static final int COMPONENT = 32;
	public static final int FUNCTION = 33;
	public static final int PACKAGE = 35;  // added by krishna on Feb 12, 2000
        public static final int INTERACTION = 36;
       	public static final int DOMAIN = 37;
	
	public static final int LABEL = 41;

	public static final int ARRAY = 51;
	public static final int RECORD = 52;
	public static final int TUPLE = 53;
	public static final int SET = 54;
	public static final int SEQUENCE = 55;

    // Added by Morel, June 30, 2001
    public static final int ENUMERATION = 56;
    public static final int ENUMERATIONITEM = 57;
	
	// The kind of this declaration
	private int kind = UNDEFINED;

	// Since each Declaration must have a type....
	private SOM_TypeDeclaration myType;
	private SOM_Name typeName;

	// The symboltable in which this declaration is entered.
	private SYM_Symboltable parentSymboltable;

	// Some declarations like TYPE, FACET etc may need their own symbol tables
	//protected transient SYM_Symboltable mySymboltable = null;
	protected SYM_Symboltable mySymboltable = null; // changed it on 080299
	protected boolean hasMySymboltable = false;


	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_Declaration() {
		super();
	}

	/** To get the kind
	 * @return Integer One of the public static final constants defined
	 * in SOM_Declaration which is the type of this object
	 */
	public int getKind() {
		return kind;
	}

	/** To set the kind of the declaration. Update this function as and when 
	 * new constants are added. But is this function really necessary?? Since
	 * any derived class can set its own kind, this function is kindof useless.
	 * So I'm making it a protected one.
	 * @param kindIn The integer to be set as the type constant
	 * @return void
	 * @exception IllegalArgumentException When kindIn is not one from
	 * among the public static final constants defined in
	 * SOM_Declaration
	 */
	protected void setKind(int kindIn) {
		switch(kindIn) {
		case UNDEFINED:
		case TYPE:
		case VARIABLE:
		case FACET:
		case COMPONENT:
		case FUNCTION:
		case PACKAGE:
		case LABEL:
		case ARRAY:
		case RECORD:

		    // Added by Morel, June 29, 2001
		case ENUMERATION:
		case ENUMERATIONITEM:
		    
		case TUPLE:
		case SET:
		case SEQUENCE:
	        case INTERACTION:
		case DOMAIN:
			kind = kindIn;
			break;
		
		default:
			throw new IllegalArgumentException
				(getClassName() + ": Illegal constant " + kindIn + 
				 " passed to setKind(int). Potentially the function "
				 + "has not been updated.");
		}
	}

	/** To get the string corresponding to the kind
	 * @param kindIn The integer representing the kind for which the string
	 * is to be generated
	 * @return String corresponding to kindIn
	 */
	public static String getStringForKind(int kindIn) {
		switch(kindIn) {
		case UNDEFINED:
			throw new RuntimeException
				("Kind undefined. No string representation possible");

		case TYPE:
			return new String("type");

		case VARIABLE:
			return new String("variable");

		case FACET:
			return new String("facet");

		case COMPONENT:
			return new String("component");

		case FUNCTION:
			return new String("function");

		// added by krishna on Feb 12, 2000
		case PACKAGE:
			return new String("package");

		case LABEL:
			return new String("label");

		case ARRAY:
			return new String("array");

		case RECORD:
			return new String("record");
			
			// Added by Morel, June 30, 2001
		case ENUMERATION:
		    return new String("enumeration");

		case ENUMERATIONITEM:
		    return new String("enumerationitem");

		case TUPLE:
			return new String("tuple");

		case SET:
			return new String("set");

		case SEQUENCE:
			return new String("sequence");

		case INTERACTION:
			return new String("interaction");

		default:
			throw new IllegalArgumentException
				("SOM_Declaration: Illegal constant " + kindIn + 
				 " passed to getStringForKind(int). Potentially " +
				 "the function has not been updated.");
		}
	}

	/** To check if the current declaration is of the required kind
	 * @param kindIn The kind that is being queried for. It is one of
	 * the public static final ints defined here
	 * @return true if the kind of this declaration is the same as kindIn
	 */
	public boolean isKind(int kindIn) {
		return kind == kindIn;
	}


	/** To set the Type for this declaration...refer the reflection semantics
	 * @param typeIn The SOM_TypeDeclaration to be set at myType
	 * @return void
	 */
	public void setType(SOM_TypeDeclaration typeIn) {
		myType = typeIn;
	}	

	/** To get the Type for this declaration...refer the reflection semantics
	 * @return The SOM_TypeDeclaration that is myType
	 */
	public SOM_TypeDeclaration getType() {
		/* if (myType.toString().equalsIgnoreCase("bunchMetaType"))
			 If type is bunchMetaType, then the declaration itself 
			 is the type 
			 return (SOM_TypeDeclaration)this; */
		// If myType is a type declaration, then return it
		if (this instanceof SOM_TypeDeclaration)
			return (SOM_TypeDeclaration)this;
		return myType;
	}	

	/** To set the typeName for this declaration
	 * @param typeNameIn The SOM_Name to be set as the typeName
	 * @return void
	 */
	public void setTypeName(SOM_Name typeNameIn) {
		typeName = typeName;
	}

	/** To get the typeName for this declaration
	 * @return  The SOM_Name that is the typeName
	 */
	public SOM_Name getTypeName() {
		return typeName;
	}

	/** To set the parentSymboltable. This is done in the symboltable just 
	 * before adding the declaration into it.
	 * @param tableIn The SYM_Symboltable to be set as this decls parent
	 * @return void
	 */
	public void setParentSymboltable(SYM_Symboltable tableIn) {
		parentSymboltable = tableIn;
	}

	/** To get the parentSymboltable
	 * @return The SYM_Symboltable which is its parent
	 */
	public SYM_Symboltable getParentSymboltable() {
		return parentSymboltable;
	}

	/** To set mySymboltable
	 * @param tableIn The SYM_Symboltable to be set as this obj's symboltable
	 * @return void
	 */
	public void setMySymboltable(SYM_Symboltable tableIn) {
		mySymboltable = tableIn;
		hasMySymboltable = true;
	}

	/** To get the symboltable. 
	 * @return The SYM_Symboltable of this object
	 */
	public SYM_Symboltable getMySymboltable() {
		if (!hasMySymboltable)
			throw new NoSuchElementException
				(getClassName() + 
				 ": call to getSymboltable() before setSymboltable()");

		// if mySymboltableName is not null, then mySymboltable exists either
		// in memory or disk. Note that we don't check here if the
		// mySymboltable has already been deserialized. That should be
		// done in the environment 
		// I decided that this should not be transient on 080299 and so it will
		// always be available
		return mySymboltable;
	}

	/** To ascertain if two declarations are distinguishable or not. This is
	 * necessary in determining redeclarations...
	 * @param declOne The first SOM_Declaration
	 * @param declTwo The second SOM_Declaration to be compared to declOne
	 * @return true if they are distinguishable, false otherwise
	 */
	public static boolean isDistinguishable(SOM_Declaration declOne,
											SOM_Declaration declTwo) { 

		// If the string reps differ then they ARE different decls
		if (!declOne.toString().equals(declTwo.toString())) return true;

		// If the kinds differ they ARE different
		if (declOne.kind != declTwo.kind) return true;

		switch (declOne.kind) {

		case UNDEFINED:
			throw new IllegalStateException
				("SOM_Declaration: Incorrectly initialized declarations" + 
				 "passed to isDistinguishable()");

		case TYPE:
			return SOM_TypeDeclaration.isDistinguishable
				((SOM_TypeDeclaration)declOne, 
				 (SOM_TypeDeclaration)declTwo);

		case VARIABLE:
			return SOM_VariableDeclaration.isDistinguishable
				((SOM_VariableDeclaration)declOne,
				 (SOM_VariableDeclaration)declTwo); 

		case FACET:
			return declOne != declTwo;

		case COMPONENT:
			return declOne != declTwo;

		// added by krishna on Feb 12, 2000
		case PACKAGE:
			return declOne != declTwo;

                case INTERACTION:
			return declOne != declTwo;

		case FUNCTION:
			return SOM_FunctionDeclaration.isDistinguishable
				((SOM_FunctionDeclaration)declOne, 
				 (SOM_FunctionDeclaration)declTwo);

		case LABEL:
			// This function is called only after ascertaining that a prior
			// declaration for a label didn't exist. A call to this will be
			// generated when adding a label with multiple types..
			return true; // as dictated by processLabel() in the parser

			//case ARRAY:			
			//return SOM_ArrayDeclaration.isDistinguishable
			//	((SOM_ArrayDeclaration)declOne, 
			//	 (SOM_ArrayDeclaration)declTwo);

		case SET:
			return SOM_SetDeclaration.isDistinguishable
				((SOM_SetDeclaration)declOne,
				 (SOM_SetDeclaration)declTwo);

			//case SEQUENCE:
			//return SOM_SequenceDeclaration.isDistinguishable
			//	((SOM_SequenceDeclaration)declOne,
			//	 (SOM_SequenceDeclaration)declTwo);

		default:
			throw new IllegalArgumentException("SOM_Declaration: Illegal value for declOne passed to isDistinguishable().  Potentially the function has not been updated.");
		}
	}

	/** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
	 * @param declLHS The decl corresponding to the lhs object
	 * @param declRHS The decl corresponding to the rhs object
	 * @return true, if the rhs is assignable to the lhs, false otherwise
	 */
	public static boolean
		isLeftTypeCompatibleWithRightType(SOM_TypeDeclaration typeLHS,
										  SOM_TypeDeclaration typeRHS)
	{ 

		// The binding is valid if the type of declLHS is a pred
		// supertype of the type of declRHS - re. type checking algo
		// for rosetta 
		// The binding is compatible if typeRHS is a predicate subtype
		// of typeLHS 
		// re. rosetta type checking algo..
		
		// WHAT OF METATYPES (bunch, facet, component, function) 
		// That stuff has to be coded
		if (typeLHS instanceof SOM_SetType && typeRHS instanceof SOM_SetType)
			return SOM_SetType.isLeftTypeCompatibleWithRightType
				((SOM_SetType)typeLHS, (SOM_SetType)typeRHS);

		if (typeLHS instanceof SOM_ArrayType 
			&& typeRHS instanceof SOM_ArrayType)
			return SOM_ArrayType.isLeftTypeCompatibleWithRightType
				((SOM_ArrayType)typeLHS, (SOM_ArrayType)typeRHS);

		if (typeLHS instanceof SOM_RecordType 
			&& typeRHS instanceof SOM_RecordType)
			return SOM_RecordType.isLeftTypeCompatibleWithRightType
				((SOM_RecordType)typeLHS, (SOM_RecordType)typeRHS);

		// Added by Morel, June 29, 2001
		if (typeLHS instanceof SOM_EnumerationType 
		    && typeRHS instanceof SOM_EnumerationType)
		    return SOM_EnumerationType.isLeftTypeCompatibleWithRightType
			((SOM_EnumerationType)typeLHS, (SOM_EnumerationType)typeRHS);

		return typeRHS.isPredicateSubtypeOf(typeLHS);
	}

	/** To help in de-Serializing this object
	 * @return void
	 */
	/* no longer needed after making mySymboltable nontransient
	private void readObject(ObjectInputStream in)
		throws IOException, ClassNotFoundException {
			in.defaultReadObject();
			//mySymboltable = null;
			// this will be deserialized if needed by the environment
	}
	*/
}
