//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_DesignFile.java,v 1.10 2001/03/26 19:04:58 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_DesignFile.java,v $
//  Revision 1.10  2001/03/26 19:04:58  cvs
//  Completed the implementation for the change in export clauses. Also
//  added code in SOM_PackageDeclaration to take care of use clauses (case
//  was missing for when these clauses are found in declaration part of
//  packages).
//  -- Cindy
//
//  Revision 1.9  2001/03/21 00:05:05  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.8  2001/02/08 22:58:52  cvs
//  Modified the classes to remove some errors while building the
//  Rosetta Object Model. 
//  --srinivas
//
//  Revision 1.7  2001/02/01 22:34:30  cvs
//  Added some accessor methods to the classes which are used in DVTG
//  --srinivas
//
//  Revision 1.6  2000/12/04 19:37:06  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  -- Cindy
//
//  Revision 1.5  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.4  2000/03/03 18:38:27  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.3  2000/02/10 00:37:13  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:21  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes 
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.1  1999/10/27 20:43:30  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:30  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.4  1999/06/30 02:18:58  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//  Revision 1.3  1999/06/25 15:10:19  rkamath
//  Modified the Makefile and changed the javadoc comments in the java files
//
//  Revision 1.2  1999/06/23 18:34:56  rkamath
//  A fresh new addition and modification of new files
//
//  Revision 1.1  1999/06/18 21:14:56  rkamath
//  The first commit for the SOM (SLDL Object Model). The DesignUnit
//  is purely to enable compilation and is yet incomplete...
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import java.io.*;
import ROM.*;

/** This represents the <B>DesignFile</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */

/** serializable change made by makarand */

public class SOM_DesignFile
extends SOM_BaseClass implements Serializable {

    private SOM_DesignUnitList designUnitList = new SOM_DesignUnitList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_DesignFile() {
		super();
		designUnitList = new SOM_DesignUnitList();
	}

	/** Add a SOM_DesignUnit object
	 * @param dUnit The SOM_DesignUnit to be added
	 * @return void
	 */
	public void addDesignUnit(SOM_DesignUnit dUnit) {
		designUnitList.addElement(dUnit);		
	}

	/** To retrieve the designUnitList
	 * @return The designUnitList
	 */
	public SOM_DesignUnitList getDesignUnitList() {
		return designUnitList;
	}

	/** To retrieve the listEnumeration of the designUnitList
	 * @return The ListEnumeration corresponding to the designUnitList
	 */
	public ListEnumeration designUnits() {
		return designUnitList.elements();
	}

	/** To build up the Rosetta object model - this object model
	 *	follows the semantics of the Semantic guide
	 * @return rootROM, The Universe root of the object model
	 */
	public Universe buildROM() {
		Factory theFactory = new Factory();
		Universe root = theFactory.newUniverse();
		Object desUnit;
		SOM_PackageDeclaration packUnit;
		LabelledItem packageItem = null;
		ItemReferenceList useRefList = new ItemReferenceList();

		ListEnumeration units = this.designUnits();
		
		while (units.hasMoreElements()) {
			desUnit = units.nextElement();
			if (desUnit instanceof SOM_UseClause) {
				ListEnumeration useList = 
					((SOM_UseClause)desUnit).selectedNames();
				SOM_SelectedName useName;
				FacetReference useRef;
							
				while (useList.hasMoreElements()) {
					useName = (SOM_SelectedName)useList.nextElement();
					try {
						// Deserialization of corresponding package ROM
						FileInputStream in = new FileInputStream
							(useName.toString() + "_ROM.ser");
						ObjectInputStream objin = new ObjectInputStream(in);
						System.out.println(" Deserializing package ROM: " +
										   useName.toString());
						packageItem = (LabelledItem)objin.readObject();
						root.addItem(packageItem);
						useRef = new FacetReference(root, packageItem);
						useRefList.add(useRef);
						System.out.println(" ... Done");
						objin.close();
					} catch(Exception e){
						System.out.println(e + " - file" +
										   useName.toString() + "_ROM.ser");
					}
				}
			} else {			
				if (desUnit instanceof SOM_PackageDeclaration) {
					packUnit = (SOM_PackageDeclaration)desUnit;
					
					if (useRefList.size() != 0) {
						packageItem = packUnit.buildROM(theFactory, root, 
														root, useRefList); 
					} else {
						packageItem = packUnit.buildROM(theFactory, root,
														root);
					}
					root.addItem(packageItem);
					try {
						// Serialization of ROM
						System.out.println("Begin Serialization ROM: "
										   + packUnit.toString());
						FileOutputStream fileoutROM = 
							new FileOutputStream(packUnit.toString() + 
												 "_ROM.ser");
						ObjectOutputStream outROM = 
							new ObjectOutputStream(fileoutROM);
						outROM.writeObject(packageItem);
						outROM.flush();
						outROM.close();
						System.out.println
							("Package ROM Object model serialized");
					} catch (FileNotFoundException e) {
						System.err.println("Serious Error: File " + 
										   packUnit.toString() + "_ROM.ser" +
										   " is not available\n");
					} catch (IOException except) {   // added by ckong
						System.err.println(except);
					}
				}
			}
			
		}
		return root;		
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		ListEnumeration listEnum = this.designUnits();
		SOM_DesignUnit dUnit;
		while (listEnum.hasMoreElements()) {
			dUnit = (SOM_DesignUnit)listEnum.nextElement();
			dUnit.publishSOM(pw);
		}
	}
}

