//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Murthy Kakarlamudi,ksatya@ittc.ku.edu
//
//  $Id: SOM_DomainDeclaration.java $
//
//  Revision History:
//  ----------------
//  $Log: SOM_DomainDeclaration.java,v $
package SOM;
import util.List;
import util.Link;
import util.ListEnumeration;

import java.io.*;
import java.lang.NullPointerException;
import java.io.Serializable;
import java.lang.String;

import ROM.*;

/** This represents the <B>DomainDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
    // made serializable by makarand

public class SOM_DomainDeclaration
extends SOM_Declaration
implements SOM_DomainDeclarativeItem, SOM_PackageDeclarativeItem, Serializable {
    
    /** The name of this Domain - and other dataMembers */
    private SOM_Identifier id;
    private SOM_InterfaceListing interfaceList = null;
    private SOM_DomainDefinition domainDefinition = null;
    
    private transient SOM_DeclarationList domainList;
    
    /** The primary constructor
     * @param domainTypeIn The typeDeclaration corresponding to the "domain type"
     * @return Not applicable
     */
    public SOM_DomainDeclaration(SOM_TypeDeclaration domainTypeIn) {
		super();
		setKind(DOMAIN);
		setType(domainTypeIn);
    }
    
    /** To set the identifier
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
    }			 
    
    /** To get the identifier
     * @return The SOM_Identifier for this designUnit
     */
    public SOM_Identifier getIdentifier() {
		return id;
    }			 
    
    /** To set the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interfaceList = interfaceIn;
    }
    
    /** To get the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
		return interfaceList;
    }
    
    /** To set the domainList - implements the function from
     * SOM_DesignUnit
     * @param domainListIn The SOM_DeclarationList object to be set
     * @return void
     */
    public void setDomainList(SOM_DeclarationList domainListIn) {
		domainList = domainListIn;
    }
    
    /** To get the domainList - implements the function from
     * SOM_DesignUnit
     * @return The SOM_DeclarationList object for this designUnit
     */
    public SOM_DeclarationList getDomainList() {
		return domainList;
    }
    
    /** To set the domainDefinition
     * @param domainDefIn The SOM_DomainDefinition object to be set
     * @return void
     */
    public void setDomainDefinition(SOM_DomainDefinition domainDefIn){
		domainDefinition = domainDefIn;
    }
    
    /** To get the domainDefinition
     * @return The SOM_DomainDefinition object to get
     */
    public SOM_DomainDefinition getDomainDefinition(){
		return domainDefinition;
    }
    
    /** added by krishna on Mar 5, 2000
     * To get the desired object by traversal of SOM
     */
    public ListEnumeration M__Terms() {
		List terms = new List();
		SOM_DomainBody domainBody = getDomainDefinition().getDomainBody();
		if(domainBody != null){
			ListEnumeration items = domainBody.items();
			if (items !=null){
				while (items.hasMoreElements()){
					SOM_DomainBodyItem domainItem =
						(SOM_DomainBodyItem)items.nextElement();
					if (domainItem !=null){
						terms.addElement
							(((Object)domainItem.getLabelledExpression()));
					}
				}
			}
		}
		if (terms.isEmpty())
			return null;
		else
			return terms.elements();
    }
    
    /** Added by srinivas on march 28 2000
     *To get all the parameters that are passed to the domain.It
	 * returns objects of type SOM_InterfaceItem
	 * @param void
	 * @return ListEnumeration
	 */
    public ListEnumeration M__Params(){
		List parameters = new List();
		if (getInterfaceListing() !=null){
	    ListEnumeration  interListEnum =
			getInterfaceListing().interfaces();
	    
	    SOM_InterfaceItem interItem;
	    if (interListEnum != null){
			while (interListEnum.hasMoreElements()){
				interItem = (SOM_InterfaceItem)
					interListEnum.nextElement();
		    if(interItem !=null)
				parameters.addElement((Object)interItem);
			}
	    }
		}
		if (parameters.isEmpty()){
			return null;
		}
		else{
			return parameters.elements();
		}
    }
    
    /**Added by srinivas on march 28 2000
     *To get all the physical variables that are defined in
     * the domain
	 * param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Pvars(){
		ListEnumeration domainDeclItemEnum;
		List pVars = new List();
		ListEnumeration idListEnum;
		SOM_DomainDefinition domainDef = getDomainDefinition();
		if (domainDef.getDomainDeclarativeBlock() != null){
			domainDeclItemEnum =
				domainDef.getDomainDeclarativeBlock().items();
			if (domainDeclItemEnum !=null){
				while (domainDeclItemEnum.hasMoreElements()){
					SOM_DomainDeclarativeItem domainDeclItem =
						((SOM_DomainDeclarativeItem)domainDeclItemEnum.
						 nextElement());
					
					if (domainDeclItem instanceof
						SOM_DomainDeclaration){
						if (((SOM_DomainDeclaration)domainDeclItem)
							!=null) 
							pVars.addElement(((Object)
											  ((SOM_DomainDeclaration)
											   domainDeclItem)
											  .getIdentifier()));
						
					} else if (domainDeclItem instanceof
							   SOM_DeclarationStatement){
						pVars.addElement(((Object)
										  ((SOM_DeclarationStatement)
										   domainDeclItem)
										  .getIdentifier()));
						if (((SOM_DeclarationStatement)domainDeclItem)
							.getIdentifierListing() !=null){
							idListEnum = ((SOM_DeclarationStatement)
										  domainDeclItem).
								getIdentifierListing().identifiers();
							while(idListEnum.hasMoreElements()){
								pVars.addElement((Object)idListEnum.
												 nextElement());
							}
						}
					}
				}
			}
		}
		if (pVars.isEmpty()){
			return null;
		} else
			return pVars.elements();
    }
	
    /**Added by srinivas on march 28 2000
	 *To get all the types that are defined in the entire domain
	 * @param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Types(){   
        List types = new List();
		ListEnumeration declStmts = getDeclarationStatements();
		if (declStmts !=null){
			while (declStmts.hasMoreElements()){
				Object type =
					((SOM_DeclarationStatement)declStmts.nextElement()).
					M__Type();
				if (type !=null)
					types.addElement(type);
			}
		}
		if (getInterfaceListing() !=null){
			ListEnumeration  interListEnum =
				getInterfaceListing().interfaces();
			if (interListEnum != null){
				while (interListEnum.hasMoreElements()){
					String nametype =
						((SOM_TypeDeclaration)
						 ((SOM_InterfaceItem)interListEnum.nextElement()).
						 getType()).toString();
					if (nametype !=null)
						types.addElement(nametype);
				}
			}
		}
		if (types.isEmpty())
			return null;
		else
			return types.elements();
    }
    
    
    /**Added by srinivas on march 28 2000
     *To get all the domain of the domain
	 * @param void
	 * @return String
     */
    public String  M__Domain(){
        SOM_DomainDefinition domainDefn = getDomainDefinition();
		try{
			if (domainDefn !=null){
				String str = new String();
				StringWriter sw = new StringWriter();
				PrintWriter pw = new PrintWriter(sw,true);
				if (domainDefn.getDomainExpression() != null)
					domainDefn.getDomainExpression().publishSOM(pw);
				pw.flush();
				pw.close();
				str = sw.toString();
				sw.close(); 
				return str;
			}else return null;
		}catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }	
    
	/**Added by srinivas on march 28 2000
	 *To get all the visible labels that are defined in the
	 * entire domain
	 * @param void
	 * return ListEnumeration
	 */
    public ListEnumeration M__Visible(){
		List visible = new List();
		ListEnumeration pvars = M__Pvars();
		ListEnumeration params = M__Params();
		if (pvars !=null){
		    while (pvars.hasMoreElements())
				visible.addElement(((Object)pvars.nextElement()));
		}
		if (params != null){
		    while (params.hasMoreElements())
				visible.addElement(((Object)params.nextElement()));
		}
		SOM_DomainBody domainBody = getDomainDefinition().getDomainBody();
		if (domainBody !=null){
		    ListEnumeration domainBodyItemEnum = domainBody.items();
		    while (domainBodyItemEnum.hasMoreElements()){
				SOM_DomainBodyItem domainBodyItem =
					((SOM_DomainBodyItem)domainBodyItemEnum.nextElement());
				SOM_LabelledExpression labelledExpr = 
					domainBodyItem.getLabelledExpression();
				SOM_Label label= labelledExpr.getLabel();
				if (label !=null) {
					visible.addElement((Object)label.getIdentifier());    
				}
		    }
		}
		
		if (visible.isEmpty())
		    return null;
		else
		    return visible.elements();
    }
    
    /** Added by srinivas on march 28 2000
     * To get all the items that are defined in the domain body
	 * @param void
	 * @return ListEnumeration
     */
    public ListEnumeration M__Items(){
		List items = new List();
		ListEnumeration params = M__Params();
		ListEnumeration pvars = M__Pvars();
		ListEnumeration terms = M__Terms();
		ListEnumeration types = M__Types();
	    String domain = M__Domain();
		ListEnumeration visibles = M__Visible();
		
		if (pvars!=null){
			while (pvars.hasMoreElements())
				items.addElement(pvars.nextElement());
		}
		if (params !=null){
			while (params.hasMoreElements())
				items.addElement(params.nextElement());
		}
		if (terms !=null){
			while (terms.hasMoreElements())
				items.addElement(terms.nextElement());
		}
		if (types !=null){
			while (types.hasMoreElements())
				items.addElement(types.nextElement());
		}
		if (domain !=null){
			items.addElement(domain);
		}
		if (visibles!=null){
			while (visibles.hasMoreElements())
				items.addElement(visibles.nextElement());
		}
		if (items.isEmpty())
		    return null;
		else
		    return items.elements();
    }
    
    /** Added by srinivas on march 28 2000
     * To get all the declaration statements that are declared
     * in the domain
	 * @param void
	 * @return ListEnumeration
     */
    public ListEnumeration getDeclarationStatements(){
		SOM_DomainDeclarativeBlock declBlock =
			domainDefinition.getDomainDeclarativeBlock();
		List declStmts = new List();
		if (declBlock != null){
			ListEnumeration domainDeclStmts = declBlock.items();
			if (domainDeclStmts !=null){
				while(domainDeclStmts.hasMoreElements()){
					SOM_DomainDeclarativeItem declItem =
						(SOM_DomainDeclarativeItem)domainDeclStmts.nextElement();
					if (declItem instanceof SOM_DeclarationStatement){
						SOM_DeclarationStatement declStmt =
							((SOM_DeclarationStatement)declItem);
						declStmts.addElement(declStmt);
					}
				}
			}
		}
		if (declStmts.isEmpty())
			return null;
		else
			return declStmts.elements();
    }

    /** To get the exports as a ListEnumeration 
     * @return ListEnumeration corresponding to the export clauses
     */
    public ListEnumeration exports() {
		SOM_ExportBlock exportBlock = domainDefinition.getExportBlock();
		if (exportBlock != null) 
			return exportBlock.exports();
		else
			return null;
    }
        
    /** To get an enumeration of the parents' Declarations for this domain
     * @return ListEnumeration representing the list of the parents' 
     * declarations for this domain
     */
    public ListEnumeration parents() {
		try {
			SOM_DeclarationList parentList =
				domainDefinition.getParentDeclarationList(); 
			return parentList.elements();
		} catch (NullPointerException except) {
			return new ListEnumeration(null);
		}
    }
    
    /** Added by srinivas on june 26 2000
     * To get the label of a domain item
     * in the domain
     */  
    public SOM_Identifier M__Label(){
		return getIdentifier();
    }
    /** Added by srinivas on june 26 2000
     * To get the type of a domain item
     * in the domain
     */  
    public String M__Type(){
		return getType().toString();
    }
    /** Added by srinivas on june 26 2000
     * To get the String of a domain item
     * in the domain
     */  
    public String M__String(){
		try{
			String str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			publishSOM(pw);
			pw.flush();
			pw.close();
			str = sw.toString();
			sw.close(); 
			return str;
		}
		catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }
    /** Added by srinivas on june 26 2000
     * To get the value of a domain item
     * in the domain
     */  
    public SOM_DomainDeclaration M__Value(){
		return this;
    }
	
    
    /** To get the string representation of this object. Overriding the
     * method in SOM_Declaration 
     * @return String The "name" of the domain corresponding to this
     * declaration 
     */
    public String toString() {
		return id.toString();
    }
    
	// Added by srinivas on Oct 27th, 2000. This function returns the
	// ROM.Expression 
	/** The function that creates the ROM.Expression for the domain 
	 * item
	 * @param Factory,BaseObject,Universe
	 * @return ROM.Expression(ROM.Domain)
	 */
	public Expression M__Value(Factory romFactory, BaseObject
							   parent, Universe root){

		Domain domainValue = (Domain)parent;
				
		// ***** Populating the domain value ***** //
		
		// Populating the interface items
		ListEnumeration paramList = M__Params();
		SOM_InterfaceItem interItem;
		if (paramList != null) {
			while (paramList.hasMoreElements()) {
				interItem = (SOM_InterfaceItem)paramList.nextElement();
				domainValue.addInterfaceItem
					(interItem.buildROM(romFactory, root, domainValue));
			}
		}

		// Populating the declared items
		ListEnumeration declList = getDeclarationStatements();
		SOM_DomainDeclarativeItem domainDeclItem;
		// It is being assumed that a domain will always have some
		// declared items
		if (declList != null){
			while (declList.hasMoreElements()) {
				domainDeclItem = 
					(SOM_DomainDeclarativeItem)declList.nextElement();
				if (domainDeclItem instanceof SOM_DomainDeclaration)
					domainValue.addItem
						(((SOM_DomainDeclaration)domainDeclItem).
						 buildROM(romFactory, root, domainValue));
				

				if (domainDeclItem instanceof SOM_DeclarationStatement) {
					domainValue.addItem
						(((SOM_DeclarationStatement)domainDeclItem).
						 buildROM(romFactory, root, domainValue));
				}
			}

			// To be completed
			//if (facetDeclItem instanceof SOM_UseClause)
		}

		// Populating the domain
		Label domainLabel = romFactory.newLabel
			((Item)domainValue.getParent(), M__Domain()); 
		domainValue.setDomain(domainLabel);

		// Populating the export list
		ListEnumeration exportList = exports();
	    SOM_Identifier exportId;
		if (exportList != null) {
			if (!exportList.hasMoreElements()) {
				domainValue.addExportLabel
					(romFactory.newLabel((Item)domainValue.getParent(),
										 "ALL"));
			} else {										  
				while (exportList.hasMoreElements()) {
					exportId = (SOM_Identifier)exportList.nextElement();
					domainValue.addExportLabel
						(romFactory.newLabel
						 ((Item)domainValue.getParent(), exportId.toString()));
				}
			}
		}
		ListEnumeration terms = M__Terms();
		if (terms != null){
			while(terms.hasMoreElements()){
				SOM_LabelledExpression labelExpr =
					((SOM_LabelledExpression)terms.nextElement());
				if (labelExpr != null)
					domainValue.addTerm
						(labelExpr.buildROM(romFactory, root, domainValue));
			}
		}
		// To be completed
		// Populating the use clause list

		return (Expression)domainValue;
	}

   
	// Added by srinivas on Oct22nd, 2000
	/** The buildROM method for this high-level item
	 * @param ROMFactory, Factory object used to create objects in ROM
	 * @return LabelledItem, corresponding to this domain  
	 */
	public LabelledItem buildROM(Factory romFactory, 
								 Universe root, BaseObject parentIn) {
		LabelledItem domainItem = romFactory.newLabelledItem
			(parentIn, null, null, null, M__String());
		Label domainLabel = romFactory.newLabel(domainItem, toString());
		domainItem.setLabel(domainLabel);
		// Creating the value first
		ROM.Domain domainValue = romFactory.newDomain(domainItem);
		((DomainType)root.getType("domain")).addOneElement(domainValue);
		domainItem.setValue(domainValue);
		// The object domainValue is being modified directly but not created
		M__Value(romFactory,(BaseObject)domainValue, root);
		Type domainType = root.getType("domain");
		domainItem.setType(domainType);

		return domainItem;
	}


    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		pw.print("domain ");
		id.publishSOM(pw);
		if (interfaceList != null) {
			interfaceList.publishSOM(pw);
		}
		pw.println(" is ");
		domainDefinition.publishSOM(pw);
		id.publishSOM(pw);
		pw.print(";\n");
		pw.println();
		
    }
}




