package SOM;

import util.ListEnumeration;

/** This represents the <B>EnumerationDeclaration</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */
public class SOM_EnumerationDeclaration
    extends SOM_Declaration {

    protected SOM_Identifier enumerationName;
    // to save the fields of the enumeration
    protected SOM_DeclarationList fieldList = new SOM_DeclarationList();

    /** The primary constructor
     * @param enumerationNameIn The SOM_Identifier for the name of the declaration
     * @param myType The SOM_TypeDeclaration corresponding to the
     * enumerationMetaType
     * @return Not applicable
     */
    public SOM_EnumerationDeclaration(SOM_Identifier enumerationNameIn,
				      SOM_TypeDeclaration myType) {
	super();
	setKind(SOM_Declaration.ENUMERATION);
	enumerationName = enumerationNameIn;
	setType(myType);
    }

    /** To get the string representation of this object. Overrides the
     * method in SOM_Declaration
     * @return String representation of this object
     */
    public String toString() {
	return enumerationName.toString();
    }

    /** To add to the fieldList
     * @param fieldIn The SOM_EnumerationItemDeclaration corresponding to (one of)
     * the fields of this enumeration
     * @return void
     */
    public void addToFieldList(SOM_EnumerationItemDeclaration fieldIn) {
	fieldList.addElement(fieldIn);
    }

    /** To get the fieldList
     * @return The SOM_DeclarationList of the domains for this enumeration
     */
    public SOM_DeclarationList getFieldList() {
	return fieldList;
    }

    /** To implement my own version of isDistinguishable for types
     * @param declOne The first SOM_EnumerationDeclaration
     * @param declTwo The next SOM_EnumerationDeclaration to be distinguished
     * against declOne
     * @return true if they are distinguishable, false otherwise
     */
    public static boolean isDistinguishable
	(SOM_EnumerationDeclaration declOne, 
	 SOM_EnumerationDeclaration declTwo) { 
	SOM_DeclarationList fieldListOne = declOne.getFieldList();
	SOM_DeclarationList fieldListTwo = declTwo.getFieldList();
	
	SOM_EnumerationType typeOne = (SOM_EnumerationType)declOne.getType();
	SOM_EnumerationType typeTwo = (SOM_EnumerationType)declTwo.getType();
	if (!declOne.toString().equalsIgnoreCase(declTwo.toString()))
	    return true;
	else
	    return(SOM_EnumerationType.isDistinguishable(typeOne, typeTwo));
    }

    /** To ascertain "compatiblity" for bindings of the kind lhs <- rhs
     * => RHS must be a subtype of LHS
     * @param declLHS The enumerationDecl corresponding to the lhs object
     * @param declRHS The enumerationDecl corresponding to the rhs object
     * @return true, if the rhs is assignable to the lhs, false otherwise
     */
    public static boolean
	isLeftCompatibleWithRight(SOM_EnumerationDeclaration declLHS,
				  SOM_EnumerationDeclaration declRHS) { 
	SOM_EnumerationType typeLHS = (SOM_EnumerationType)declLHS.getType();
	SOM_EnumerationType typeRHS = (SOM_EnumerationType)declRHS.getType();
	
	return SOM_EnumerationType.isLeftTypeCompatibleWithRightType(typeLHS, typeRHS);
    }
}
