//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_EnumerationListing.java,v 1.3 2001/07/03 14:17:59 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_EnumerationListing.java,v $
//  Revision 1.3  2001/07/03 14:17:59  cvs
//
//  Initial implementation of enumerations.
//  -- Morel
//
//  Revision 1.2  2000/02/10 00:37:14  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.1  1999/10/27 20:43:32  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:49:44  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.4  1999/07/28 01:59:37  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.3  1999/07/16 21:50:27  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.2  1999/07/13 14:27:12  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.1  1999/07/06 21:01:35  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//
//=============================================================================
package SOM;

/** This represents the <B>EnumerationListing</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */

// Added by Morel, June 30, 2001
import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

public class SOM_EnumerationListing
    extends SOM_AbstractExpression
    implements SOM_Factor {
    
    // List of enumeration items
    private SOM_EnumerationItemList enumList = new SOM_EnumerationItemList();
    
    /** The primary Constructor
     * @return Not applicable
     */
    public SOM_EnumerationListing() {
	super();
    }

    /** To add to the enumerationList
     * @param item The SOM_EnumerationItem to be added
     * @return void
     */
    public void addToEnumerationItemList(SOM_EnumerationItem item) {
	enumList.addElement(item);
    }
    
    // Added by Morel, June 30, 2001
    /** to retrieve the item list
     * @return SOM_EnumerationItemList
     */
    public SOM_EnumerationItemList getEnumerationItemList() {
	return enumList;
    }
    
    // Added by Morel, June 30, 2001
    /**
     * @param declsIn The SOM_DeclarationList to be set
     * @return void
     */
    public void setTypeDeclaration(SOM_TypeDeclaration currDecl) {
	
	SOM_TreeNodeList toReturn = new SOM_TreeNodeList();
	SOM_TreeNode node;
	
	node = new SOM_TreeNode(currDecl);
	toReturn.addElement(node);
	setTypeList(toReturn);
    }

    // Added by Morel, June 30, 2001
    /** To do the semcheck. Overrides that in SOM_AbstractExpression
     * Note that it assumes a semChecked expression - consistent with
     * the semChecks elsewhere..
     * @return void
     */
    public void semCheck() {
    }  
    
    // Added by Morel, June 30, 2001
    /** Overrides the resolveFinalType from SOM_AbstractExpression. This is
     * called by the external world.
     * @return void
     */
    public void resolveFinalType() {
	setResolvedType(((SOM_TreeNode)getTypeList().first()).
			getResultType()); 
    }
    
    // Added by Morel, June 30, 2001
    /** overrides the resolveFinalType from SOM_AbstractExpression.
     * @param typeIn The SOM_TypeExpression which is the expected final type
     * @return void
     */
    public void resolveFinalType(SOM_TypeDeclaration typeIn) {
	SOM_TypeDeclaration manifestType = 
	    ((SOM_TreeNode)getTypeList().first()).getResultType();
	if (SOM_TypeDeclaration.resolveFinalType(manifestType, 
						 typeIn))
	    setResolvedType(manifestType);
    }
    
    // Added by Morel, June 30, 2001
    /** Overrides the getNames from SOM_AbstractExpression
     * @return SOM_DeclarationList with all the names and their declarations
     */
    public SOM_DeclarationList getNames() {
	return null;
    }  
    
    // Added by Morel, June 30, 2001
    /** To retrieve an enumeration of the enumeration items
     * @return A ListEnumeration corresponding to the enumeration items stored
     * in the enumList
     */
    public ListEnumeration enumerations() {
	return enumList.elements();
    }
         
  //Added by Morel, June 30, 2001
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
     *@param Universe root
     * @return Expression
     */
  public Expression M__Value(Factory romFactory,
			     BaseObject parent,Universe root){
    EnumerationValue enumVal = new EnumerationValue(parent);
    ExpressionList exprList = new ExpressionList();
    ListEnumeration enums = enumerations();
    if (enums !=null){
      while (enums.hasMoreElements()){
	Object obj = enums.nextElement();
	if (obj instanceof SOM_EnumerationItem){
	  Expression romExpr =
	    ((SOM_Expression)obj).
	    M__Value(romFactory,enumVal,root);
	  exprList.addItem(romExpr);
	  	}
      }
      enumVal.setElements(exprList);
    }
    //set the type
    return (Expression)enumVal;
  }
    
    // EnumerationValue enumerationVal = new EnumerationValue(parent);
    // return (Expression)enumerationVal;
    
    // Added by Morel, June 30, 2001
    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
        ListEnumeration enumerationEnum = enumerations();
        SOM_EnumerationItem enumeration;
        pw.print("enumeration");
        pw.print("[");
        enumeration = (SOM_EnumerationItem)enumerationEnum.nextElement();
        enumeration.publishSOM(pw);
        while (enumerationEnum.hasMoreElements()) {
            pw.print(" | ");
            enumeration = (SOM_EnumerationItem)enumerationEnum.nextElement();
            enumeration.publishSOM(pw);
        }
        pw.print("]");
    }
    
    // Added by Morel, June 30, 2001
    /** dummy functions
     */
    public SOM_Label getLabel() {
	return (SOM_Label)null;
    }
    
    public SOM_Expression getLabelExpression() {
	return (SOM_Expression)null;
    }
    
    /** The publishPVS routine for this class
     * @return void
     * To be implemented later
     */
    public void publishPVS(PrintWriter pw) {
	publishSOM(pw);
    }
}
