//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ExpressionWithDyadicOperator.java,v 1.12 2001/11/02 16:19:28 cvs Exp $ 
//
//  Revision History:
//  ----------------
//  $Log: SOM_ExpressionWithDyadicOperator.java,v $
//  Revision 1.12  2001/11/02 16:19:28  cvs
//  Made changes to support facet as a type.
//    --Murthy
//
//  Revision 1.11  2001/10/01 18:46:08  cvs
//  When parsing, type info is only printed out when printInfo flag in
//  Sldl.java is set to true. Other changes include tabulation set to 2.
//  -- Cindy
//
//  Revision 1.10  2001/08/28 23:27:00  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.9  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.8  2000/06/22 17:22:41  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.7  2000/04/28 19:34:07  cvs
//  Added some more of typechecking. Added parsing of the $ operator
//  and constant literal. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:19  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/02/10 00:37:16  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:25  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.1  1999/11/05 23:43:07  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//
//=============================================================================
package SOM;

import util.List;
import util.ListEnumeration;

/** This represents the <B>ExpressionWithDyadicOperator</B> object
 * The generic rule for this expression is
 * <weak expression> ::==
 *   <stronger expression> (<weak dyadic_op> <stronger expression>)*
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu) 
 */
public abstract class SOM_ExpressionWithDyadicOperator
extends SOM_AbstractExpression {

  private SOM_AbstractExpression strongerExpr = null;
  private SOM_AbstractExpressionTailList tailList = 
    new	SOM_AbstractExpressionTailList();  

  /** The primary constructor
   * @return Not applicable
   */
  public SOM_ExpressionWithDyadicOperator() {
    super();
  }

  /** To set the strongerExpr object
   * @param exprIn The SOM_AbstractExpression object to be set
   * @return void
   */
  protected void setStrongerExpression(SOM_AbstractExpression exprIn) {
    strongerExpr = exprIn;
  }

  /** To get the expression object
   * @return The SOM_AbstractExpression - the strongerExpr
   */
  public SOM_AbstractExpression getExpression() {
    return strongerExpr;
  }

  /** To add to the abstractionExpressionTailList
   * @param exprIn The SOM_AbstractionExpressionTail object
   * @return void
   */
  protected void addToAbstractExpressionTailList
    (SOM_AbstractExpressionTail exprIn) { 
    tailList.addElement(exprIn);
  }		

  /** To get the abstractionExpressionTailList
   * @return The SOM_AbstractExpressionTailList object
   */
  protected SOM_AbstractExpressionTailList getExpressionTailList() {
    return tailList;
  }

  /** To do the sem check. A call to this function SHOULD set its typeList
   * Note that this will be inherited by each child and is generic enough
   * for most kinds of "expression" classes. Note that it implicitly
   * assumes that the strongerExpression has been set to a valid object
   * if not it will throw a NullPointerException
   * @return void
   */
  public void semCheck() {
    //System.out.println("In semCheck for: " + getClass().getName());
    ListEnumeration tailEnum = tailList.elements();

    if (tailEnum.hasMoreElements()) { // It has some dyadic ops to
      // be tackled 
			
      SOM_DyadicOperator oper;
      SOM_AbstractExpression lhsExpr, rhsExpr;
      SOM_AbstractExpressionTail exprTail;
			
      // The initial lhsExpr
      lhsExpr = strongerExpr;
			
      do { // there is atleast one element in the enum!!
	exprTail =
	  (SOM_AbstractExpressionTail)tailEnum.nextElement(); 

				// Initialize the oper and do semCheck
	oper = exprTail.getOperator();
	oper.setLeftOperand(lhsExpr);
	oper.setRightOperand(exprTail.getExpression());
	oper.semCheck();
	
	// Now for the next operator, the current oper is the
	// lhsExpr 
	lhsExpr = oper;
      } while (tailEnum.hasMoreElements());

      // Now that the semCheck is done, set my type.
      setTypeList(oper.getTypeList());
    }
    else setTypeList(strongerExpr.getTypeList()); // This is the only
    // thing here!

  }

  /** Overrides the resolveFinalType from SOM_AbstractExpression. This is
   * called by the external world. This is generic enough and should be
   * overridden at the leaves of the expression tree!
   * @return void
   */
  public void resolveFinalType() {
    ListEnumeration reverseEnum = tailList.reverseElements();
    //System.out.println("In resolveFinalType for: " + 
    //getClass().getName());
    if (reverseEnum.hasMoreElements()) {
      // Get the operator at the top of the expression tree
      SOM_AbstractExpressionTail exprTail = 
	(SOM_AbstractExpressionTail) reverseEnum.nextElement();
      SOM_DyadicOperator oper = exprTail.getOperator();

      // resolveFinalType on the operator should recursively cascade
      // down to each element
      try {
	oper.resolveFinalType();
      }catch (Exception e) {
	System.out.println("the error is in "+getClassName()+e);
      }
      //System.out.println("In ExpressionWithDyadicOperator" +
      //				   " resolvedtype on: " + oper.toString()
      //				   + " is: " + oper.getResolvedType()); 
      setResolvedType(oper.getResolvedType());
    }
    else {
      strongerExpr.resolveFinalType();
      setResolvedType(strongerExpr.getResolvedType());
    }
  }

  /** overrides the resolveFinalType from SOM_AbstractExpression.
   * @param typeIn The SOM_TypeExpression which is the expected final type
   * @return void
   */
  public void resolveFinalType(SOM_TypeDeclaration typeIn) {
    ListEnumeration reverseEnum = tailList.reverseElements();
    //System.out.println("In resolveFinalType(type) for: " + 
    //				   getClass().getName());
    if (reverseEnum.hasMoreElements()) {
      // Get the operator at the top of the expression tree
      SOM_AbstractExpressionTail exprTail = 
	(SOM_AbstractExpressionTail) reverseEnum.nextElement();
      SOM_DyadicOperator oper = exprTail.getOperator();

      // now resolveFinalType on the operator should recursively cascade
      // down to each element
      oper.resolveFinalType(typeIn);
      setResolvedType(oper.getResolvedType());
    }
    else {
      strongerExpr.resolveFinalType(typeIn);
      setResolvedType(strongerExpr.getResolvedType());
    }
  }

  /** Overrides the getNames from SOM_AbstractExpression
   * @return SOM_DeclarationList with all the names and their declarations
   */
  public SOM_DeclarationList getNames() {
    //System.out.println("In ExpressionWithDyadicOperator, calling"
    //				   + " getNames() of: " 
    //				   + strongerExpr.getClass().getName());
    SOM_DeclarationList toReturn = strongerExpr.getNames();

    ListEnumeration enum = tailList.elements();
    if (enum.hasMoreElements()) {
      SOM_AbstractExpressionTail exprTail;
      SOM_AbstractExpression currExpr;
      while (enum.hasMoreElements()) {
	exprTail = (SOM_AbstractExpressionTail) enum.nextElement();
	currExpr = exprTail.getExpression();
	List.concat(toReturn, currExpr.getNames());
      }
    }

    return toReturn;
  }	
}

