//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ExpressionWithMonoadicOperator.java,v 1.11 2001/03/21 00:05:05 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ExpressionWithMonoadicOperator.java,v $
//  Revision 1.11  2001/03/21 00:05:05  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.10  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.9  2000/08/01 22:48:47  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.8  2000/07/24 23:41:02  cvs
//  Some type checking added for record and sequence.
//  -- Cindy
//
//  Revision 1.7  2000/06/22 17:22:41  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.6  2000/04/06 19:13:20  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.5  2000/02/10 00:37:16  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:26  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.2  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.1  1999/11/05 23:43:05  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;

/** This represents the <B>ExpressionWithMonoadicOperator</B> object
 * The generic rule for this expression is
 * <weak expression> ::==
 *   <monoadic operator> <stronger expression>
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu) 
 */
public abstract class SOM_ExpressionWithMonoadicOperator
extends SOM_AbstractExpression {

	private SOM_AbstractExpression strongerExpr = null;
	private SOM_MonoadicOperator monoadicOp = null;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_ExpressionWithMonoadicOperator() {
		super();
	}

	/** To set the strongerExpr object
	 * @param exprIn The SOM_AbstractExpression object to be set
	 * @return void
	 */
	protected void setStrongerExpression(SOM_AbstractExpression exprIn) {
		strongerExpr = exprIn;
	}

	/** To get the expression object
	 * @return The SOM_AbstractExpression - the strongerExpr
	 */
	public SOM_AbstractExpression getExpression() {
		return strongerExpr;
	}

	/** To set the Monoadic Operator
	 * @param opIn The SOM_MonoadicOperator to be set
	 * @return void
	 */
	public void setMonoadicOperator(SOM_MonoadicOperator opIn) {
		monoadicOp = opIn;
	}

	/** To get the Monoadic Operator
	 * @return The SOM_MonoadicOperator for this object
	 */
	public SOM_MonoadicOperator getMonoadicOperator() {
		return monoadicOp;
	}

	/** To do the sem check. A call to this function SHOULD set its typeList.
	 * Note that it implicitly assumes that the strongerExpression and
	 * monoadicOp have been set to valid objects if not it will throw a
	 * NullPointerException
	 * @return void
	 */
	public void semCheck() {
		monoadicOp.setOperand(strongerExpr);
		monoadicOp.semCheck();
		setTypeList(monoadicOp.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		// resolveFinalType on the operator should recursively cascade
		// down to each element
		monoadicOp.resolveFinalType();
		setResolvedType(monoadicOp.getResolvedType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// now resolveFinalType on the operator should recursively cascade
		// down to each element
		monoadicOp.resolveFinalType(typeIn);
		setResolvedType(monoadicOp.getResolvedType());
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		return strongerExpr.getNames();
	}

    /** publishSOM routine to print the object module
     * @param PrintWriter
     * @return void - mak
     */	
     public void publishSOM(PrintWriter pw){
		 pw.print(" " + monoadicOp + " ");
		 pw.print(" " + strongerExpr + " ");
     }
		
	/** publishPVS routine to print the object module
     * @param PrintWriter
     * @return void - mak
     */	
	public void publishPVS(PrintWriter pw){
		/*		if (String.valueOf(monoadicOp) == "-"){
				pw.print(" NOT");
				}
				else {*/
		pw.print(" " + monoadicOp + " ");
		//}
		pw.print(" " + strongerExpr + " ");
	}
}
