//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_FacetDefinition.java,v 1.14 2001/03/12 23:54:38 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FacetDefinition.java,v $
//  Revision 1.14  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.13  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.12  2000/05/18 18:03:44  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.12  2000/05/05 19:28:31  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.11  2000/04/21 02:09:39  cvs
//  Made changes to the publishSOM routine so that the ObjectModel
//  file is correct 
//  --srinivas
//
//  Revision 1.10  2000/04/08 02:21:59  cvs
//  The Meta Functions have been implemented and these are the files
//  that had to be modified.Addded some accessor methods to classes
//  like SOM_DeclarationStatement, SOM_Expression,
//  SOM_RecordTupleDeclarationStatement.The Meta functions for facet
//  terms and the term items are implemented 
//  --srinivas
//
//  Revision 1.9  2000/04/06 19:13:20  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.8  2000/03/23 00:22:47  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.7  2000/03/03 18:38:28  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.6  2000/02/17 01:27:04  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//  Revision 1.5  2000/02/10 00:37:18  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:27  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/16 04:05:24  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.2  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.1  1999/11/11 00:04:22  cvs
//  Fresh changes as part of the streamlining process with new
//  object model and grammar
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import java.io.Serializable;

/** This represents the <B>FacetDefinition</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */

// made serializable by makarand

public class SOM_FacetDefinition
extends SOM_BaseClass 
implements Serializable {

	/** Local data members */
	SOM_FacetDeclarativeBlock declBlock = null; //optional
	SOM_ExportBlock exportBlock = null; //optional
	SOM_FacetExpression domainExpression = null; // since it may be
	// the null keyword 
	SOM_FacetBody facetBody = null; // optional

	/* This list will store the declarations of this facet's parents as
		 given by the domainexpression */
	SOM_DeclarationList parentDeclList = null;

	/** The primary constructor
	 * @return Not applicable
	 */
	public SOM_FacetDefinition() {
		super();
	}

	/** To set the declBlock
	 * @param declBlockIn The SOM_FacetDeclarativeBlock to be set
	 * @return void
	 */
	public void setFacetDeclarativeBlock(SOM_FacetDeclarativeBlock 
										 declBlockIn) {
		declBlock = declBlockIn;
	}

	/** To get the declBlock
	 * @return The SOM_FacetDeclarativeBlock of this facet definition
	 */
	public SOM_FacetDeclarativeBlock getFacetDeclarativeBlock() {
		return declBlock;
	}


	/** To set the exportBlock
	 * @param exportBlockIn The SOM_ExportBlock to be set
	 * @return void
	 */
	public void setExportBlock(SOM_ExportBlock exportBlockIn) {
		exportBlock = exportBlockIn;
	}

	/** To get the exportBlock
	 * @return The SOM_ExportBlock of this facet definition
	 */
	public SOM_ExportBlock getExportBlock() {
		return exportBlock;
	}
		
	/** To set the domainExpression. Note assumes that the domain expression
	 * has been typechecked and resolved to type facet
	 * @param expressionIn The SOM_Expression object to be set
	 * @return void
	 */
	public void setDomainExpression(SOM_FacetExpression expressionIn) {
		domainExpression = expressionIn;
		// notice the fresh creation of the parentDeclList!
		// The alternative is to concat it to any already existing
		// parentDeclList 
		// the implementation will depend on the semantics of facet expressions
		// Commented out by Cindy Feb 22, 2000
		//parentDeclList = domainExpression.getNames();
	}
	
	/** added by srinivas to get the domain expression
	 *@param void
	 * @ return SOM_FacetExpression
	 */
    public SOM_FacetExpression getDomainExpression(){
		return domainExpression;
    }


	/** To get the parentDeclList
	 * @return SOM_DeclarationList that is the parentDeclList
	 */
	public SOM_DeclarationList getParentDeclarationList() {
		return parentDeclList;
	}

	/** To set the facetBody
	 * @param bodyIn The SOM_FacetBody to be set
	 * @return void
	 */
	public void setFacetBody(SOM_FacetBody bodyIn) {
		facetBody = bodyIn;
	}

	/** To get the facetBody
	 * @return The SOM_FacetBody of this facet definition
	 */
	public SOM_FacetBody getFacetBody() {
		return facetBody;
	}
  
   	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		if (declBlock != null) {
			declBlock.publishSOM(pw);
		}
		if (exportBlock != null) {
			exportBlock.publishSOM(pw);
		}
		pw.print("BEGIN ");
		if (domainExpression != null) {
			domainExpression.publishSOM(pw);
			pw.println();
		} else {
			pw.println("null ");
		}
		if (facetBody != null) {
			facetBody.publishSOM(pw);
		}
		pw.print("END ");
	}

}
