//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_FacetExpression.java,v 1.6 2001/10/01 18:46:09 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FacetExpression.java,v $
//  Revision 1.6  2001/10/01 18:46:09  cvs
//  When parsing, type info is only printed out when printInfo flag in
//  Sldl.java is set to true. Other changes include tabulation set to 2.
//  -- Cindy
//
//  Revision 1.5  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.4  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.3  2000/04/21 02:09:40  cvs
//  Made changes to the publishSOM routine so that the ObjectModel
//  file is correct 
//  --srinivas
//
//  Revision 1.2  2000/04/06 19:13:20  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy 
//
//  Revision 1.1  2000/02/17 01:27:04  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import java.io.PrintWriter;
import ROM.*;

/** This represents the <B>FacetExpression</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_FacetExpression
extends SOM_Expression {
  // This represents the <facetExpression> object, for which the grammar is
  // <facetExpression> ::==
  //   <facetAndExpression> (<facetExpressionTail>)*
  //
  // <facetExpressionTail> ::== (IMPLIES | OR) <facetAndExpression>

	/** The primary Constructor
	 * @return Not applicable
	 */
  public SOM_FacetExpression() {
    super();
  }

  /** To set the facetAndExpression
	 * @param expressionIn The SOM_FacetAndExpression object to be set
	 * @return void
	 */
  public void setFacetAndExpression(SOM_FacetAndExpression expressionIn) {
    setStrongerExpression(expressionIn);
  }

  /** To get the facetAndExpression
	 * @return SOM_FacetAndExpression
	 */
  public SOM_FacetAndExpression getFacetAndExpression() {
    return (SOM_FacetAndExpression)getExpression();
  }

  /** To add to the facetExprTailList
	 * @param facetExprTail The SOM_FacetExpressionTail object to be added
	 * @return void
	 */
  public void addToFacetExpressionTailList(SOM_FacetExpressionTail 
					   facetExprTail) {
    addToAbstractExpressionTailList(facetExprTail);
  }

  /** To retrieve the listEnumeration of the facetExprTailList
	 * @return The ListEnumeration corresponding to the facetExprTailList
	 */
  public ListEnumeration facetExpressions() {
    return getExpressionTailList().elements();
  }

  //added by ckong March 5, 2001
  /*The Meta Function to get the value of the item
   *@param BaseObject parent
   *@param Universe root
   * @return Expression
   */
  public Expression M__Value(Factory romFactory, BaseObject parent,
			     Universe root){
    Expression leftNode;
    ListEnumeration expressionTailEnum = facetExpressions();
    if (expressionTailEnum != null){
      if(expressionTailEnum.hasMoreElements()){
	Formula intermExpr = new Formula(parent);
	leftNode = getFacetAndExpression().M__Value(romFactory,
						    intermExpr, root);
	Formula retExpr = intermExpr;
	while (expressionTailEnum.hasMoreElements()) {
	  SOM_FacetExpressionTail exprTail =
	    (SOM_FacetExpressionTail)expressionTailEnum.nextElement();
	  SOM_DyadicOperator oper = exprTail.getOperator();
	  Expression rightNode = exprTail.getFacetAndExpression().
	    M__Value(romFactory, intermExpr, root);
	  intermExpr.setLeftOp(leftNode);
	  String str = oper.toString();
	  Type operType = root.getType(oper.getResolvedType().toString());
	  OperatorReference operRef = new OperatorReference
	    (intermExpr, root.labelLookup(str), operType);
	  intermExpr.setOperator(operRef);
	  intermExpr.setRightOp(rightNode);
	  retExpr = intermExpr;
	  leftNode = intermExpr;
	  intermExpr = new Formula(parent);
	}
	return retExpr;
      }else {
	leftNode = getFacetAndExpression().M__Value(romFactory,parent,root);
	return leftNode;
      }
    } else {
      leftNode = getFacetAndExpression().M__Value(romFactory,parent,root);
      return leftNode;
    }
  }


  /** The publishSOM routine for this class
	 * @return void
	 */
  public void publishSOM(PrintWriter pw) {
    ListEnumeration facetExprTailEnum = this.facetExpressions();
    SOM_FacetExpressionTail facetExprTail;

    getExpression().publishSOM(pw);

    while (facetExprTailEnum.hasMoreElements()) {
      facetExprTail = 
	(SOM_FacetExpressionTail)facetExprTailEnum.nextElement();
      facetExprTail.publishSOM(pw);
    }	
  }

  /** The publishPVS routine for this class
	 * @return void
     * To be implemented as needed */
  public void publishPVS(PrintWriter pw) {
    publishSOM(pw);
  }
}
