//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_FacetSelectedName.java,v 1.4 2001/03/12 23:54:38 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_FacetSelectedName.java,v $
//  Revision 1.4  2001/03/12 23:54:38  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.3  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.2  2000/04/06 19:13:20  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.1  2000/02/17 01:27:05  cvs
//  Implemented the facet expression grammar to incorporate facet composition.
//  Also updated the "use" clause with the above changes.
//  - krishna.
//
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import ROM.*;

/** This represents the <B>FacetSelectedName</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_FacetSelectedName
extends SOM_ExpressionWithMonoadicOperator {

	//private SOM_SelectedName selName;
	//private SOM_Name name;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_FacetSelectedName() {
		super();
	}

	/** To set the operator
	 * @param opIn The SOM_MonoadicOperator to be set
	 * @return void
	 */
	public void setOperator(SOM_MonoadicOperator opIn) {
		super.setMonoadicOperator(opIn);
	}

	/** To set the SelectedName
	 * @param selNameIn The SOM_SelectedName to be set
	 * @return void
	 */
	//public void setSelectedName(SOM_SelectedName selNameIn) {
	//	selName = selNameIn;
	//}
	public void setName(SOM_Name nameIn) {
		//name = nameIn;
		setStrongerExpression(nameIn);
	}

	/** To get the SelectedName
	 * @return SOM_SelectedName
	 */
	//public SOM_SelectedName getSelectedName() {
	//	return selName;
	//}
	public SOM_Name getName() {
		//return name;
		return (SOM_Name)getExpression();
	}
	
	/** To do the sem check. A call to this function SHOULD set its typeList.
	 * Note that it implicitly assumes that the strongerExpression and
	 * monoadicOp have been set to valid objects if not it will throw a
	 * NullPointerException
	 * @return void
	 */
	public void semCheck() {
		if (getMonoadicOperator() == null)
			setTypeList(getExpression().getTypeList());
		else 
			super.semCheck();
	}

	/** overrides the resolveFinalType from SOM_ExpressionWithMonoadicOperator.
	 * @return void
	 */
	public void resolveFinalType() {
		if (getMonoadicOperator() == null) {
			getExpression().resolveFinalType();
			setResolvedType(getExpression().getResolvedType());
		} else
			super.resolveFinalType();
	}

	/** overrides the resolveFinalType from SOM_ExpressionWithMonoadicOperator.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		if (getMonoadicOperator() == null) {
			getExpression().resolveFinalType(typeIn);
			setResolvedType(getExpression().getResolvedType());
		} else
			super.resolveFinalType(typeIn);
	}

	//added by ckong March 6, 2001
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory, BaseObject parent,
							   Universe root){
		Expression expr;
		SOM_MonoadicOperator op = getMonoadicOperator();
		Formula retExpr = new Formula(parent);
		if (op != null) {
			String str = op.toString();
			Type operType = root.getType(getMonoadicOperator().
										 getResolvedType().toString());
			OperatorReference operRef = new
				OperatorReference(retExpr, root.labelLookup(str), operType);
			retExpr.setOperator(operRef);
			//expr = getSelectedName().M__Value(romFactory, retExpr, root);
			expr = getName().M__Value(romFactory, retExpr, root);
			retExpr.setRightOp(expr);
			return retExpr;
		}
		//expr = getSelectedName().M__Value(romFactory, parent, root);
		expr = getName().M__Value(romFactory, retExpr, root);
		return expr;
	}
	
	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		if(getMonoadicOperator() != null)
			getMonoadicOperator().publishSOM(pw);
		getName().publishSOM(pw);
	}

}
