//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_GroupedExpression.java,v 1.13 2000/11/18 00:18:27 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_GroupedExpression.java,v $
//  Revision 1.13  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.12  2000/10/30 22:13:23  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.11  2000/08/01 22:48:49  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.10  2000/06/22 17:22:43  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.9  2000/04/06 19:13:21  cvs
//  Type checking is done for declaration of variables (simple types), for declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.8  2000/03/23 00:22:47  cvs
//  Huge commit! Added all the essential accessor methods for the following classes
//  - Krishna and Makarand
//
//  Revision 1.7  2000/02/10 00:37:20  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  2000/02/08 18:42:28  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.5  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/05 23:43:10  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:37  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:15  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/07/14 23:23:48  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.1  1999/06/27 17:33:30  rkamath
//  Added some of the object model for the Expression grammar and a few changes to
//  the other files..
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import ROM.*;
/** This represents the <B>GroupedExpression</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_GroupedExpression
extends SOM_AbstractExpression
implements SOM_Factor {

	private SOM_AbstractExpression expression;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_GroupedExpression() {
		super();
	}

	/** To set the expression
	 * @param expressionIn the SOM_Expression to be set
	 * @return void
	 */
	public void setExpression(SOM_Expression expressionIn) {
		expression = expressionIn;
	}

	/** To get the expression
	 * @return The SOM_Expression
	 */
	public SOM_Expression getExpression() {
		return (SOM_Expression)expression;
	}

	/** To do the semcheck. Overrides that in SOM_AbstractExpression
	 * Note that it assumes a semChecked expression - consistent with
	 * the semChecks elsewhere..
	 * @return void
	 */
	public void semCheck() {
		setTypeList(expression.getTypeList());
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world. 
	 * @return void
	 */
	public void resolveFinalType() {
		expression.resolveFinalType();
		setResolvedType(expression.getResolvedType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		expression.resolveFinalType(typeIn);
		setResolvedType(expression.getResolvedType());
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		return expression.getNames();
	}


	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent,
							   Universe root){
		if (getExpression() instanceof SOM_Expression){
			return getExpression().M__Value(romFactory,parent,root);
		}else
			return null;
	}
	
	/** The publishSOM routine for this class
	 * @return void
	 */
    public void publishSOM(PrintWriter pw)
    {
		pw.print("(");
		expression.publishSOM(pw);
		pw.print(")");
	}

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
	
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}
	
	/** The publishPVS routine for this class
	 * @return void
	 */
    public void publishPVS(PrintWriter pw)
    {
		pw.print("(");
		expression.publishPVS(pw);
		pw.print(")");
	}
}



