//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_IdentifierListing.java,v 1.5 2000/02/10 00:37:20 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_IdentifierListing.java,v $
//  Revision 1.5  2000/02/10 00:37:20  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:30  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.3  1999/11/16 04:05:24  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.2  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:38  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:20  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.1  1999/06/30 22:19:59  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;

import java.io.PrintWriter;

/** This represents the <B>IdentifierListing</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_IdentifierListing
extends SOM_BaseClass {

	private SOM_IdentifierList identifierList = new SOM_IdentifierList();

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_IdentifierListing() {
		super();
	}

	/** To add an identifier
	 * @param idIn The SOM_Identifer to be added
	 * @return void
	 */
	public void addToIdentifierList(SOM_Identifier idIn) {
		identifierList.addElement(idIn);
	}

	/** to retreive the identifier list
	 * @return SOM_IdentifierList 
	 */
	public SOM_IdentifierList getIdentifierList() {
		return identifierList;
	}

	/** To retreive an enumeration of the identifiers
	 * @return A ListEnumeration corresponding to the identifiers stored
	 * in the identifierList
	 */
	public ListEnumeration identifiers() {
		return identifierList.elements();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
	    ListEnumeration identifierEnum = identifiers();
	    SOM_Identifier id;
	    
		id = (SOM_Identifier)identifierEnum.nextElement();
	    id.publishSOM(pw);
	    while (identifierEnum.hasMoreElements()) {
		    pw.print(", ");
		    id=(SOM_Identifier)identifierEnum.nextElement();
			id.publishSOM(pw);
	    }
	}
}
