//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_ImpliesExpressionTail.java,v 1.5 2000/08/01 22:48:50 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_ImpliesExpressionTail.java,v $
//  Revision 1.5  2000/08/01 22:48:50  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.4  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the following classes
//  - Krishna and Makarand
//
//  Revision 1.3  2000/02/10 00:37:21  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  2000/02/08 18:42:31  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes (with the help
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the object model.
//  -- Cindy.
//
//  Revision 1.1  1999/10/27 20:43:38  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:24  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:23:52  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/06/30 22:20:02  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.1  1999/06/30 02:19:05  rkamath
//  Added a lot more files dealing with the expression OM.
//  Also modified some previous classes
//  Now the ROOT in the hierarchy is the SOM_Unifier interface.
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;

/** This represents the <B>ImpliesExpressionTail</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_ImpliesExpressionTail
extends SOM_AbstractExpressionTail {

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_ImpliesExpressionTail() {
		super();
	}

	/** To set the implies operator
	 * @param opIn The SOM_ImpliesOperator to be set
	 * @return void
	 */
	public void setImpliesOperator(SOM_DyadicOperator opIn) {
		setOperator(opIn);
	}

	/** To get the implies operator
	 * @return The SOM_ImpliesOperator for this expression
	 */
	public SOM_DyadicOperator getImpliesOperator() {
		return getOperator();
	}

	/** To set the andExpression
	 * @param andIn The SOM_AndExpression to be set
	 * @return void
	 */
	public void setAndExpression(SOM_AndExpression andIn) {
		setExpression(andIn);
	}

	/** To get the andExpression
	 * @return The SOM_AndExpression for this expression
	 */
	public SOM_AndExpression getAndExpression() {
		return (SOM_AndExpression)getExpression();
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
		getOperator().publishSOM(pw);
		getExpression().publishSOM(pw);
	}

		/** The publishPVS routine for this class
		 * @return void
		 */
		public void publishPVS(PrintWriter pw) {
				getOperator().publishPVS(pw);
				getExpression().publishPVS(pw);
		}
}
