
package SOM;
import util.List;
import util.Link;
import util.ListEnumeration;

import java.io.*;
import java.lang.NullPointerException;
import java.io.Serializable;
import java.lang.String;

import ROM.*;

/** This represents the <B>InteractionDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
    // made serializable by makarand

public class SOM_InteractionDeclaration
extends SOM_Declaration
implements  SOM_PackageDeclarativeItem,SOM_InteractionDeclarativeItem, Serializable {
    
    /** The name of this Interaction - and other dataMembers */
    private SOM_Identifier id;
    private SOM_InterfaceListing interfaceList = null;
    private SOM_InteractionDefinition interactionDefinition = null;
    
    private transient SOM_DeclarationList domainList;
    
    /** The primary constructor
     * @param interTypeIn The typeDeclaration corresponding to the "interaction type"
     * @return Not applicable
     */
    public SOM_InteractionDeclaration(SOM_TypeDeclaration interTypeIn) {
		super();
		setKind(INTERACTION);
		setType(interTypeIn);
    }
    
    /** To set the identifier
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
    }			 
    
    /** To get the identifier
     * @return The SOM_Identifier for this designUnit
     */
    public SOM_Identifier getIdentifier() {
		return id;
    }			 
    
    /** To set the interfaceListing - implements the function from
     * SOM_DesignUnit

     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interfaceList = interfaceIn;
    }
    
    /** To get the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
		return interfaceList;
    }
    
    /** To set the domainList - implements the function from
     * SOM_DesignUnit
     * @param domainListIn The SOM_DeclarationList object to be set
     * @return void
     */
    public void setDomainList(SOM_DeclarationList domainListIn) {
		domainList = domainListIn;
    }
    
    /** To get the domainList - implements the function from
     * SOM_DesignUnit
     * @return The SOM_DeclarationList object for this designUnit
     */
    public SOM_DeclarationList getDomainList() {
		return domainList;
    }
    
    /** To set the interactionDefinition
     * @param interDefIn The SOM_InteractionDefinition object to be set
     * @return void
     */
    public void setInteractionDefinition(SOM_InteractionDefinition interDefIn){
		interactionDefinition = interDefIn;
    }
    
    /** To get the intearctionDefinition
     * @return The SOM_InteractionDefinition object to get
     */
    public SOM_InteractionDefinition getInteractionDefinition(){
		return interactionDefinition;
    }

    /** To get the exports as a ListEnumeration
     * @return ListEnumeration corresponding to the export clauses
     */
    public ListEnumeration exports() {
                SOM_ExportBlock exportBlock = interactionDefinition.getExportBlock();
                if (exportBlock != null)
                        return exportBlock.exports();
                else
                        return null;
    }

    public ListEnumeration M__Terms()
    {
	List terms = new List();
	SOM_InteractionBody interactionBody = getInteractionDefinition().getInteractionBody();
	if(interactionBody != null)
	    {
		ListEnumeration items = interactionBody.items();
		if(items != null)
		    {
			while(items.hasMoreElements())
			    {
				SOM_InteractionBodyItem interactionItem = 
				    (SOM_InteractionBodyItem)items.nextElement();
				if(interactionItem != null)
				    {
					terms.addElement(
							 ((Object)interactionItem.getLabelledExpression()));
				    }
			    }
		    }
	    }
	if(terms.isEmpty())
	    return null;
	else
	    return terms.elements();
    }

    public ListEnumeration M__Params()
    {
	List parameters = new List();
	if(getInterfaceListing() != null)
	    {
		ListEnumeration interListEnum = 
		    getInterfaceListing().interfaces();
		SOM_InterfaceItem interItem;
		if(interListEnum != null)
		    {
			while(interListEnum.hasMoreElements())
			    {
				interItem =(SOM_InterfaceItem)interListEnum.nextElement();
				if(interItem != null)
				    parameters.addElement((Object)interItem);
			    }
		    }
	    }
	if(parameters.isEmpty())
	    {
		return null;
	    }
	else
	    {
		return parameters.elements();
	    }
    }
    /* To get the declaration statements that are declared
     * in the Interaction
     * @param void
     * @return ListEnumeration
     */
    public ListEnumeration getDeclarationStatements()
    {
	SOM_InteractionDeclarativeBlock declBlock = interactionDefinition.getInteractionDeclarativeBlock();
	List declStmts = new List();
	if(declBlock != null)
	    {
		ListEnumeration interactionDeclStmts = declBlock.items();
		if(interactionDeclStmts != null)
		    {
			while(interactionDeclStmts.hasMoreElements())
			    {
				SOM_InteractionDeclarativeItem declItem = 
				    (SOM_InteractionDeclarativeItem)interactionDeclStmts.nextElement();
				if(declItem instanceof SOM_DeclarationStatement)
				    {
					SOM_DeclarationStatement declStmt = ((SOM_DeclarationStatement)declItem);
					declStmts.addElement(declStmt);
				    }
			    }
		    }
	    }
	if(declStmts.isEmpty())
	    return null;
	else
	    return declStmts.elements();
    }

    /* To get the domain of the interaction
     * @param void
     * @return String
     */

    public String M__Domain()
    {
	SOM_InteractionDefinition interactionDefn = getInteractionDefinition();
	try
	    {
		if(interactionDefn != null)
		    {
			String str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			if(interactionDefn.getDomainExpression() != null)
			    interactionDefn.getDomainExpression().publishSOM(pw);
			pw.flush();
			pw.close();
			str = sw.toString();
			sw.close();
			return str;
		    }
		else return null;
	    }
	catch(Exception e)
	    {
		System.out.println(e);
		return (String)null;
	    }
    }

    /* To get the String representation of this object overriding 
     * the method in SOM_Declaration 
     * @return String the name that cooresponds to the 
     * name of the interaction
     */   
       public String toString() {
       return id.toString();
       }
 
    /* The function that cretes the ROM.Expression for the InteractionItem 
     * @param Factory,BaseObject,Universe
     * @return ROM.Expession(ROM.Interaction)
     */

    public Expression M__Value(Factory romFactory, BaseObject parent,Universe root)
    {
	Interaction interactionValue = (Interaction)parent;
	ListEnumeration paramList = M__Params();
	SOM_InterfaceItem interItem;
	if(paramList != null)
	    {
		while(paramList.hasMoreElements())
		    {
			interItem = (SOM_InterfaceItem)paramList.nextElement();
			interactionValue.addInterfaceItem
			    (interItem.buildROM(romFactory,root,interactionValue));
		    }
		ListEnumeration decList = getDeclarationStatements();
		SOM_InteractionDeclarativeItem interactionDeclItem;

		if(decList != null)
		    {
			while(decList.hasMoreElements())
			    {
				interactionDeclItem = (SOM_InteractionDeclarativeItem)decList.nextElement();
				if(interactionDeclItem instanceof SOM_InteractionDeclaration)
				    interactionValue.addItem(
							     ((SOM_InteractionDeclaration)interactionDeclItem).
							     buildROM(romFactory,root,interactionValue));

				if(interactionDeclItem instanceof SOM_DeclarationStatement)
				    interactionValue.addItem(
							     ((SOM_DeclarationStatement)interactionDeclItem).
							     buildROM(romFactory,root,interactionValue));
			    }
			Label domainLabel = romFactory.newLabel((Item)interactionValue.getParent(),M__Domain());
			interactionValue.setDomain(domainLabel);

			ListEnumeration exportList = exports();
			SOM_Identifier exportId=null;
			if(exportList != null)
			    {
				if(!exportList.hasMoreElements()){
				    interactionValue.addExportLabel(romFactory.newLabel
								    ((Item)interactionValue.getParent(),exportId.toString()));
				}
			    }
		    }
		ListEnumeration terms = M__Terms();
		if(terms != null)
		    {
			while(terms.hasMoreElements())
			    {
				SOM_LabelledExpression labelExpr = ((SOM_LabelledExpression)terms.nextElement());
				if(labelExpr != null)
				    interactionValue.addTerm(labelExpr.buildROM(romFactory,root,interactionValue));
			    }
		    }

	    }
		return (Expression)interactionValue;
    }
							     
    /*To get the String of a interaction item
     * in the interaction
     */
    public String M__String(){
	try{
	    String str = new String();
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    publishSOM(pw);
	    pw.flush();
	    pw.close();
	    str = sw.toString();
	    sw.close();
	    return str;
	}
	catch (Exception e) {
	    System.out.println(e);
	    return (String)null;
	}
    }
    
    
    
    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
	pw.print("INTERACTION ");
	id.publishSOM(pw);
	if (interfaceList != null) {
	    interfaceList.publishSOM(pw);
	}
	pw.println(" IS ");
	interactionDefinition.publishSOM(pw);
	id.publishSOM(pw);
                pw.print(";\n");
                pw.println();
		
    }
    
    /** The buildROM method for this high-level item
     * @param ROMFactory, Factory object used to create objects in ROM
     * @return LabelledItem, corresponding to this intraction
     */
    public LabelledItem buildROM(Factory romFactory,
				 Universe root, BaseObject parentIn) {
	LabelledItem interactionItem = romFactory.newLabelledItem
	    (parentIn,null,null,null,M__String());
	Label interactionLabel = romFactory.newLabel(interactionItem,toString());
	interactionItem.setLabel(interactionLabel);
	//To create the value
	ROM.Interaction interactionValue = romFactory.newInteraction(interactionItem);
	((InteractionType)root.getType("interaction")).addOneElement(interactionValue);
	interactionItem.setValue(interactionValue);
	M__Value(romFactory,(BaseObject)interactionValue,root);
	Type interactionType = root.getType("interaction");
	interactionItem.setType(interactionType);
	
	return interactionItem;
    }				
}







