//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_InterfaceItem.java,v 1.17 2001/08/28 23:27:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_InterfaceItem.java,v $
//  Revision 1.17  2001/08/28 23:27:01  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.16  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.15  2001/02/08 22:58:52  cvs
//  Modified the classes to remove some errors while building the
//  Rosetta Object Model. 
//  --srinivas
//
//  Revision 1.14  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.13  2000/10/22 18:42:02  cvs
//  Meta functions are added to the SOM_InterfaceItem class and the
//  meta function M__Domain is changed so that it returns a string and
//  not SOM_FacetExpression 
//  --srinivas
//
//  Revision 1.12  2000/08/18 15:17:41  cvs
//  modified publishPVS routines to support quantified expression -
//  changes for soccer, makarand 
//
//  Revision 1.11  2000/08/01 22:48:51  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.10  2000/06/02 15:41:30  cvs
//  All output var are transformed to tick variables in PVS - for soccer
//
//  Revision 1.9  2000/05/21 17:54:59  cvs
//  modified some changes lost during the file system crash
//
//  Revision 1.8  2000/05/19 00:03:59  cvs
//  restoring the files after the file system had crashed
//  --srinivas
//
//  Revision 1.11  2000/05/10 16:22:36  cvs
//  used the typeName to extract the type information of components
//  needed in SOCCER - makarand 
//
//  Revision 1.10  2000/05/08 22:57:47  cvs
//  Passing type information in the publishPVS routine needed by
//  SOCCER interface -makarand 
//
//  Revision 1.9  2000/05/05 19:28:32  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.8  2000/05/03 00:37:06  cvs
//  change to support post conditions with no ticks i.e with label
//  post; - made by makarand for soccer 
//
//  Revision 1.7  2000/05/02 15:52:42  cvs
//  modified the publishPVS routine to support tick expressions for
//  soccer - makarand 
//
//  Revision 1.6  2000/04/02 18:25:55  cvs
//  modified publishPVS functions to return 'in' and 'out' variables -
//  by makarand for soccer 
//
//  Revision 1.5  2000/03/23 19:16:39  cvs
//  added publishPVS routines for SOCCER - makarand
//
//  Revision 1.4  2000/02/10 00:37:22  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.3  2000/02/08 18:42:33  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.2  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:39  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:31  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.8  1999/10/01 15:23:29  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.7  1999/07/28 01:59:41  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.6  1999/07/22 22:53:21  rkamath
//  Lot of changes especially in terms of adding accessors to the OM
//  This does not compile completely, the toString() of SOM_FormalIdentifier
//  is to be completed. This commit is purely for records
//
//  Revision 1.5  1999/07/09 22:21:14  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//  Revision 1.4  1999/07/06 21:01:38  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//  Revision 1.3  1999/06/30 22:20:03  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.2  1999/06/25 15:10:32  rkamath
//  Modified the Makefile and changed the javadoc comments in the java files
//
//  Revision 1.1  1999/06/23 18:35:10  rkamath
//  A fresh new addition and modification of new files
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import java.io.*;
import java.io.PrintWriter;

import ROM.*;

/** This represents the <B>InterfaceItem</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_InterfaceItem
extends SOM_BaseClass{

	private SOM_Identifier id;
	private SOM_Mode mode = null;

	// An InterfaceItem should have a well defined and resolved type!
	private SOM_Name typeName;
	private SOM_TypeDeclaration type;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_InterfaceItem() {
		super();
	}

	/** To set the idListing
	 * @param idIn The SOM_Identifier to be set
	 * @return void
	 */
	public void setIdentifier(SOM_Identifier idIn){
		id = idIn;
	}

	/** To retreive the id
	 * @return The id
	 */
	public SOM_Identifier getIdentifier() {
		return id;
	}

	/** To set the mode
	 * @param modeIn The SOM_Mode object to be set
	 * @return void
	 */
	public void setMode(SOM_Mode modeIn) {
		mode = modeIn;
	}

	/** To get the Mode
	 * @return the SOM_Mode corresponding to the mode
	 */
	public SOM_Mode getMode() {
		return mode;
	}

	/** To set the typeName
	 * @param typeNameIn The SOM_Name corresponding to the type
	 * @return void
	 */
	public void setTypeName(SOM_Name typeNameIn) {
		typeName = typeNameIn;
	}

	/** To get the typeName
	 * @return The SOM_Name corresponding to the type
	 */
	public SOM_Name getTypeName() {
		return typeName;
	}

	/** To set the type
	 * @param typeIn The SOM_Declaration corresponding to the type
	 * @return void
	 */
	public void setType(SOM_TypeDeclaration typeIn) {
		type = typeIn;
	}

	/** To get the type
	 * @return The SOM_Declaration corresponding to the type
	 */
	public SOM_TypeDeclaration getType() {
		return type;
	}
	
	/**added by srinivas on Oct 21st,2000.
	 * To get the label of the interface item
	 * @param void
	 * @return String
	 */
	public String M__Label(){
		return id.toString();
	}
	
	/**added by srinivas on Oct21st, 2000.
	 * To get the string of the object
	 * @param void
	 * return String
	 */
	public String M__String(){
		try{
			String str;
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			publishSOM(pw);
			pw.flush();
			pw.close();
			str=sw.toString();
			sw.close();
			return str;
		}catch(Exception e){
			System.out.println(e);
			return null;
		}
	}
	
	/**added by srinivas on Oct21st, 2000
	 * To get the type of the identifier in the Interface Item.
	 * @param void
	 * @return String
	 */
	public String M__Type(){
		try{
			String str;
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			typeName.publishSOM(pw);
			pw.flush();
			pw.close();
			str = sw.toString();
			sw.close();
			return str;
		}catch(Exception e){
			System.out.println(e);
			return null;
		}
	}
	
	/**added by srinivas on Oct21st, 2000
	 * To get the value of the Interface Item.
	 * @param void
	 * @return Expression
	 */
	public Expression M__Value(){
		return null;
	}
		
	/** The buildROM method for this high-level item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe root of the ROM
	 * @param parentIn, BaseObject parent of item
	 * @return LabelledItem, corresponding to this interface item
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn) {

		Type type = root.getType(M__Type());
		if (type == null)
			if (parentIn instanceof Facet)
				type = ((Facet)parentIn).getType(M__Type());
		ParameterItem paramItem = romFactory.newParameterItem
			(parentIn, null, type, M__Value(), 
			 root.getMode(mode), M__String());
		Label paramLabel = romFactory.newLabel(paramItem, M__Label());
		paramItem.setLabel(paramLabel);
		return paramItem;
	}

	/** To print the object module
     * @param Printwriter file handle
     * @return void
     */
    public void publishSOM(PrintWriter pw){
		id.publishSOM(pw);
		pw.print(":: ");
		if(mode != null){
			mode.publishSOM(pw);
		}
		if(typeName != null) {
		typeName.publishSOM(pw);}
    } 
		
	/** To print the PVS file for SOCCER -added by makarand on:  Mar 8 2000
     * @param Printwriter file handle
     * @return void
     */
    public void publishPVS(PrintWriter pw, SOM_List prelist, SOM_List
						   inplist, SOM_List outlist){ 
		try{
			int idmode;
			String tickid;
			/*id.publishSOM(pw);
			  pw.print(": ");
			  typeName.publishSOM(pw);
			  pw.println("");*/
			if(mode != null){
				idmode = mode.getMode();
				switch(idmode){
				case 0:
					// input mode
					prelist.addElement((String)("isinput(" + id + ")"));
					inplist.addElement("( |" + id.toString() + "| |" 
									   + typeName.toString() + "| )");
					id.publishSOM(pw);
					pw.print(": ");
					typeName.publishSOM(pw);
					pw.println("");
					break;
				case 1:
					// output mode
					//prelist.addElement((String)("isinput(" + id + ")"));
					prelist.addElement((String)("isoutput(" + id + "_)"));
					/* id can be a tick variable so declaring id_ as PVS
					 * variable of same type*/ 
					tickid = id.getIdentifier() + "_" ;
					//outlist.addElement("( |" + id.toString() + "| " 
					//					 + "|" + typeName.toString() + "| )");
					outlist.addElement("( |" + tickid + "| "
									   + " |"+ typeName.toString() + "| )");
					pw.print(id.getIdentifier() + ": ");
					typeName.publishSOM(pw); 
					pw.println("\n");
					pw.print(tickid + " : ");
					typeName.publishSOM(pw);
					break;
				case 2:
					// input output mode
					//prelist.addElement((String)("isoutput(" + id + ")"));
					prelist.addElement((String)("isinput(" + id + ")"));
					prelist.addElement((String)("isoutput(" + id + "_)"));
					//inplist.addElement(id.toString());
					inplist.addElement("( |" + id.toString() + "| |" 
									   + typeName.toString() + "| )");
					//outlist.addElement(id.toString());
					// - Not needed as this is an output variable
					/* id IS A a tick variable so declaring id_ as PVS
					 * variable of same type*/ 
					tickid = id.getIdentifier() + "_" ;
					//outlist.addElement(tickid);
					outlist.addElement("( |" + tickid + "| |"
									   + typeName.toString() + "| )");
					id.publishSOM(pw);
					pw.print(": ");
					typeName.publishSOM(pw);
					pw.println("");
					pw.print(tickid + " : ");
					typeName.publishSOM(pw);
					pw.println("");
					break;
				default:
					System.out.println("Error: Unknown identifier type: " 
									   + idmode);
					break;
				}
			}
			else{
				id.publishSOM(pw);
				pw.print(":");
				typeName.publishSOM(pw);
			}
		}
		catch(Exception e)
			{System.out.println("Error in SOM_InterfaceItem.publishPVS" + e);}
	}
}
		

