//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_LabelledExpression.java,v 1.21 2001/10/01 18:46:09 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_LabelledExpression.java,v $
//  Revision 1.21  2001/10/01 18:46:09  cvs
//  When parsing, type info is only printed out when printInfo flag in
//  Sldl.java is set to true. Other changes include tabulation set to 2.
//  -- Cindy
//
//  Revision 1.20  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.19  2000/12/07 01:23:19  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes
//  to the other classes 
//  --srinivas
//
//  Revision 1.18  2000/11/18 00:18:27  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.17  2000/11/06 21:58:39  cvs
//  Added the buildROM method (which was previously forgotten) to this file.
//  -- Cindy
//
//  Revision 1.16  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the
//  terms(SOM_LabelledExpression) and it returns a ROM Expression. 
//  --srinivas
//
//  Revision 1.15  2000/10/04 03:05:25  cvs
//  Changed some of the meta functions and the facetDeclaration and
//  componentDeclration no longer implement the design unit.This is
//  because all the files should be a package now 
//  --srinivas
//
//  Revision 1.14  2000/08/01 22:48:51  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.13  2000/07/14 23:01:23  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.12  2000/06/22 17:22:44  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.11  2000/05/19 00:04:00  cvs
//  restoring the files after the file system had crashed
//  --srinivas
//
//  Revision 1.11  2000/05/06 00:14:53  cvs
//   The Meta Functions for the term items have been changed to the
//   class SOM_LabelledExpression .The M__Value and M__String have
//   been implemented for the AssociationListing.The meta functions
//   for the declaration items are also implemented in the
//   SOM_DeclarationStatement 
//  --srinivas
//
//  Revision 1.10  2000/05/05 19:28:32  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.9  2000/04/06 19:13:22  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy 
//
//  Revision 1.8  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.7  2000/02/10 00:37:24  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  2000/02/08 18:42:34  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.5  1999/11/15 22:59:15  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/15 03:16:28  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:27  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/05 23:43:10  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:41  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:50:46  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.3  1999/07/14 23:23:56  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/06/30 22:20:09  rkamath
//  Compeleted the object model coding for the expression grammar
//  The expression objects now inherit from a AbstractExpression class
//
//  Revision 1.1  1999/06/27 17:33:33  rkamath
//  Added some of the object model for the Expression grammar and a
//  few changes to the other files..
//
//
//=============================================================================
package SOM;
import util.List;
import util.Link;
import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;
import java.lang.NullPointerException;
import java.io.Serializable;
import java.lang.String;
import java.io.*;

/** This represents the <B>LabelledExpression</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_LabelledExpression
extends SOM_AbstractExpression
implements SOM_Factor {

  private SOM_AbstractExpression expression;
  private SOM_Label label;
	
  /** The primary Constructor
   * @return Not applicable
   */
  public SOM_LabelledExpression() {
    super();
  }

  /** To set the label
   * @param labelIn The SOM_Label to be set
   * @return void
   */
  public void setLabel(SOM_Label labelIn) {
    label = labelIn;
  }

  /** To get the label
   * @return The SOM_Label 
   */
  public SOM_Label getLabel() {
    return label;
  }

  /** To set the expression
   * @param expressionIn the SOM_Expression to be set
   * @return void
   */
  public void setExpression(SOM_Expression expressionIn) {
    expression = expressionIn;
  }

  /** To get the expression
   * @return the SOM_Expression 
   */
  public SOM_Expression getLabelExpression() {
    return (SOM_Expression)expression;
  }

  /** To do the semcheck. Overrides that in SOM_AbstractExpression
   * Note that it assumes a semChecked expression - consistent with
   * the semChecks elsewhere..
   * should the label's typeList also be set here? Better to do it in
   * the grammar file!
   * @return void
   */
  public void semCheck() {
    setTypeList(expression.getTypeList());
  }

  /** Overrides the resolveFinalType from SOM_AbstractExpression. This is
   * called by the external world. 
   * @return void
   */
  public void resolveFinalType() {
    expression.resolveFinalType();
    setResolvedType(expression.getResolvedType());
  }

  /** overrides the resolveFinalType from SOM_AbstractExpression.
   * @param typeIn The SOM_TypeExpression which is the expected final type
   * @return void
   */
  public void resolveFinalType(SOM_TypeDeclaration typeIn) {
    expression.resolveFinalType(typeIn);
    setResolvedType(expression.getResolvedType());
  }

  /** Overrides the getNames from SOM_AbstractExpression
   * @return SOM_DeclarationList with all the names and their declarations
   */
  public SOM_DeclarationList getNames() {
    return expression.getNames();
  }

  //added by srinivas on Oct 24th 2000
  /*The Meta Function to get the value of the item
   *@param BaseObject parent
   *@param Universe root
   * @return Expression
   */
  public Expression M__Value(Factory romFactory, BaseObject parent,
			     Universe root){
    if (expression instanceof SOM_Expression){
      return
	((SOM_Expression)expression).
	M__Value(romFactory,(BaseObject)parent, root);
    }
    return null;
  }
	
  //added by srinivas on may6th 2000
  /*The Meta Function to get the string of the term item
   *@params void
   *@return String
   */
  public String M__String(){
    try{
      String labelString = new String();
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw,true);
      publishSOM(pw);
      pw.flush();
      pw.close();
      labelString = sw.toString();
      sw.close();
      return labelString;
    }
    catch (Exception e) {
      System.out.println(e);
      return (String)null;
    }
  }
	
  /** The buildROM method for this high-level item
   * @param romFactory, Factory object used to create objects in ROM
   * @param root, Universe object 
   * @param parentIn, BaseObject parent of item
   * @return LabelledItem, corresponding to this labelledExpression
   */
  public LabelledItem buildROM(Factory romFactory, Universe root, 
			       BaseObject parentIn) {
    LabelledItem termItem = romFactory.newLabelledItem
      (parentIn, null, null, null, M__String());
    Label termLabel = romFactory.newLabel(termItem, M__Label().toString());
    termItem.setLabel(termLabel);
    Expression termValue = M__Value(romFactory, termItem, root);
    termItem.setValue(termValue);
    String str = M__Type();
    if (str != null){
      Type termType = root.getType(M__Type());
      termItem.setType(termType);
    }else {
      termItem.setType(null);
    }
    return termItem;
  }

  //added by srinivas on the may6th 2000
  /*The Meta Function that gives the label of term item
   *@params void
   *@return SOM_Label
   */
  public SOM_Identifier M__Label(){
    return label.getIdentifier();
  }
	
  //added by srinivas on the may6th 2000
  /*The Meta Function to get the type of the term item
   *@params void
   *@return String
   */
  public String M__Type(){
    String str = null;
    SOM_TypeDeclaration type = null;
    try{
      type = expression.getResolvedType();
    }catch (Exception e){
      System.out.println(e);
    }
    if (type != null)
      str = type.toString();
    return str;
  }
	
  /** The publishSOM routine for this class
   * @return void
   */
  public void publishSOM(PrintWriter pw)
  {
    label.publishSOM(pw);
    expression.publishSOM(pw);
  }
	
  /** The publishPVS routine for this class
   * @return void
   * To be implemented as needed */
  public void publishPVS(PrintWriter pw) {
    publishSOM(pw);
  }
}

