//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_LetExpression.java,v 1.9 2000/11/18 00:18:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_LetExpression.java,v $
//  Revision 1.9  2000/11/18 00:18:28  cvs
//  Changed the M__Value method so that it takes ROM.Factory as a parameter
//  --srinivas
//
//  Revision 1.8  2000/10/30 22:13:24  cvs
//  The M__Value  function is implemented for the terms(SOM_LabelledExpression) and it returns a ROM Expression.
//  --srinivas
//
//  Revision 1.7  2000/08/01 22:48:51  cvs
//  added publishPVS templates in each of the expression subclasses.
//  Currently SOCCER supports PVS translation of "%" and "-" unary operator,
//  other translations can be done in these publishPVS templates as and when
//  needed - makarand.
//
//  Revision 1.6  2000/06/22 17:22:44  cvs
//  I modified the grammar to remove an error during parsing for record and tuple.
//  -- Cindy
//
//  Revision 1.5  2000/04/06 19:13:23  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.4  2000/03/23 00:22:48  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.3  2000/03/08 01:07:19  cvs
//  modified the grammar and the class file due to change in the let
//  expression.the test cases have also been changed accordingly 
//  --srinivas
//
//  Revision 1.2  2000/02/13 01:15:54  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.1  2000/02/10 03:56:29  cvs
//  Modified the grammar for incorporating records, tuples and "let"
//  expressions. Also populated the object model with these above
//  constructs, and completed the pretty printing for them. 
//  - krishna
//
//
//
//=============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;

/** This represents the <B>LetExpression</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_LetExpression
extends SOM_AbstractExpression
implements SOM_Factor {
	// This represents the <letExpression> object, for which the grammar is
	// <letExpression> ::== 
	//     <LET> <letDeclarationStatement> (, <letDeclarationStatement>)*
	//     <IN> <expression>
	
	SOM_LetDeclarationStatementListing letDeclStmtList;
	SOM_Expression expr;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_LetExpression() {
		super();
	}

  /** To set the LetDeclarationStatementList
	 * @param letDeclStmtListIn The SOM_LetDeclarationStatementListing
	 * to be set 
	 * @return void
	 */
	public void setLetDeclarationStatementListing
		(SOM_LetDeclarationStatementListing letDeclStmtListIn) { 
			letDeclStmtList = letDeclStmtListIn;
	}

	/** To set the Expression
	 * @param exprIn The SOM_Expression to be set
	 * @return void
	 */
	public void setExpression(SOM_Expression exprIn) {
		expr = exprIn;
	}

	/** To do the sem check. Overrides the method in abstract expression.
	 * @return void
	 */
	public void semCheck() {
		if (expr.getTypeList() == null)
			expr.semCheck();
	}

	/** To unfold the overloading resolution algorithm. Assumes semCheck() 
	 * was called and it successfully set the typeList. 
	 * Overriddes the method in abstract expression
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if the type cannot be resolved
	 */
	public void resolveFinalType() {
		expr.resolveFinalType();
		setResolvedType(expr.getResolvedType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		expr.resolveFinalType(typeIn);
		setResolvedType(expr.getResolvedType());
	}

	/** Overrides the getNames from SOM_AbstractExpression
	 * @return SOM_DeclarationList with all the names and their declarations
	 */
	public SOM_DeclarationList getNames() {
		SOM_DeclarationList toReturn = expr.getNames();
		return toReturn;
	}
	
	//added by srinivas on Oct 24th 2000
    /*The Meta Function to get the value of the item
     *@param BaseObject parent
	 *@param Universe root
     * @return Expression
     */	
	public Expression M__Value(Factory romFactory,BaseObject parent, 
							   Universe root){
		return null;
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw) {
	    pw.print("LET ");
	    pw.print("( ");
	    letDeclStmtList.publishSOM(pw);
	    pw.print(") ");
	    pw.print(" IN ");
	    expr.publishSOM(pw);
	}

	/** dummy functions added by krishna and makarand on Mar 7, 2000
     */
	public SOM_Label getLabel() {
		return (SOM_Label)null;
	}
	
	public SOM_Expression getLabelExpression() {
		return (SOM_Expression)null;
	}
		
		/** The publishPVS routine for this class
		 * @return void
		 * To be implemented as needed
		 */
		public void publishPVS(PrintWriter pw) {
				publishSOM(pw);
		}
}




