//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_MonoadicOperator.java,v 1.10 2001/03/12 23:54:39 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_MonoadicOperator.java,v $
//  Revision 1.10  2001/03/12 23:54:39  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.9  2000/07/11 00:11:13  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are 
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.8  2000/06/22 17:22:45  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.7  2000/02/10 00:37:26  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  1999/11/17 18:31:55  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.5  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.4  1999/11/15 03:16:29  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.3  1999/11/13 20:01:28  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.2  1999/11/05 23:43:11  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//  Revision 1.1  1999/10/27 20:43:43  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:00  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.6  1999/07/28 01:59:43  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.5  1999/07/21 00:28:15  rkamath
//  Small changes and consistency corrections...
//
//  Revision 1.4  1999/07/16 21:50:32  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.3  1999/07/14 23:24:00  rkamath
//  Added a generic semCheck function in SOM_AbstractExpression
//  All the other "expressionTail"s now inherit from SOM_AbstractExpressionTail
//  So also for the "expressionTailList"s
//  SOM_Operator now inherits from SOM_Function
//
//  Revision 1.2  1999/07/13 19:29:09  rkamath
//  Moved the List and ListEnumeration code to a new util directory
//  Mode the operators as children of AbstractExpression
//  Introduced the FunctionDeclaration and made OperatorDeclaration its child
//
//  Revision 1.1  1999/06/25 18:07:16  rkamath
//  Added the Operators and the Terminal interface (changed from a abstract class
//  to an interface)
//
//
//=============================================================================
package SOM;

import SYM.SYM_StaticFunctions;
import SYM.SYM_UnresolvableException;

import util.ListEnumeration;

/** This represents the <B>MonoadicOperator</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public abstract class SOM_MonoadicOperator
extends SOM_Operator 
implements SOM_Terminal {

	protected SOM_AbstractExpression operand;

	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_MonoadicOperator() {
		super();
	}

	/** To set the operand
	 * @param operandIn The SOM_AbstractExpression to be set as the operand
	 * @return void
	 */
	public void setOperand(SOM_AbstractExpression operandIn) {
		operand = operandIn;
	}

	/** Overrides the semCheck in SOM_AbstractExpression
	 * @return void
	 */
	public void semCheck() {
		//System.out.println("Doing semCheck() in " + getClassName());
		SOM_TreeNodeList typeListNow = 
			SYM_StaticFunctions.buildTree
			(getDeclarations(), new SOM_ArgumentList(operand.getTypeList()));
		if (typeListNow.isEmpty())
			throw new SYM_UnresolvableException
				("No declaration with the correct argument types found for " + 
				 getClassName());
		setTypeList(typeListNow);
	}

	/** Overrides the resolveFinalType from SOM_AbstractExpression. This is
	 * called by the external world.
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if a final type cannot be
	 * resolved
	 */
	public void resolveFinalType() {
		// The final overload resolution should have resulted in a single 
		// type.
		SOM_TreeNodeList typeList = getTypeList();
		if (typeList.countElements() != 1) 
			throw  new SYM_UnresolvableException
			  ("Cannot resolve final type for expression in MonoadicOperator");
		
		// else the final type is the sole element of the typeList.
		SOM_TreeNode onlyNode = (SOM_TreeNode)typeList.first();
		setResolvedType(onlyNode.getResultType());

		ListEnumeration childEnum = onlyNode.children();
		operand.resolveFinalType
			(((SOM_TreeNode)childEnum.nextElement()).getResultType());
	}

	/** overrides the resolveFinalType from SOM_AbstractExpression.
	 * @param typeIn The SOM_TypeExpression which is the expected final type
	 * @return void
	 * @exception SYM.SYM_UnresolvableException if a final type cannot be
	 * resolved
	 */
	public void resolveFinalType(SOM_TypeDeclaration typeIn) {
		// typeIn should be one of the types in typeList

		ListEnumeration typeEnum = getTypeList().elements();
		SOM_TreeNode currNode;
		SOM_TypeDeclaration currType;

		//System.out.println("Fresh TCC Generation");

		while (typeEnum.hasMoreElements()) {
			//System.out.println("New Cycle");

			currNode = (SOM_TreeNode)typeEnum.nextElement();
			currType = currNode.getResultType();

			try {
				if (SOM_TypeDeclaration.resolveFinalType(currType, typeIn)) {
					setResolvedType(currType);
					ListEnumeration childEnum = currNode.children();
					operand.resolveFinalType
						(((SOM_TreeNode)childEnum.nextElement()).
						 getResultType());
					//System.out.println("End Fresh TCC Generation");
					return;
				}
			} catch (SYM_UnresolvableException  except) {
				// means that some resolution failed
				// go to the next type enum
				//System.out.println("Flush previous cycle");
				//System.out.println("New Cycle");
			}
		}

		//System.out.println("End Fresh TCC Generation");

		// typeIn was not found in the overload resovled types.
		throw  new SYM_UnresolvableException
			("Cannot resolve final type for expression in MonoadicOperator");
	}
}
