package SOM;

import util.ListEnumeration;
import java.io.*;
import java.io.PrintWriter;

import ROM.*;

/** This represents the <B>ObserverItem</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_ObserverItem
    extends SOM_BaseClass{

    private SOM_Identifier id;
    private SOM_Mode mode = null;
    private SOM_InterfaceListing interList = null;

    // An ObserverItem should have a well defined and resolved type!
    private SOM_Name typeName;
    private SOM_TypeDeclaration type;

    // Generated observer function parts
    private SOM_Expression genFncInputs;
    private SOM_Expression genFncCall;

    /** The primary Constructor
     * @return Not applicable
     */
    public SOM_ObserverItem() {
	super();
    }
    
    /** To set the idListing
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn){
	id = idIn;
    }

    /** To retreive the id
     * @return The id
     */
    public SOM_Identifier getIdentifier() {
	return id;
    }

    /** To set the interfaceListing
     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
	interList = interfaceIn;
    }
    
    /** To get the interfaceListing
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
	return interList;
    }
    
    /** To set the mode
     * @param modeIn The SOM_Mode object to be set
     * @return void
     */
    public void setMode(SOM_Mode modeIn) {
	mode = modeIn;
    }
    
    /** To get the Mode
     * @return the SOM_Mode corresponding to the mode
     */
    public SOM_Mode getMode() {
	return mode;
    }
    
    /** To set the typeName
     * @param typeNameIn The SOM_Name corresponding to the type
     * @return void
     */
    public void setTypeName(SOM_Name typeNameIn) {
	typeName = typeNameIn;
    }

    /** To get the typeName
     * @return The SOM_Name corresponding to the type
     */
    public SOM_Name getTypeName() {
	return typeName;
    }

    /** To set the type
     * @param typeIn The SOM_Declaration corresponding to the type
     * @return void
     */
    public void setType(SOM_TypeDeclaration typeIn) {
	type = typeIn;
    }

    /** To get the type
     * @return The SOM_Declaration corresponding to the type
     */
    public SOM_TypeDeclaration getType() {
	return type;
    }

    /** To set the generated function input value
     * @param valIn The SOM_Expression corresponding to the input value
     * @return void
     */
    public void setGenFncInputs(SOM_Expression valIn) {
	genFncInputs = valIn;
    }

    /** To get the generated function input value
     * @return The SOM_Expression corresponding to the input value
     */
    public SOM_Expression getGenFncInputs() {
	return genFncInputs;
    }

    /** To set the generated function call
     * @param callIn The SOM_Expression corresponding to the function call
     * @return void
     */
    public void setGenFncCall(SOM_Expression callIn) {
	genFncCall = callIn;
    }

    /** To get the generated function call
     * @return The SOM_Expression corresponding to the function call
     */
    public SOM_Expression getGenFncCall() {
	return genFncCall;
    }
	
    /** To create an input value call for a function
     * @param inId The SOM_Identifier corresponding to the input value
     * @return Expression
     */
    public SOM_Expression createInputValue(SOM_Identifier inId) {
	SOM_Name name;
	name = (SOM_Name)inId;
	SOM_AtTickExpression InputVariable = new SOM_AtTickExpression();
	InputVariable.setName(name);
	SOM_ExpTerm obsExpTerm = new SOM_ExpTerm();
	obsExpTerm.setFactor((SOM_Factor)InputVariable);
	SOM_Term obsTerm = new SOM_Term();
	obsTerm.setExpTerm(obsExpTerm);
	SOM_SimpleExpression obsSimpleExpr = new SOM_SimpleExpression();
	obsSimpleExpr.setTerm(obsTerm);
	SOM_RelationalTermExpression obsRelTermExpr = new SOM_RelationalTermExpression();
	obsRelTermExpr.setSimpleExpression(obsSimpleExpr);
	SOM_AndExpression obsAndExpr = new SOM_AndExpression();
	obsAndExpr.setRelationalTermExpression(obsRelTermExpr);
	SOM_ImpliesExpression obsImpliesExpr = new SOM_ImpliesExpression();
	obsImpliesExpr.setAndExpression(obsAndExpr);
	SOM_TermExpression obsTermExpr = new SOM_TermExpression();
	obsTermExpr.setImpliesExpression(obsImpliesExpr);
	SOM_Expression obsExpr = new SOM_Expression();
	obsExpr.setTermExpression(obsTermExpr);
	return obsExpr;
    }

    /** To create a function
     * @param callVariables The SOM_Expression corresponding to the input value
     * @param fncId The SOM_Identifier corresponding to the function id
     * @return Expression
     */
    public SOM_Expression createFunctionCall(SOM_Expression callVariables, SOM_Identifier fncId) {
	SOM_AssociationListing obsAssocList = new SOM_AssociationListing();
	obsAssocList.addToList(callVariables);
	SOM_FormalName obsFormalName = new SOM_FormalName();
	obsFormalName.setName((SOM_Name)fncId);
	obsFormalName.setAssociationListing(obsAssocList);
	SOM_AtTickExpression InputVariable = new SOM_AtTickExpression();
	InputVariable.setName(obsFormalName);
	SOM_ExpTerm obsExpTerm = new SOM_ExpTerm();
	obsExpTerm.setFactor((SOM_Factor)InputVariable);
	SOM_Term obsTerm = new SOM_Term();
	obsTerm.setExpTerm(obsExpTerm);
	SOM_SimpleExpression obsSimpleExpr = new SOM_SimpleExpression();
	obsSimpleExpr.setTerm(obsTerm);
	SOM_RelationalTermExpression obsRelTermExpr = new SOM_RelationalTermExpression();
	obsRelTermExpr.setSimpleExpression(obsSimpleExpr);
	SOM_AndExpression obsAndExpr = new SOM_AndExpression();
	obsAndExpr.setRelationalTermExpression(obsRelTermExpr);
	SOM_ImpliesExpression obsImpliesExpr = new SOM_ImpliesExpression();
	obsImpliesExpr.setAndExpression(obsAndExpr);
	SOM_TermExpression obsTermExpr = new SOM_TermExpression();
	obsTermExpr.setImpliesExpression(obsImpliesExpr);
	SOM_Expression obsExpr = new SOM_Expression();
	obsExpr.setTermExpression(obsTermExpr);
	return obsExpr;
    }

    /** To create interfaceListing
     * @param idIn the String representation of the id
     * @param nameIn the SOM_Name of the id type
     * @return SOM_InterfaceListing the interface created
     */
    public SOM_InterfaceListing createInterface(String idIn, SOM_Name nameIn) {
	SOM_Identifier functGenInputId = new SOM_Identifier(idIn);
	SOM_InterfaceItem functGenInputItem = new SOM_InterfaceItem();
	functGenInputItem.setIdentifier(functGenInputId); 
	functGenInputItem.setTypeName(nameIn); 
	SOM_InterfaceListing genInterList = new SOM_InterfaceListing();
	genInterList.addToInterfaceItemList(functGenInputItem);
	return genInterList;
    }

    /** To convert a quantified expression to an expression
     * @param qExprIn the SOM_QuantifiedExpression to be converted
     * @return SOM_Expression the expression converted
     */
    public SOM_Expression convertQExpr2Expr(SOM_QuantifiedExpression qExprIn) {
	SOM_ExpTerm obsExpTerm = new SOM_ExpTerm();
	obsExpTerm.setFactor((SOM_Factor)qExprIn);
	SOM_Term obsTerm = new SOM_Term();
	obsTerm.setExpTerm(obsExpTerm);
	SOM_SimpleExpression obsSimpleExpr = new SOM_SimpleExpression();
	obsSimpleExpr.setTerm(obsTerm);
	SOM_RelationalTermExpression obsRelTermExpr = new SOM_RelationalTermExpression();
	obsRelTermExpr.setSimpleExpression(obsSimpleExpr);
	SOM_AndExpression obsAndExpr = new SOM_AndExpression();
	obsAndExpr.setRelationalTermExpression(obsRelTermExpr);
	SOM_ImpliesExpression obsImpliesExpr = new SOM_ImpliesExpression();
	obsImpliesExpr.setAndExpression(obsAndExpr);
	SOM_TermExpression obsTermExpr = new SOM_TermExpression();
	obsTermExpr.setImpliesExpression(obsImpliesExpr);
	SOM_Expression obsExpr = new SOM_Expression();
	obsExpr.setTermExpression(obsTermExpr);
	return obsExpr;
    }
    
    /* To get the label of the interface item
     * @param void
     * @return String
     */
    public String M__Label(){
	return id.toString();
    }
	
    /* To get the string of the object
     * @param void
     * return String
     */
    public String M__String(){
	try{
	    String str;
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    publishSOM(pw);
	    pw.flush();
	    pw.close();
	    str=sw.toString();
	    sw.close();
	    return str;
	}catch(Exception e){
	    System.out.println(e);
	    return null;
	}
    }
	
    /* To get the type of the identifier in the Observer Item.
     * @param void
     * @return String
     */
    public String M__Type(){
	try{
	    String str;
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    typeName.publishSOM(pw);
	    pw.flush();
	    pw.close();
	    str = sw.toString();
	    sw.close();
	    return str;
	}catch(Exception e){
	    System.out.println(e);
	    return null;
	}
    }
	
    /* To get the value of the Interface Item.
     * @param void
     * @return Expression
     */
    public Expression M__Value(){
	return null;
    }
		
    /** The buildROM method for this high-level item
     * @param romFactory, Factory object used to create objects in ROM
     * @param root, Universe root of the ROM
     * @param parentIn, BaseObject parent of item
     * @return LabelledItem, corresponding to this interface item
     */
    public LabelledItem buildROM(Factory romFactory, Universe root, 
				 BaseObject parentIn) {	
	Type type = root.getType(M__Type());
	if (type == null)
	    if (parentIn instanceof Facet)
		type = ((Facet)parentIn).getType(M__Type());
	ParameterItem paramItem = romFactory.newParameterItem
	    (parentIn, null, type, M__Value(), 
	     root.getMode(mode), M__String());
	Label paramLabel = romFactory.newLabel(paramItem, M__Label());
	paramItem.setLabel(paramLabel);
	return paramItem;
    }

    /** To print the object module
     * @param Printwriter file handle
     * @return void
     */
    public void publishSOM(PrintWriter pw){
		id.publishSOM(pw);
		pw.print(":: ");
		if(mode != null){
			mode.publishSOM(pw);
		}
		typeName.publishSOM(pw);
    } 


    /** publishFunctionGeneration routine to print object module
     * Differs from publishSOM because the function is generated, not given
     * @param PrintWriter
     * @return void
     */
    public void publishFunctionGeneration(PrintWriter pw) {
	id.publishSOM(pw);
	if (interList != null) {
	    interList.publishSOM(pw);
	}
	pw.print("::");
	pw.print(getType().getName().toString());
    }

    /** To get the string representation of the function generated
     * @return void
     */
    public String M__FncGenString(){
	try{
	    String str = new String();
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    publishFunctionGeneration(pw);
	    pw.flush();
	    pw.close();
	    str = sw.toString();
	    sw.close(); 
	    return str;
	}
	catch (Exception e) {
	    System.out.println(e);
	    return (String)null;
	}
    }

    /** The function that creates the ROM.Expression for the function
     * @param Factory,BaseObject,Universe
     * @return ROM.Expression(ROM.Facet)
     */
    public Expression M__Value(Factory romFactory,
			       BaseObject parentIn, Universe root){
	// I have only implemented function generation, the expression to this
	// function has NOT been implemented!!!
	// Will be null for now!!
	SOM_Expression expr = null;
	
	if (getInterfaceListing() != null){
	    Function function= romFactory.newFunction(parentIn);
	    ListEnumeration interList = getInterfaceListing().interfaces();
	    while (interList.hasMoreElements()){
		SOM_InterfaceItem interItem = (SOM_InterfaceItem)
		    interList.nextElement();
		function.addInterfaceItem(interItem.buildROM
					  (romFactory, root, function));
	    }
	    if (expr != null)
		function.setExpression
		    (expr.M__Value(romFactory, function, root));
	    return (Expression)function;
	}

	else
	    return null;
    }

    /* To get the type of the function generation
     * TO BE COMPLETED WHEN TYPE CHECKING IS FIXED
     * RETURN NULL FOR NOW
     * @param void
     * @return String
     */
    public Type M__Type(BaseObject parent, BaseObject
			newParent,Universe root, String identifier,
			Expression value) {
	Type type = null;
	return type;
    }

    // Two buildROMS!  One for the observer listing and one for the function generation
    /** The buildROM method for this high-level item
     * VOID BECAUSE IT IS GENERATION FUNCTIONS AND DIRECTLY ADDING TO ROOT!
     * @param ROMFactory, Factory object used to create objects in ROM
     * @return LabelledItem, corresponding to this facet  
     */
    public void buildFncGenROM(Factory romFactory, BaseObject parentIn, Universe
			       root) {
	LabelledItem declItem;
	declItem = romFactory.newLabelledItem(parentIn, null, null, null, M__String());
	Label declLabel =  romFactory.newLabel(declItem, id.toString());
	declItem.setLabel(declLabel);
	Expression declValue = M__Value(romFactory, declItem, root);
	declItem.setValue(declValue);
	Type type = M__Type(parentIn, declItem, root, id.toString(),
			    declValue); 
	declItem.setType(type);

        // Added by Morel, July 14, 2001
	// Convert parent back to package or facet then manually add the generated fnc
	ROM.Facet packORfacetValue = (ROM.Facet)parentIn;
	packORfacetValue.addItem(declItem);
    }

		
    /** To print the PVS file for SOCCER
     * @param Printwriter file handle
     * @return void
     */
    public void publishPVS(PrintWriter pw, SOM_List prelist, SOM_List
						   inplist, SOM_List outlist){
	
    }
}
		

