//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_PackageDeclaration.java,v 1.18 2001/08/28 23:27:01 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_PackageDeclaration.java,v $
//  Revision 1.18  2001/08/28 23:27:01  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.17  2001/03/26 19:04:58  cvs
//  Completed the implementation for the change in export clauses. Also
//  added code in SOM_PackageDeclaration to take care of use clauses (case
//  was missing for when these clauses are found in declaration part of
//  packages).
//  -- Cindy
//
//  Revision 1.16  2001/03/21 00:05:05  cvs
//  Added the "design" mode for parameters (SOM_Design.java). Changed the
//  semantics for export clauses. Removed the expression member from
//  SOM_FacetDeclaration.java and SOM_PackageDeclaration.java. Added code
//  to catch error when type-checking fails. Removed the redundant methods
//  getExportList() from SOM_FacetDeclaration.java,
//  SOM_PackageDeclaration.java and SOM_ComponentDeclaration.java.
//  -- Cindy
//
//  Revision 1.15  2001/02/07 20:51:08  cvs
//  There was an error causing an infinite loop when doing label lookup in
//  use clauses. Modifications were made in M__Value and buildROM methods
//  to remove error.
//  -- Cindy
//
//  Revision 1.14  2000/12/04 19:37:06  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  -- Cindy
//
//  Revision 1.13  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.12  2000/10/22 18:42:02  cvs
//  Meta functions are added to the SOM_InterfaceItem class and the
//  meta function M__Domain is changed so that it returns a string and
//  not SOM_FacetExpression 
//  --srinivas
//
//  Revision 1.11  2000/09/30 17:57:15  cvs
//  Added a function in the package declaration and facet declaration
//  that retrieves all the identifiers that are exported. 
//  --srinivas
//
//  Revision 1.9  2000/07/14 23:01:24  cvs
//  Made changes to the meta functions and added some meta functions
//  to the identifier object. 
//  --srinivas
//
//  Revision 1.8  2000/06/22 17:22:45  cvs
//  I modified the grammar to remove an error during parsing for
//  record and tuple. 
//  -- Cindy
//
//  Revision 1.7  2000/05/18 18:03:46  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.7  2000/05/05 19:28:33  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.6  2000/04/21 02:09:39  cvs
//  Made changes to the publishSOM routine so that the ObjectModel
//  file is correct 
//  --srinivas
//
//  Revision 1.5  2000/04/09 01:50:04  cvs
//   The identifier had a space printed in the publishSOM which was
//   not required for the tick and the at operators so changing the
//   classes and the grammar had the tick and single quote
//   interchanged so modifing the grammar also 
//  --srinivas
//
//  Revision 1.4 2000/04/08 02:21:58 cvs 
//  The Meta Functions have been implemented and these are the files
//  that had to be modified.Addded some accessor methods to classes
//  like SOM_DeclarationStatement, SOM_Expression,
//  SOM_RecordTupleDeclarationStatement.The Meta functions for facet
//  terms and the term items are implemented --srinivas
//
//  Revision 1.3 2000/03/23 00:22:49 cvs 
//  Huge commit! Added all the essential accessor methods for the
//  following classes - Krishna and Makarand
//
//  Revision 1.2  2000/03/03 18:38:29  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.1  2000/02/13 01:15:55  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//
//
//=============================================================================
package SOM;

import util.Link;
import util.ListEnumeration;
import util.List;
import java.io.PrintWriter;
import java.lang.NullPointerException;
import java.io.*;
import ROM.*;

/** This represents the <B>PackageDeclaration</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public class SOM_PackageDeclaration
extends SOM_Declaration 
implements SOM_DesignUnit, SOM_PackageDeclarativeItem {
    
    /** The name of this Package - and other dataMembers */
    private SOM_Identifier id;
    private SOM_InterfaceListing interfaceList = null;
    private SOM_PackageDefinition packageDefinition = null;
    
    private transient SOM_DeclarationList domainList;
    
    /** The primary constructor
     * @param packageTypeIn The typeDeclaration corresponding to the
	 * "package type" 
     * @return Not applicable
     */
    public SOM_PackageDeclaration(SOM_TypeDeclaration packageTypeIn) {
		super();
		setKind(PACKAGE);
		setType(packageTypeIn);
    }
    
    /** To set the identifier
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
		id = idIn;
    }			 
    
    /** To set the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
		interfaceList = interfaceIn;
    }
    
    /** To get the interfaceListing - implements the function from
     * SOM_DesignUnit
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
		return interfaceList;
    }
    
    /** To set the domainList - implements the function from
     * SOM_DesignUnit
     * @param domainListIn The SOM_DeclarationList object to be set
     * @return void
     */
    public void setDomainList(SOM_DeclarationList domainListIn) {
		domainList = domainListIn;
    }
    
    /** To get the domainList - implements the function from
     * SOM_DesignUnit
     * @return The SOM_DeclarationList object for this designUnit
     */
    public SOM_DeclarationList getDomainList() {
		return domainList;
    }
    
    /** To set the packageDefinition
     * @param packageDefIn The SOM_PackageDefinition object to be set
     * @return void
     */
    public void setPackageDefinition(SOM_PackageDefinition packageDefIn){
		packageDefinition = packageDefIn;
    }
    
    /** To get the exports as a ListEnumeration 
     * @return ListEnumeration corresponding to the export clauses
     */
    public ListEnumeration exports() {
		SOM_ExportBlock exportBlock = packageDefinition.getExportBlock();
		if (exportBlock != null) 
			return exportBlock.exports();
		else 
			return null;
    }
    
    /** To get an enumeration of the parents' Declarations for this package
     * @return ListEnumeration representing the list of the parents' 
     * declarations for this package
     */
    public ListEnumeration parents() {
		try {
			SOM_DeclarationList parentList =
				packageDefinition.getParentDeclarationList(); 
			return parentList.elements();
		} catch (NullPointerException except) {
			return new ListEnumeration(null);
		}
    }
    
    /** To get the string representation of this object. Overriding the
     * method in SOM_Declaration 
     * @return String The "name" of the package corresponding to this
     * declaration 
     */
    public String toString() {
		return id.toString();
    }
    
    /** added by srinivas on sept27, 2000
     * To get the list of parameters passed to the package
	 * @param void
	 * @return ListEnumeration of objects of type SOM_InterfaceItem.
     */
    public ListEnumeration M__Params() {
		List parameters = new List();
		if (getInterfaceListing() !=null){
			ListEnumeration  interListEnum =
				getInterfaceListing().interfaces();
			
			SOM_InterfaceItem interItem;
			if (interListEnum != null){
				while (interListEnum.hasMoreElements()){
					interItem = (SOM_InterfaceItem)interListEnum.nextElement();
					
					if(interItem !=null)
						//String iden = interItem.getIdentifier().toString();
						parameters.addElement((Object)interItem);
				}
			}
		}
		if (parameters.isEmpty()){
			return null;
		}
		else{
			return parameters.elements();
		}
    }
    
    /** added by krishna on Mar 5, 2000
     * To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Terms() {
		return new ListEnumeration(null);
	}
    
    /** added by srinivas on Mar 27, 2000
     * To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Pvars() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
    }

    /** added by srinivas on Mar 29, 2000
     * To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Visible() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
    }

    /** added by srinivas on Mar 29, 2000
     * To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Types() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
    }

    /** added by srinivas on Mar 29, 2000
     * To get the desired object by traversal of SOM
     * to be implemented later
     */
    public ListEnumeration M__Items() {
		//System.out.println("Compo decl entered");
		//return getFacetDefinition().getFacetBody().items();
		return new ListEnumeration(null);
    }
    
    /** added by srinivas on Mar 29, 2000
	 * To get the domain of the package
	 * @param void
	 * @return String 
     */
    public String M__Domain() {
		try{
			if (packageDefinition !=null){
				String str = new String();
				StringWriter sw = new StringWriter();
				PrintWriter pw = new PrintWriter(sw,true);
				if (packageDefinition.getDomainExpression() != null)
					packageDefinition.getDomainExpression().publishSOM(pw);
				pw.flush();
				pw.close();
				str = sw.toString();
				sw.close(); 
				return str;
			}else return null;
		}catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }

    /*added by srinivas to get the list of the declaration items in
	 * the package.The return value is a list of objects of type
	 * SOM_ComponentDeclaration or SOM_FacetDeclaration or
	 * SOM_DeclarationStatement or SOM_UseClause
	 * @param void
	 * @return ListEnumeration 
	 */
    public ListEnumeration getDeclarationItems(){
		List declItems = new List();
		if (packageDefinition !=null){
			SOM_PackageDeclarativeBlock packDeclBlk =
				packageDefinition.getPackageDeclarativeBlock();
			if (packDeclBlk != null){
				ListEnumeration items = packDeclBlk.items();
				if (items != null){
					while (items.hasMoreElements()){
						declItems.addElement(items.nextElement());
					}
				}
			}
		}
		if (declItems.isEmpty())
			return null;
		else
			return declItems.elements();
    }

    /**added by srinivas on sept 25th, 2000
     *To get the label of the package
     */
    public SOM_Identifier M__Label(){
		return id;
    }
	
    /**added by srinivas to on sept25th, 2000 to get the string
     * @param void
     * @return String
     */
    public String M__String(){
		try{
			String str = new String();
			StringWriter sw = new StringWriter();
			PrintWriter pw = new PrintWriter(sw,true);
			publishSOM(pw);
			pw.flush();
			pw.close();
			str = sw.toString();
			sw.close(); 
			return str;
		}
		catch (Exception e) {
			System.out.println(e);
			return (String)null;
		}
    }
    /**added by srinivas on sept25th, 2000
     * @param void
     * @return String
     */
    public String M__Type(){
		String str = "package";
		return str;
    }
    
	// Added by srinivas on Oct 27th, 2000. This function returns the
	// ROM.Expression 
	/** The function that creates the ROM.Expression for the package
	 * item
	 * @param Factory, BaseObject, Universe
	 * @return ROM.Expression(ROM.Package)
	 */
	public Expression M__Value(Factory romFactory, BaseObject parent, 
							   Universe root){ 
		
		ROM.Package packageValue = (ROM.Package)parent;

		// ***** Populating the package value ***** //		
		// Populating the interface items
		ListEnumeration paramList = M__Params();
		SOM_InterfaceItem interItem;
		if (paramList != null) {
			while (paramList.hasMoreElements()) {
				interItem = (SOM_InterfaceItem)paramList.nextElement();
				packageValue.addInterfaceItem
					(interItem.buildROM(romFactory, root, packageValue));
			}
		}

		// Populating the declared items
		ListEnumeration declList = getDeclarationItems();
		SOM_PackageDeclarativeItem packDeclItem;
		// It is being assumed that a package will always have some
		// declared items
		ItemReferenceList useRefList = packageValue.getUseClause();
		while (declList.hasMoreElements()) {
			packDeclItem = (SOM_PackageDeclarativeItem)
				(declList.nextElement());
			if (packDeclItem instanceof SOM_UseClause) {
				ListEnumeration useList = 
					((SOM_UseClause)packDeclItem).selectedNames();
				SOM_SelectedName useName;
				FacetReference useRef;
				LabelledItem packageItem = null;
							
				if (useRefList == null) {
					useRefList = new ItemReferenceList();
					packageValue.setUseClauseList(useRefList);
				}

				while (useList.hasMoreElements()) {
					useName = (SOM_SelectedName)useList.nextElement();
					try {
						// Deserialization of corresponding package ROM
						FileInputStream in = new FileInputStream
							(useName.toString() + "_ROM.ser");
						ObjectInputStream objin = new ObjectInputStream(in);
						System.out.println(" Deserializing package ROM: " +
										   useName.toString());
						packageItem = (LabelledItem)objin.readObject();
						root.addItem(packageItem);
						useRef = new FacetReference(root, packageItem);
						useRefList.add(useRef);
						System.out.println(" ... Done");
						objin.close();
					} catch(Exception e){
						System.out.println(e + " - file" +
										   useName.toString() + "_ROM.ser");
					}
				}
			}
			else {
				packageValue.addItem(packDeclItem.buildROM
									 (romFactory, root, packageValue));
			}		
		}

		// Populating the domain
		Label domainLabel = romFactory.newLabel
			((Item)packageValue.getParent(), M__Domain()); 
		packageValue.setDomain(domainLabel);

		// Populating the export list
		ListEnumeration exportList = exports();
		SOM_Identifier exportId;
		if (exportList != null) {
			if (!exportList.hasMoreElements()) {
				packageValue.addExportLabel
					(romFactory.newLabel((Item)packageValue.getParent(),
										 "ALL"));
			} else { 		  
				while (exportList.hasMoreElements()) {
					exportId = (SOM_Identifier)exportList.nextElement();
					packageValue.addExportLabel
						(romFactory.newLabel((Item)packageValue.getParent(), 
											 exportId.toString()));
				}
			}
		}
		return (Expression)packageValue;
	}

	/** The buildROM method for this high-level item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe object 
	 * @param parentIn, BaseObject parent of item
	 * @return LabelledItem, corresponding to this package
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn) {
		LabelledItem packageItem = romFactory.newLabelledItem
			(parentIn, null, null, null, M__String());
		Label packageLabel = romFactory.newLabel(packageItem, toString());
		packageItem.setLabel(packageLabel);
		ROM.Package packageValue = romFactory.newPackage(packageItem);
		((PackageType)root.getType("package")).addOneElement(packageValue);
		packageItem.setValue(packageValue);
		// The object packageValue is being modified directly but not created
		M__Value(romFactory, packageValue, root);
		Type packageType = root.getType("package");
		packageItem.setType(packageType);

		return packageItem;
	}

	/** The buildROM method for this high-level item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe object 
	 * @param parentIn, BaseObject parent of item
	 * @param useRefList, List containing item referenced in use clause
	 * @return LabelledItem, corresponding to this package
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn,
								 ItemReferenceList useRefList) {
		LabelledItem packageItem = romFactory.newLabelledItem
			(parentIn, null, null, null, M__String());
		Label packageLabel = romFactory.newLabel(packageItem, toString());
		packageItem.setLabel(packageLabel);
		ROM.Package packageValue = romFactory.newPackage(packageItem); 
		packageValue.setUseClauseList(useRefList);
		((PackageType)root.getType("package")).addOneElement(packageValue);
		packageItem.setValue(packageValue);
		// The object packageValue is being modified directly but not created
		M__Value(romFactory, packageValue, root);
		Type packageType = root.getType("package");
		packageItem.setType(packageType);

		return packageItem;
	}

    /** The publishSOM routine for this class
     * @return void
     */
    public void publishSOM(PrintWriter pw) {
		pw.print("PACKAGE ");
		id.publishSOM(pw);
		if (interfaceList != null) {
			interfaceList.publishSOM(pw);
		}
		pw.println(" IS ");
		packageDefinition.publishSOM(pw);
		id.publishSOM(pw);
		pw.println(";");
		pw.println();
    }
}
