//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Krishna  krishna@ittc.ukans.edu
//
//  $Id: SOM_PackageDeclarativeItem.java,v 1.5 2000/12/05 03:05:03 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_PackageDeclarativeItem.java,v $
//  Revision 1.5  2000/12/05 03:05:03  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.4  2000/11/06 18:12:01  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.3  2000/05/18 18:03:46  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the files again.
//  -- Cindy
//
//  Revision 1.3  2000/05/05 19:28:33  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.2  2000/03/23 00:22:49  cvs
//  Huge commit! Added all the essential accessor methods for the following classes
//  - Krishna and Makarand
//
//  Revision 1.1  2000/02/13 01:15:55  cvs
//  Completed the implementation for package declarations, and the "use" clause.
//  Modified the grammar to incorporate these changes. Also populated the object
//  model with these new objects and completed the pretty-printing for these new
//  constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import java.io.Serializable;

import ROM.Factory;
import ROM.Universe;
import ROM.BaseObject;
import ROM.LabelledItem;
import util.ListEnumeration;

/** This represents the <B>PackageDeclarativeItem</B> object
 * @author <I>Krishna Ranganathan</I> (krishna@ittc.ukans.edu)
 */
public interface SOM_PackageDeclarativeItem 
extends Serializable {


	/** To set the SOM_InterfaceListing object for this designUnit
	 * @param interfaceIn The SOM_InterfaceListing object to be set
	 * @return void
	 */
	public void setInterfaceListing(SOM_InterfaceListing interfaceIn);

	/** To get the interfaceListing
	 * @return The SOM_InterfaceListing object for this designUnit
	 */
	public SOM_InterfaceListing getInterfaceListing();

	/** The domain list is only needed for overload resolution.
	 * It is added here so that the domain list does not have to be
	 * processed more than once.
	 * To set the SOM_DeclarationList associated with the interface
	 * listing 
	 * @param domainListIn The SOM_DeclarationList to be set
	 * @return void
	 */
	public void setDomainList(SOM_DeclarationList domainListIn);

	/** To get the domainList
	 * @return The SOM_DeclarationList object for this designUnit
	 */
	public SOM_DeclarationList getDomainList();

	/** To get the exports
	 * @return The ListEnumeration corresponding to the export clauses
	 */
	public ListEnumeration exports();

	/** The buildROM method for this high-level item
	 * @param romFactory, Factory object used to create objects in ROM
	 * @param root, Universe object 
	 * @param parentIn, BaseObject parent of item
	 * @return LabelledItem, corresponding to this package
	 */
	public LabelledItem buildROM(Factory romFactory, Universe root, 
								 BaseObject parentIn);

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw);

}
