//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_Quantifier.java,v 1.5 2000/12/07 01:23:19 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_Quantifier.java,v $
//  Revision 1.5  2000/12/07 01:23:19  cvs
//  Adding the M__Value to all the Quantifiers. Making minor changes to the other classes
//  --srinivas
//
//  Revision 1.4  2000/05/18 18:03:47  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.4  2000/05/05 19:28:34  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.3  2000/02/10 00:37:30  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.2  1999/11/05 23:43:11  cvs
//  Modified this file as part of streamlining the ObjectModel and bringing
//  it in line with the new grammar
//
//
//=============================================================================
package SOM;

import util.List;
import util.Link;
import util.ListEnumeration;

import java.io.*;
import java.lang.NullPointerException;
import java.lang.String;

import ROM.*;

/** This is the <B>Quantifier</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public interface SOM_Quantifier {

	/** The constants */
	public static final int FORALL = 1;
	public static final int EXISTS = 2;
	public static final int MIN = 3;
	public static final int MAX = 4;
	public static final int SEL = 5;
	public static final int RAN = 6;

	/** To return the kind of this quantifier
	 * @return int that is the kind of this quantifier
	 */
	public int getQuantifier();

	/** The publishSOM routine for this class
	 * @return void
	 */
	public abstract void publishSOM(PrintWriter pw);

	/** The abstract function to get the Value of the Quantified
	 * Expression
	 * @param romFactory factory through which the items are built
	 * @param root The Universe to search for predefined functions
	 * @param expr The Expression object 
	 * @param interfaceItem ListEnumeration
	 * @return ROM.Expression
	 */
	public abstract Expression M__Value(Factory romFactory,Universe root,
										BaseObject parent,SOM_Expression expr,
										ListEnumeration interfaceEnum);
}
