package SOM;
import util.Link;
import util.ListEnumeration;
import ROM.*;
import java.io.PrintWriter;
import java.io.*;
import util.List;


/** This represents the <B>Recognizer</B> object
 * @author <I>Brandon Morel</I> (morel@ittc.ku.edu)
 */
public class SOM_Recognizer
    extends SOM_BaseClass {

    SOM_Identifier id;
    SOM_TypeDeclaration functGenInputType = null;
    private SOM_InterfaceListing interList = null;

    /** The primary Constructor
     *@param typeIn SOM_TypeDeclaration, the input type for the function generation
     * @return Not applicable
     */
    public SOM_Recognizer(SOM_TypeDeclaration typeIn) {
	super();
	functGenInputType = typeIn;
    }

    /** The secondary Constructor
     * @return Not applicable
     */
    public SOM_Recognizer() {
	super();
    }

    /** To get the string representation of this object
     * @return String representation of this SOM_Recognizer
     */
    public String toString() {
	return id.toString();		
    }

    /** To get the identifier
     * @return The SOM_Identifier corresponding to the identifier
     */
    public SOM_Identifier getIdentifier() {
	return id;
    }

    /** To set the identifer
     * @param idIn The SOM_Identifier to be set
     * @return void
     */
    public void setIdentifier(SOM_Identifier idIn) {
	id = idIn;
    }

    /** To get the inputType for the function generation
     * @return The SOM_TypeDeclaration corresponding to the input type
     */
    public SOM_TypeDeclaration getInputType() {
	return functGenInputType;
    }

    /** To set the input type for the function generation
     * @param typeIn The SOM_TypeDeclaration to be set
     * @return void
     */
    public void setInputType(SOM_TypeDeclaration typeIn) {
	functGenInputType = typeIn;
    }

    /** To set the interfaceListing
     * @param interfaceIn The SOM_InterfaceListing object to be set
     * @return void
     */
    public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
	interList = interfaceIn;
    }
    
    /** To get the interfaceListing
     * @return The SOM_InterfaceListing object for this designUnit
     */
    public SOM_InterfaceListing getInterfaceListing() {
	return interList;
    }

    /** To create interfaceListing
     * @param idIn the String representation of the id
     * @param nameIn the SOM_Name of the id type
     * @return SOM_InterfaceListing the interface created
     */
    public SOM_InterfaceListing createInterface(String idIn, SOM_Name nameIn) {
	SOM_Identifier functGenInputId = new SOM_Identifier(idIn);
	SOM_InterfaceItem functGenInputItem = new SOM_InterfaceItem();
	functGenInputItem.setIdentifier(functGenInputId); 
	functGenInputItem.setTypeName(nameIn); 
	SOM_InterfaceListing genInterList = new SOM_InterfaceListing();
	genInterList.addToInterfaceItemList(functGenInputItem);
	return genInterList;
    }

    /** publishSOM routine needed to print the object module
     * @return void
     */
    public void publishSOM(PrintWriter pw){
        id.publishSOM(pw);
    }


    /** publishFunctionGeneration routine to print object module
     * Differs from publishSOM because the function is generated, not given
     * @param PrintWriter
     * @return void
     */
    public void publishFunctionGeneration(PrintWriter pw) {
	id.publishSOM(pw);
	if (interList != null) {
	    interList.publishSOM(pw);
	}
	pw.print(" :: boolean ");
    }

    /* To get the string of the function generation
     * @param void
     * @return String
     */
    public String M__String(){
	try{
	    String str = new String();
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw,true);
	    publishFunctionGeneration(pw);
	    pw.flush();
	    pw.close();
	    str = sw.toString();
	    sw.close(); 
	    return str;
	}
	catch (Exception e) {
	    System.out.println(e);
	    return (String)null;
	}
    }

    /** The function that creates the ROM.Expression for the function
     * @param Factory,BaseObject,Universe
     * @return ROM.Expression(ROM.Facet)
     */
    public Expression M__Value(Factory romFactory,
			       BaseObject parentIn, Universe root){
	// I have only implemented function generation, the expression to this
	// function has NOT been implemented!!!
	// Will be null for now!!
	SOM_Expression expr = null;
	    
	if (getInterfaceListing() != null){
	    Function function= romFactory.newFunction(parentIn);
	    ListEnumeration interList = getInterfaceListing().interfaces();
	    while (interList.hasMoreElements()){
		SOM_InterfaceItem interItem = (SOM_InterfaceItem)
		    interList.nextElement();
		function.addInterfaceItem(interItem.buildROM
					  (romFactory, root, function));
	    }
	    if (expr != null)
		function.setExpression
		    (expr.M__Value(romFactory, function, root));
	    return (Expression)function;
	}

	else
	    return null;
    }

    /* To get the type of the function generation
     * TO BE COMPLETED WHEN TYPE CHECKING IS FIXED
     * RETURN NULL FOR NOW
     * @param void
     * @return String
     */
    public Type M__Type(BaseObject parent, BaseObject
			newParent,Universe root, String identifier,
			Expression value) {
	Type type = null;
	return type;
    }

    /** The buildROM method for this high-level item
     * VOID BECAUSE IT IS GENERATION FUNCTIONS AND DIRECTLY ADDING TO ROOT!
     * @param ROMFactory, Factory object used to create objects in ROM
     * @return LabelledItem, corresponding to this facet  
     */
    public void buildROM(Factory romFactory, BaseObject parentIn, Universe
			 root) {
	LabelledItem declItem;
	declItem = romFactory.newLabelledItem(parentIn, null, null, null, M__String());
	Label declLabel =  romFactory.newLabel(declItem, id.toString());
	declItem.setLabel(declLabel);
	Expression declValue = M__Value(romFactory, declItem, root);
	declItem.setValue(declValue);
	Type type = M__Type(parentIn, declItem, root, id.toString(),
			    declValue); 
	declItem.setType(type);

        // Added by Morel, July 14, 2001
	// Convert parent back to package or facet then manually add the generated fnc
	ROM.Facet packORfacetValue = (ROM.Facet)parentIn;
	packORfacetValue.addItem(declItem);
    }
}
