//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: SOM_SequenceDeclarationStatement.java,v 1.3 2001/11/02 16:19:28 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_SequenceDeclarationStatement.java,v $
//  Revision 1.3  2001/11/02 16:19:28  cvs
//  Made changes to support facet as a type.
//    --Murthy
//
//  Revision 1.2  2000/12/03 23:55:26  cvs
//  Added the M__Value method to the Literal classes.Previously they were returning null.The M__Type method for all the Declaration statement classes has been completed
//  --srinivas
//
//  Revision 1.1  2000/07/11 00:11:13  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and sequence are
//  keywords. Typechecking for set is done. Added SetType and FunctionType.
//  False and true are predefined as boolean.
//  -- Cindy
//
//
//============================================================================
package SOM;

import util.ListEnumeration;
import ROM.*;
import java.io.*;
import java.util.*;

public class SOM_SequenceDeclarationStatement 
extends SOM_BaseClass {

	private SOM_Expression typeExpr = null;
	private SOM_Expression typeValue = null;
	
	/** The primary Constructor
	 * @return Not applicable
	 */
	public SOM_SequenceDeclarationStatement()
	{
		super();
	}

	/** To set the expression for the type
	 * @param typeExprIn The SOM_Expression representing the type
	 * @return void
	 */
	public void setType(SOM_Expression typeExprIn) {
		typeExpr = typeExprIn;
	}

	/** To get the expression for the type
	 * @return The SOM_Expression representing the type
	 */
	public SOM_Expression getType() {
		return typeExpr;
	}
 
	/** To set the expression for the values of the type
	 * @param typeValueIn The SOM_Expression for the values in type
	 * @return void
	 */
	public void setValue(SOM_Expression typeValueIn) {
		typeValue = typeValueIn;
	}

	/** To get the expression for the values of the type
	 * @return The SOM_Expression for the values in the type
	 */
	public SOM_Expression getValue() {
		return typeValue;
	}

	//Added by srinivas on Nov 29th, 2000
	/** The function is to get the Type of the sequence declaration
	 * @param parent
	 * @param root The Universe object to search for predefined types
	 * @param value The Expression object to add to the bunchValue of type
	 * @return ROM.Type
	 */
	public Type M__Type(BaseObject parent,Universe root, Expression value){
		SOM_Expression expr = getType();
		Type type = null;
		if (expr != null){
			String str = expr.getResolvedType().toString();
			if (str.equals("integer")){
				type = root.getType("sequenceInteger");
				if (value != null)
					((SequenceIntegerType)type).addOneElement(value);
			}else if (str.equals("real")){
				type = root.getType("sequenceReal");
				if (value != null)
					((SequenceRealType)type).addOneElement(value);
			}else if (str.equals("bit")){
				type = root.getType("sequenceBit");
				if (value != null)
					((SequenceBitType)type).addOneElement(value);
			}else if (str.equals("boolean")){
				type = root.getType("sequenceBoolean");
				if (value != null)
					((SequenceBooleanType)type).addOneElement(value);
			}else{
				Type result=null;
				if (parent instanceof Facet){
					type = ((Facet)parent).getType(str);
					result =
						((Facet)parent).alreadyDeclared("sequence",type);
				}
				if (type == null){
					type = root.getType(str);
					result = root.alreadyDeclared("sequence",type);
				}
				if (result == null){
					BunchValue sequenceBunch =
						new BunchValue(null, null, new HashSet());
					SequenceType sequence = new
						SequenceType(parent,sequenceBunch,type);
					sequenceBunch.setType(sequence);
					if (value != null)
						((SequenceType)sequence).addOneElement(value);
					if (parent instanceof Facet)
						((Facet)parent).addTypeElement(sequence);
					return (Type)sequence;
				}else {
					if (value != null)
						((SequenceType)result).addOneElement(value);
					return result;
				}
			}
		}
		return type;
	}

	/** The publishSOM routine for this class
	 * @return void
	 */
	public void publishSOM(PrintWriter pw){
		pw.print(" SEQUENCE");
		pw.print("(");
		if (typeExpr != null) {
		  typeExpr.publishSOM(pw);
		}
		pw.print(")");
	    if (typeValue != null){
			pw.print(" IS");
			typeValue.publishSOM(pw);
	    }
		pw.print(";");
	}
}
