//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SOM_VariableDeclaration.java,v 1.16 2001/10/01 18:46:09 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SOM_VariableDeclaration.java,v $
//  Revision 1.16  2001/10/01 18:46:09  cvs
//  When parsing, type info is only printed out when printInfo flag in
//  Sldl.java is set to true. Other changes include tabulation set to 2.
//  -- Cindy
//
//  Revision 1.15  2001/08/28 23:27:01  cvs
//  Added the initial implementation of Domains & Interactions.
//          --Murthy
//
//  Revision 1.14  2000/12/05 03:05:04  cvs
//  Modified semantic checking accordingly after grammar was changed to
//  have all files starting with package.
//  -- Cindy
//
//  Revision 1.13  2000/11/06 18:20:55  cvs
//  Corrected an error.
//  -- Cindy
//
//  Revision 1.12  2000/11/06 18:12:02  cvs
//  Implementing the loader.
//  -- Cindy
//
//  Revision 1.11  2000/07/11 00:11:17  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.10  2000/05/18 18:03:50  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the files again.
//  -- Cindy
//
//  Revision 1.10  2000/05/05 19:28:37  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  Also, SOM_FacetBodyItem is no longer an interface, but a class.
//  - Cindy
//
//  Revision 1.9  2000/04/11 03:17:54  cvs
//  The dummy M__Label and M__Value functions have been removed .
//  --srinivas
//
//  Revision 1.8  2000/04/06 19:13:26  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators. 
//   -- Cindy
//
//  Revision 1.7  2000/03/23 00:22:51  cvs
//  Huge commit! Added all the essential accessor methods for the
//  following classes 
//  - Krishna and Makarand
//
//  Revision 1.6  2000/02/13 01:15:57  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a type.
//  These above changes have been compiled and tested successfully.
//  - Krishna.
//
//  Revision 1.5  2000/02/10 00:37:35  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.4  2000/02/08 18:42:45  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help of Makarand and Srini). Removed some unnecessary
//  classes (due to changes brought to the grammar). Renamed
//  SOM_EqualityOperator to SOM_EqualEqualOperator. Modified
//  SldlParser.jj.h to populate the object model. 
//  -- Cindy.
//
//  Revision 1.3  1999/11/15 03:16:29  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.2  1999/11/13 20:01:29  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:51  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:51:56  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.8  1999/10/01 15:23:37  rkamath
//  Commit for record - more cleaner typing ideas.
//
//  Revision 1.7  1999/08/01 20:54:41  rkamath
//  Made a lot of changes with regard to types and checking. I'm convinced that
//  there are going to me hundreds more changes until I stabilize on how to
//  import and [de]serialize
//
//  Revision 1.6  1999/07/28 01:59:48  rkamath
//  Changed all the SOM_Declaration references to SYM_SymboltableEntry as far
//  as lookUp and typeResolution are concerned
//
//  Revision 1.5  1999/07/16 21:50:36  rkamath
//  Changed everyexpression to point to the type declaration instead of having
//  a private variable which would indicate the resulting type
//
//  Revision 1.4  1999/07/13 14:27:50  rkamath
//  Changed the publishRosetta() function to take in a PrintWriter
//  Added some new classes too.
//  Also commented out currency based operations from the SOM_List class. Hope-
//  fully at a later date I'll include them in the SOM_ListEnumeration class if
//  found necessary.
//
//  Revision 1.3  1999/07/10 21:31:27  rkamath
//  The commit for today. Introduced a toString() function in
//  SOM_Declaration that 
//  will be used to hash "declarations" into the symboltable
//
//  Revision 1.2  1999/07/09 22:21:24  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//  Revision 1.1  1999/07/06 21:01:50  rkamath
//  Having added the type expression syntax the corresponding OM...etc
//  Also now, only subclasses of RuntimeException are being thrown
//
//
//=============================================================================
package SOM;

import java.io.PrintWriter;
import util.ListEnumeration;
import ROM.*;

/** This represents the <B>VariableDeclaration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SOM_VariableDeclaration
  extends SOM_Declaration 
  implements SOM_FacetDeclarativeItem, SOM_ComponentDeclarativeItem,
	     SOM_PackageDeclarativeItem, SOM_InteractionDeclarativeItem,
	     SOM_DomainDeclarativeItem {  

  private SOM_Identifier variable;

  /** The primary constructor
   * @param varIn The SOM_Identifier that is the variable
   * @param typeIn The SOM_TypeDeclaration corresponding to the type
   * @return Not applicable
   */
  public SOM_VariableDeclaration(SOM_Identifier varIn, SOM_TypeDeclaration typeIn) {
    super();
    setKind(VARIABLE);
    setType(typeIn);
    variable = varIn;
  }

  /** To get the identifier of the variable
   * @return The SOM_Identifier for the variable declaration
   */
  public SOM_Identifier getIdentifier() {
    return variable;
  }

  /** To implement my own version of isDistinguishable for variables
   * @param declOne The first SOM_VariableDeclaration
   * @param declTwo The next SOM_VariableDeclaration to be distinguished
   * against declOne
   * @return true if they are distinguishable, false otherwise
   */
  public static boolean isDistinguishable(SOM_VariableDeclaration
					  declOne,
					  SOM_VariableDeclaration
					  declTwo) { 
    // currently two variable declarations are different if their types
    // are distinguishable
    return SOM_TypeDeclaration.isDistinguishable
      (declOne.getType(), declTwo.getType());
  }

  /** To ascertain "compatiblity" for bindings of the kind 
   * lhs <- rhs
   * @param declLHS The typeDecl corresponding to the lhs object
   * @param declRHS The typeDecl corresponding to the rhs object
   * @return true, if the rhs is assignable to the lhs, false
   * otherwise 
   */
  public static boolean
    isLeftCompatibleWithRight(SOM_VariableDeclaration declLHS,
			      SOM_VariableDeclaration declRHS) { 
    // The assignment is valid if the type of declLHS is a pred
    // supertype of the type of declRHS - re. type checking algo
    // for rosetta 
    SOM_TypeDeclaration typeLHS, typeRHS;
    typeLHS = declLHS.getType(); 
    typeRHS = declRHS.getType(); 

    // The binding is compatible if typeRHS is a predicate subtype
    // of typeLHS 
    // re. rosetta type checking algo..
    return typeRHS.isPredicateSubtypeOf(typeLHS); 
  }

  /** To get the string representation of this object. Overrides the
   * method in SOM_Declaration
   * @return String representation of this object
   */
  public String toString() {
    return variable.toString();
  }

  /** The buildROM method for this high-level item
   * @param romFactory, Factory object used to create objects in ROM
   * @param root, Universe object 
   * @param parentIn, BaseObject parent of item
   * @return LabelledItem, corresponding to this package
   */
  public LabelledItem buildROM(Factory romFactory, Universe root, 
			       BaseObject parentIn){
    return null;
  }
	
  /** publishSOM routine 
   * @return void
   */
  public void publishSOM (PrintWriter pw) {
    variable.publishSOM(pw);
    getType().publishSOM(pw);
  }

  // the next three productions are not actually needed per se
  // they are added only to instantiate the methods for the abstract
  // class SOM_PackageDeclarativeItem - krishna on Feb 12, 2000
  /** To set the SOM_InterfaceListing object for this designUnit
   * @param interfaceIn The SOM_InterfaceListing object to be set
   * @return void
   */
  public void setInterfaceListing(SOM_InterfaceListing interfaceIn) {
  }

  /** To get the interfaceListing
   * @return The SOM_InterfaceListing object for this designUnit
   */
  public SOM_InterfaceListing getInterfaceListing() {
    return null;
  }

  /** To set the domainList - implements the function from
   * SOM_PackageDeclarativeItem
   * @param domainListIn The SOM_DeclarationList object to be set
   * @return void
   */
  public void setDomainList(SOM_DeclarationList domainListIn) {
  }

  /** To get the domainList - implements the function from
   * SOM_PackageDeclarativeItem
   * @return The SOM_DeclarationList object for this designUnit
   */
  public SOM_DeclarationList getDomainList() {
    return null;
  }
		
  /** To get the exports
   * @return The ListEnumeration corresponding to the export clauses
   */
  public ListEnumeration exports() {
    return null;
  }
}

