//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SYM_Symboltable.java,v 1.21 2001/08/28 23:31:37 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SYM_Symboltable.java,v $
//  Revision 1.21  2001/08/28 23:31:37  cvs
//  Added the implementation of Domains & Interactions
//       --Murthy
//
//  Revision 1.20  2001/07/03 14:53:58  cvs
//
//  Initial implementation of enumerations.
//  --Morel
//
//  Revision 1.19  2001/05/24 20:16:04  cvs
//  Some small changes. Removed an error in the declaration of "abs" in
//  SYM_SymboltableManager.
//  -- Cindy
//
//  Revision 1.18  2001/03/21 00:06:11  cvs
//  Changed the semantics for export clauses.
//  -- Cindy
//
//  Revision 1.17  2001/03/12 23:55:05  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.16  2000/12/05 03:06:28  cvs
//  Modified semantic checking, needed after grammar was changed to have
//  all files starting in a package.
//  -- Cindy
//
//  Revision 1.15  2000/08/18 19:50:47  cvs
//  Added some semantic checking to use of underscore as parameter to
//  function.
//  -- Cindy
//
//  Revision 1.14  2000/07/24 23:37:23  cvs
//  Confusion between # (size for set and arrays) and $ (bunch and
//  sequence).
//  -- Cindy
//
//  Revision 1.13  2000/07/11 00:11:54  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.12  2000/06/22 17:31:16  cvs
//  Strategy for typechecking of functions was slightly changed. Lists of
//  possible declarations of a function are made for each parameter to the
//  function. Then the first declaration in the intersection of the lists
//  gives the correct function declaration.
//  -- Cindy
//
//  Revision 1.11  2000/05/18 19:51:58  cvs
//  Added a + missing in a printout statement.
//  -- Cindy
//
//  Revision 1.10  2000/05/18 18:04:46  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the file again.
//  -- Cindy
//
//  Revision 1.9  2000/04/12 18:41:36  cvs
//  Implemented some type checking for "use" clause.
//  -- Cindy
//
//  Revision 1.8  2000/04/06 19:14:29  cvs
//  Type checking is done for declaration of variables (simple types), for
//  declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.7  2000/02/10 00:37:37  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  1999/11/17 18:31:56  cvs
//  Coded resolution of final type (unfolding overload resolution) and included
//  actions for TCC generation
//
//  Revision 1.5  1999/11/16 04:05:24  cvs
//  Added support for export clause and visiblity
//
//  Revision 1.4  1999/11/15 22:59:16  cvs
//  Changes to incorporate unfolding of overloading
//
//  Revision 1.3  1999/11/15 03:16:30  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.2  1999/11/13 20:01:30  cvs
//  Changed to the new OM and grammar. Some more features are to be added
//  and many bugs to be corrected. This is a commit for record and it compiles
//
//  Revision 1.1  1999/10/27 20:43:55  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:52:18  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.10  1999/10/01 15:25:00  rkamath
//  working prototype commit
//
//  Revision 1.9  1999/08/01 20:56:31  rkamath
//  Cleaned up (code and my thinking too :) still struggling with how to
//  implement serialization etc to minimize sideeffects..
//
//  Revision 1.8  1999/07/28 02:01:12  rkamath
//  Coded my more clearer concepts. Commit for record
//
//  Revision 1.7  1999/07/22 22:51:23  rkamath
//  Intermediate commit for safety and records. This does not compile because
//  the toString() in SOM_FormalIdentifier has not been completely coded
//  Will do that tommorrow
//
//  Revision 1.6  1999/07/16 21:51:41  rkamath
//  Some small changes. Commit for records...
//
//  Revision 1.5  1999/07/14 23:26:37  rkamath
//  Commit for today. Introduced static functions in SYM_SymboltableManager
//  and SOM_BaseClass for cleaner segregation of tasks and implementation
//
//  Revision 1.4  1999/07/13 19:33:34  rkamath
//  Intermediate commit for records..
//
//  Revision 1.3  1999/07/10 21:34:23  rkamath
//  Added some more code - a generic commit. Some rudimentary form of
//  declaration checking exists. Now to populate the symbol table and
//  code it in the .jj file 
//
//  Revision 1.2  1999/07/09 22:21:33  rkamath
//  A lot of changes to incorporate the intent of type checking
//
//  Revision 1.1  1999/07/08 23:25:24  rkamath
//  A fresh repository going towards an elementary type checker - and
//  mebbe more 
//
//=============================================================================
package SYM;

import SOM.SOM_ArgumentList;
import SOM.SOM_ArrayListing;
import SOM.SOM_AssociationItem;
import SOM.SOM_AssociationListing;
import SOM.SOM_Declaration;
import SOM.SOM_DeclarationList;
import SOM.SOM_Expression;
import SOM.SOM_FacetDeclaration;
import SOM.SOM_FormalName;
import SOM.SOM_Identifier;
import SOM.SOM_InterfaceItem;
import SOM.SOM_InterfaceListing;
import SOM.SOM_OperatorDeclaration;
import SOM.SOM_PackageDeclarativeItem;
import SOM.SOM_VariableDeclaration;
import SOM.SOM_Name;
import SOM.SOM_ReferencedName;
import SOM.SOM_SelectedName;
import SOM.SOM_TreeNode;
import SOM.SOM_TreeNodeList;

import util.List;
import util.ListEnumeration;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.NoSuchElementException;

/** This is the <B>SYM_Symboltable</B> object
 * This will be used to code ONLY the scoping and type checking. The other
 * semantic checks should be coded in the SOM classes.
 * All symbol tables only hold the top level terms in the final analysis!
 * Any block declaration that needs its own symboltable will have a pointer
 * to that table within the declaration entry in this table
 * It has a hashtable: each entry corresponds to a SYM_SymboltableEntryList
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SYM_Symboltable
extends Object
implements Serializable {
	
	private String name;
	private SOM_Declaration parentDeclaration = null;

	private Hashtable hashtable = new Hashtable();
	private transient int currentScope;

	private transient List usableTables = new List();
	private List importClauseList = new List();

	// This object is thrown everytime a noDeclarationException is to be
	// thrown. This saves on object creation time!
	private static SYM_NoDeclarationException noDeclExpObj = 
		new SYM_NoDeclarationException
		("SYM_Symboltable: The Static SYM_NoDeclarationException Object");


	/** The Primary constructor.
	 * @param nameIn The String corresponding to the design unit for which
	 * this SYM_Symboltable exists
	 * @return Not applicable
	 */
	/*
	public SYM_Symboltable(String nameIn) {
		super();
		name = nameIn;
		currentScope = 0;
	}
	*/

	/** The second constructor. This will probably be most used one!!
	 * @param nameIn The String corresponding to the design unit/type for which
	 * this SYM_Symboltable exists
	 * @param parentDeclIn The SOM_Declaration for which this symboltable
	 * is made
	 * @return Not applicable
	 */
	public SYM_Symboltable(String nameIn, SOM_Declaration parentDeclIn) {
		super();
		name = nameIn;
		currentScope = 0;
		parentDeclaration = parentDeclIn;
	}

	/** To add a new import clause to the importClauseList.
	 * @param importIn The String corresponding to the importClause to be added
	 * @return void
	 */
	public void addToImportClauseList(String importIn) {
		importClauseList.addElement(importIn);
	}

	/** To add a table to the usableTables list - it will be prepended
	 * @param tableIn The SYM_Symboltable to be added to the usableTables list
	 * @return void
	 */
	public void addToUsableTablesList(SYM_Symboltable tableIn) {
		usableTables.prependElement(tableIn);
	}
	 
	/** To open a new scope. Increments the currenScope variable
	 * @return void
	 */
	public void openScope() {
		currentScope++;
	}

	/** To close the current scope. Decrements the currentScope variable.
	 * @return void
	 * @exception SYM.SYM_NullScopeException If this function is called when
	 * the currentScope is 0
	 */
	public void closeScope() {
		if (currentScope == 0) 
			throw new SYM_NullScopeException
				("SYM_Symboltable: Called closeScope on 0 current scope");
		else {
			// First clean up the symbol table of current scope symbols
			Enumeration enum = hashtable.keys();
			SYM_SymboltableEntryList list;
			while (enum.hasMoreElements()) {
				Object currKey = enum.nextElement();
				list  = (SYM_SymboltableEntryList)hashtable.get(currKey);
				list.flushEntriesOfScope(currentScope);
				if (list.isEmpty()) hashtable.remove(currKey);
			}
			currentScope--;
		}
	}

	/** To add a SOM_Declaration for a variable to the symbol
	 * table. It is first embedded within a SYM_SymboltableEntry which
	 * is then added to the symbol table 
	 * @param declIn The SOM_Declaration to be added
	 * @return void
	 */
	public void addDeclaration(SOM_Declaration declIn) {
		// First add the qualified name for this decl
		declIn.setParentSymboltable(this);
		SYM_SymboltableEntry entry =
			new SYM_SymboltableEntry(declIn, currentScope);
		// update the export information
		addEntry(entry, false);
	}

	/** To add a SOM_Declaration for functions to the symbol
	 * table. It is first embedded within a SYM_SymboltableEntry which
	 * is then added to the symbol table 
	 * @param declIn The SOM_Declaration to be added
	 * @return void
	 */
	public void addFuncDeclaration(SOM_Declaration declIn) {
		// First add the qualified name for this decl
		declIn.setParentSymboltable(this);
		SYM_SymboltableEntry entry =
			new SYM_SymboltableEntry(declIn, currentScope);
		// update the export information
		addEntry(entry, true);
	}

	/** To add a SYM_SymboltableEntry to the symboltable.
	 * This should actually be implemented in the SYM_SymboltableManager once
	 * it is fully done. So also for the checkForRedeclaration function
	 * @param entry The SYM_SymboltableEntry to be added
	 * @return void
	 * @exception java.lang.NullPointerException If entry is null
	 * @exception SYM.SYM_IllegalRedeclarationException If a declaration
	 * corresponding to the entry already exists
	 */
	protected void addEntry(SYM_SymboltableEntry entry, boolean overload) {
		String hashString = entry.toString();
		Object list = hashtable.get(hashString);

		if (list == null) {
			hashtable.put(hashString, new SYM_SymboltableEntryList(entry));
		}
		else {
			SYM_SymboltableEntryList castedList = 
				(SYM_SymboltableEntryList)list;
			ListEnumeration enum = castedList.elements();
			SOM_Declaration entryDecl = entry.getDeclaration();
			int entryScope = entry.getScope();

			// Now First check for redeclaration in the current Scope
			// Then for overshadowing names in previous scopes
			// Note the assumption that the current scope entries are at the
			// head of the list and those beyond are from lesser scopes...
			SYM_SymboltableEntry cursor;
			boolean overshadow = false;
	
			while (enum.hasMoreElements() && overshadow == false) {
				cursor = (SYM_SymboltableEntry)enum.nextElement();
				if (cursor.getScope() == entryScope) {

				    // Added by Morel, June 30, 2001
				    // Handles enumeration declarations
				    if (cursor.getDeclaration().getKind() == SOM_Declaration.ENUMERATIONITEM)
					throw new SYM_IllegalEnumerationRedeclarationException
					    ("SYM_SymboltableEntryList: Illegal" + 
					     "redeclaration of name " + entry.toString());

				    // still enumerating in the current scope
				    else if (!overload || 
					     !SOM_Declaration.isDistinguishable
					     (cursor.getDeclaration(), entryDecl))
					throw new SYM_IllegalRedeclarationException
					    ("SYM_SymboltableEntryList: Illegal" + 
					     "redeclaration of name " + entry.toString());
				}
				
				else {
					
					// check for shadowing. This is purely for
					// information purposes 
					// and does not serve to do any critical semantic check
					if (!SOM_Declaration.isDistinguishable
							(cursor.getDeclaration(),entryDecl))
						overshadow = true;
				}
			}
			if (overshadow == true) 
				System.err.println("Warning: name " + entry.toString() 
								   + " overshadows those in previous scopes");
			// If no exception was thrown, we can add this decl to the head
			castedList.prependElement(entry);
		}
	}

	/** To delete a SYM_SymboltableEntry from the symbol table. If the
	 * entry exists it is deleted, else nothing happens.
	 * @param entry The SYM_SymboltableEntry to be added
	 * @return void
	 * @exception java.lang.NullPointerException If entry is null
	 */
	protected void removeEntry(SYM_SymboltableEntry entry) {
		SYM_SymboltableEntryList list = 
			(SYM_SymboltableEntryList)hashtable.get(entry.toString());
		if (list != null) 
			list.removeElement(entry);
		if (list.isEmpty()) hashtable.remove(entry.toString());
	}

	/** To lookup a name
	 * @param nameIn The SOM_Name to be lookedup
	 * @return A SOM_DeclarationList with all declarations
	 */
	public SOM_DeclarationList lookUp(SOM_Name nameIn) {

		SOM_Name internalName;

		if (nameIn instanceof SOM_Identifier)
			return lookUp(nameIn.toString()); // the terminating case
		else internalName = nameIn.getName();
		
		// note that internalList corresponds to all the declarations
		// for internalName
		SOM_DeclarationList internalList = lookUp(internalName);

		SOM_DeclarationList toReturn = new SOM_DeclarationList();

		if (nameIn instanceof SOM_FormalName) {

			// Commented out the validateAssociationListing until
			// binding to formals is implemented....currently using
			// overload resolution !!
			ListEnumeration assocEnum = 
				((SOM_FormalName)nameIn).getAssociationListing().
				associations();

			// Now generate the SOM_Tree from the internalList ... 
			SOM_ArgumentList assocAsArgs = new SOM_ArgumentList();
			SOM_Expression currItem;
			Object currArg;
			while (assocEnum.hasMoreElements()) {
				currArg = assocEnum.nextElement();
				if (currArg instanceof SOM_Expression) {
					currItem = (SOM_Expression)currArg;
					// Commented out by ckong Feb 29, 2000
					// if (currItem.getIdentifier() != null)
					// throw new RuntimeException("Yet to implement"
					// + "formal bindings in association listings"); 
					//assocAsArgs.addElement(currItem.getExpression().
					//					   getTypeList()); 
					assocAsArgs.addElement(currItem.getTypeList());
				} else {
					// The element is an underscore
					SOM_DeclarationList typeList = lookUp("_");
					SOM_VariableDeclaration underscoreDecl =
						(SOM_VariableDeclaration)typeList.getFirst();
					SOM_TreeNodeList nodeList = new SOM_TreeNodeList();
					nodeList.add(new SOM_TreeNode(underscoreDecl.getType()));
					assocAsArgs.addElement(nodeList);
				
				}
			}

			// Now the overload resolution
			SOM_TreeNodeList returnList = 
				SYM_StaticFunctions.buildTree(internalList, assocAsArgs);

			// Now follows a hack! I've to save this returnList in
			// SOM_FormalName 
			// since it is needed for resolveFinalType - hence,
			// setDeclarations in SOM_FormalName should not setTypeList!!
			((SOM_FormalName)nameIn).setTypeList(returnList);
			
			// Now Unfortunately we have to convert back to
			// SOM_DeclarationList... 
			// it would be great if we could just return this since as it is
			// the SOM_DeclarationList is converted to SOM_TreeNodeList within
			// setDeclarations() of SOM_Name....
			// returning SOM_TreeNodeList will be an excellent optimization...
			ListEnumeration returnEnum = returnList.elements();
			SOM_TreeNode currNode;
			while (returnEnum.hasMoreElements()) {
				currNode = (SOM_TreeNode)returnEnum.nextElement();
				// DO NOT RETRIEVE THE resultType here !! That is for
				// type resolution 
				// when overloading. Here we want the declaration
				// corresponding to 
				// nameIn!!
				toReturn.addElement(currNode.getLookUpDecl());
			}

			/* The old envisaged idea of looking up asociation bindings...
				// validate the association_listing
				if (currSymbtab.validateAssociationListing
				(((SOM_FormalName)nameIn).getAssociationListing()))
				toReturn.addElement(currDecl);
				*/
		}
		else if (nameIn instanceof SOM_ReferencedName) {
			throw new RuntimeException("SYM_Symboltable::validateArrayListing"
									   + "not implemented yet"); 
		}
		else if (nameIn instanceof SOM_SelectedName) {

			ListEnumeration internalEnum = internalList.elements();
			SOM_Declaration currDecl;
			SYM_Symboltable currSymbtab;
			while (internalEnum.hasMoreElements()) {
				try {
					currDecl = (SOM_Declaration)internalEnum.nextElement();
					currSymbtab = currDecl.getMySymboltable();
					List.concat(toReturn, currSymbtab.lookUp
								(((SOM_SelectedName)nameIn).getIdentifier().
								 toString()));
				} catch (NoSuchElementException except) {
					System.out.println("(Temporary)In lookUp: " + except);
					// means this decl didn't have a
					// symboltable...safely ignore it 
				} catch (SYM_NoDeclarationException except) {
					System.out.println("(Temporary)In lookUp: " + except);
					// means that this currSymbtab didn't succed in
					// looking up the selected name...simply ignore it...
				}
			}
		}

		if (toReturn.isEmpty()) throw noDeclExpObj;
		
		return toReturn;
	}

	/** To lookup a string. This is the heart of the lookup in this symboltable
	 * and should handle all the parents etc when they are included
	 * THIS SHOULD DISAMBIGUATE SHADOWED DECLARATIONS AND NOT RETURN THEM!!
   * I'M FOLLOWING THE C++ SCOPING RULES. Any lookup returns the declarations
	 * found in the most recent scope only and none beyond it!
	 * @param hashString The String to be looked up
	 * @return A SOM_DeclaratoinList with all the decls for hashString
	 */
	public SOM_DeclarationList lookUp(String hashString) {
		SOM_DeclarationList toReturn = new SOM_DeclarationList();

		// First get all the local declarations
		SYM_SymboltableEntryList entryList = 
			(SYM_SymboltableEntryList)hashtable.get(hashString);

		if (entryList != null) {
			// Now pick out those in the nearest scope....
			int nearestScope =
				((SYM_SymboltableEntry)entryList.first()).getScope(); 

			ListEnumeration enum = entryList.elements();
			SYM_SymboltableEntry currEntry;
      
			while (enum.hasMoreElements()) {
				currEntry = (SYM_SymboltableEntry)enum.nextElement();
				if (currEntry.getScope() == nearestScope) {
					toReturn.addElement((SOM_Declaration)currEntry.
										getDeclaration()); 
				}
				else break; // faster exit if the scope has decreased!
			}
		}
		if (toReturn.isEmpty() && parentDeclaration != null) {
			// (parentDeclaration can be null for fileScope Symboltable)
			// HANDLE ALL PARENTS HERE - EACH PARENT IS AN ORTHOGONAL AXIS OF
			// SCOPING - SO CALL lookUp ON EACH PARENT AND APPEND THE RESULT
			// TO toReturn
			if (parentDeclaration instanceof SOM_FacetDeclaration) {
				ListEnumeration parentEnum = 
					((SOM_FacetDeclaration)parentDeclaration).parents();
				SOM_FacetDeclaration currParent;
				while (parentEnum.hasMoreElements()) {
					currParent =
						(SOM_FacetDeclaration)parentEnum.nextElement(); 
					try {
						List.concat(toReturn,
									currParent.getMySymboltable().
									lookUp(hashString));
					} catch (SYM_NoDeclarationException except) {
						// safely go to the next parent
					}
				}
			}
		}

		// If no decls were found, throw an exception	
		if (toReturn.isEmpty())
			throw noDeclExpObj;
		// throw new SYM_NoDeclarationException("SYM_Symboltable:" +
		// " No Declarations found for " + hashString + " in table " +
		// getName()); 

		return toReturn;
	}


	/** To validate an associationListing for this symboltable
	 * NOTE: I'M YET TO IMPLEMENT BINDING OF FORMALS TO ACTUALS  -
	 * CURRENTLY NO FORMALS ARE ASSUMED
	 * @param assocIn The SOM_AssociationListing to be validated
	 * @return true if assocIn could be validated - false otherwise
	 */
/*
  public boolean validateAssociationListing(SOM_AssociationListing
	assocIn) { 
	// I'm following the VHDL style here!
	// If any formal is bound to an actual, then all the following
		// items must 
		// be bound too
		// Else until a formal binding is located, all assocations are assumed
		// to be in order of interface items

		if (assocIn == null) return true;

		// First get the interfaces for the parentDeclaration
		SOM_InterfaceListing myInterfaceListing;
		if (parentDeclaration instanceof SOM_PackageDeclarativeItem)
			myInterfaceListing = ((SOM_PackageDeclarativeItem)parentDeclaration).
			getInterfaceListing();
		else return false;
		// If this is not a design unit then there can be no associations

		ListEnumeration interfaceEnum;
    if (myInterfaceListing != null)
			interfaceEnum = myInterfaceListing.interfaces();
		else return false;
		// if there is not interface there can be no associations...
		
		// Now match up each interface to each association
		ListEnumeration assocItemEnum = assocIn.associations();
		SOM_AssociationItem currItem;
    SOM_InterfaceItem currFormal;
		while (assocItemEnum.hasMoreElements() && interfaceEnum.hasMoreElements()) {
			currItem = (SOM_AssociationItem)assocItemEnum.nextElement();
      currFormal = (SOM_InterfaceItem)interfaceEnum.nextElement();

			if (currItem.getIdentifier() != null)
				throw new RuntimeException("Support for Formals in AssociationItems is not implemented yet");
			if (!currItem.getExpression().isOfType(currFormal.getType())) {
        return false;
				//throw new SYM_TypeInconsistencyException("Type of formal (" + currFormal.getName() + ") does not match any type of actual");
			}
		}
		// Exit from the while loop is only when either one (or both) of
    // assocItemEnum or interfaceEnum is exhausted

    if (assocItemEnum.hasMoreElements()) {
      // means that the association had more elements than interfaceEnum
			return false;
		}

		// The complementary case to the above
		//	 if (interfaceEnum.hasMoreElements()) {
		//	   return false;
		//	 }
		//	 should return false if incomplete instantiation is not allowed
				
		return true;
	}
	*/


	/** To validate an arrayListing for this symboltable
	 * @param arrayIn the SOM_ArrayListing to be validated
	 * @return true
	 */
	/*
	public boolean validateArrayListing(SOM_ArrayListing arrayIn) {
		throw new
		* RuntimeException("SYM_Symboltable::validateArrayListing not
		* implemented yet"); 
	}
	*/

	/** To process the export lists for design units. Currently I handle only
	 * the locally defined symbols. By default, all symbols that were
	 * previously exported in the parents are exported here!
	 * @return void
	 */
	public void processExports() {
		if (parentDeclaration == null) return;

		// Exporting is only for PackageDeclarativeItem
		if (parentDeclaration instanceof SOM_PackageDeclarativeItem) {
			SYM_SymboltableEntryList currList;

			ListEnumeration exportEnum =
				((SOM_PackageDeclarativeItem)parentDeclaration).exports();

			// exportEnum is empty    -> export all 
			// exportEnum is null     -> do not export anything
			// exportEnum is nonempty -> export only what is in list

			if (exportEnum != null) {
				// note that exportEnum will have at least one element
				// - otherwise it means that the export clause was
				// absent (equiv to exportAll clause present)
				if (!exportEnum.hasMoreElements()) {
					// if this symboltable is serialized later all the
					// decls will be serialized...
					return;
				}

				// Now iterate for each export element
				SOM_Identifier currExport;
				while (exportEnum.hasMoreElements()) {
					currExport = (SOM_Identifier)exportEnum.nextElement();
					// look up all local declarations for this id and set their
					// visibility to true
					currList = (SYM_SymboltableEntryList)
						hashtable.get(currExport.toString());
					if (currList == null) 
						throw new SYM_NoDeclarationException
							("Use of label " + currExport.toString() + 
							 " illegal. No declaration found for it");
					currList.setIsVisible(true);
				}
			}

			// now clean up the symboltable for each nonvisible entry
			Enumeration keyEnum = hashtable.keys();
			Object currKey;

			while (keyEnum.hasMoreElements()) {
				currKey = keyEnum.nextElement();
				currList  = (SYM_SymboltableEntryList) 
					hashtable.get(currKey);
				if (currList.getIsVisible() == false) 
					hashtable.remove(currKey);
			}
		}
		
	}

	/** To retreive the name of this symbol table
	 * @return String which is the name of the symboltable
	 */
	public String getName() {
		return name;
	}
}
















