//====================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: SYM_SymboltableManager.java,v 1.22 2001/11/02 03:18:41 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: SYM_SymboltableManager.java,v $
//  Revision 1.22  2001/11/02 03:18:41  cvs
//  Added class SYM_SymboltableNotFound.java to indicate a file being used that hasn't been parsed
//   Altered SYM_SymboltableManager.java to throw the new exception and provide the file name.
//    --Joseph,Garrin,Wang,Murthy.
//
//  Revision 1.21  2001/08/28 23:31:37  cvs
//  Added the implementation of Domains & Interactions
//       --Murthy
//
//  Revision 1.20  2001/07/03 14:53:58  cvs
//
//  Initial implementation of enumerations.
//  --Morel
//
//  Revision 1.19  2001/05/24 20:16:04  cvs
//  Some small changes. Removed an error in the declaration of "abs" in
//  SYM_SymboltableManager.
//  -- Cindy
//
//  Revision 1.18  2001/05/21 20:29:38  cvs
//  Added type signature for the div and mod functions. e.g. div(a, b)
//  where a and b are integers is an integer.
//  -- Cindy
//
//  Revision 1.17  2001/03/12 23:55:05  cvs
//  Changed the order of precedence of the "=" operator. It now has lowest
//  precedence of all operators except for the "==" operator. Added some
//  type checking for facet expressions and as a result, domain names have
//  to be legal facet names. Removed several bugs as well.
//  -- Cindy
//
//  Revision 1.16  2000/08/18 19:50:48  cvs
//  Added some semantic checking to use of underscore as parameter to
//  function.
//  -- Cindy
//
//  Revision 1.15  2000/07/24 23:37:23  cvs
//  Confusion between # (size for set and arrays) and $ (bunch and
//  sequence).
//  -- Cindy
//
//  Revision 1.14  2000/07/11 00:11:55  cvs
//  Change in the grammar: type, set, bunch, array, record, tuple and
//  sequence are keywords. Typechecking for set is done. Added SetType
//  and FunctionType. 
//  False and true are predefined as boolean.
//  -- Cindy
//
//  Revision 1.13  2000/06/22 17:31:16  cvs
//  Strategy for typechecking of functions was slightly changed. Lists of
//  possible declarations of a function are made for each parameter to the
//  function. Then the first declaration in the intersection of the lists
//  gives the correct function declaration.
//  -- Cindy
//
//  Revision 1.12  2000/05/18 18:04:46  cvs
//  Due to /projects directory failing and the latest version
//  available is old, I am committing the file again.
//  -- Cindy
//
//  Revision 1.12  2000/05/05 19:29:43  cvs
//  Added:
//  	- use of record and tuple keywords for record and tuple formers
//  	- new syntax for sel, forall, exists, min, max and ran functions
//  	- all expressions between "begin" and "end" are labelled.
//  - Cindy
//
//  Revision 1.11  2000/04/28 19:31:01  cvs
//  Added some types, operators and predefinded functions that were
//  previously missing. 
//  -- Cindy
//
//  Revision 1.10  2000/04/12 18:41:36  cvs
//  Implemented some type checking for "use" clause.
//  -- Cindy
//
//  Revision 1.9  2000/04/06 19:14:30  cvs
//  Type checking is done for declaration of variables (simple types),
//  for declaration of new types and for use of basic operators.
//   -- Cindy
//
//  Revision 1.8  2000/02/13 01:15:59  cvs
//  Completed the implementation for package declarations, and the
//  "use" clause. 
//  Modified the grammar to incorporate these changes. Also populated
//  the object model with these new objects and completed the
//  pretty-printing for these new constructs.
//  Also modified the parser to now recognize the keyword "type" as a
//  type. These above changes have been compiled and tested
//  successfully. 
//  - Krishna.
//
//  Revision 1.7  2000/02/10 00:37:38  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.6  2000/01/29 22:37:03  cvs
//  parser update by krishna -
//  completed some major changes to the expression grammar to
//  incorporate all the basic and composite types (including arrays,
//  sets and bunches), and their operations as given in the usage
//  guide. now the parser also recognizes the various forms of
//  function declarations and function usage. The code compiles, but
//  has not been tested comprehensively as yet. 
//
//  Revision 1.5  2000/01/21 05:53:28  cvs
//  updated by krishna -
//  added new types and operatots into the standard symbol table, in
//  order to account for all the basic types as defined in the usage
//  guide. 
//
//  Revision 1.4  2000/01/04 00:03:42  cvs
//  modified the implementation of type definitions for the basic data
//  types. more precisely, the parser will now recognize univ,
//  integer, real, boolean, natural and character types; as against
//  the previous types which were real, int, bool, nat and char. 
//
//  Revision 1.3  1999/11/15 03:16:30  cvs
//  Many changes - especially with unfolding of
//  overloading. Everything compiles now. Some features are yet to be
//  coded...but it is now again a rudimentary parser (as it was two
//  weeks ago :) 
//
//  Revision 1.2  1999/11/13 20:01:30  cvs
//  Changed to the new OM and grammar. Some more features are to be
//  added and many bugs to be corrected. This is a commit for record
//  and it compiles 
//
//  Revision 1.1  1999/10/27 20:43:56  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to
//  kansas. The SldlParser.jj still has holes - but committing it
//  anyway. 
//
//  Revision 1.1  1999/10/20 21:52:23  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse"
//  archive) before it moves to Kansas...
//  These files will have the log-info from my archive; they can be
//  ignored. 
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.9  1999/10/20 21:19:41  rkamath
//  Commit before the archive moves over to Kansas
//
//  Revision 1.8  1999/10/01 20:01:28  rkamath
//  For record..
//
//  Revision 1.7  1999/10/01 15:25:02  rkamath
//  working prototype commit
//
//  Revision 1.6  1999/08/01 20:56:33  rkamath
//  Cleaned up (code and my thinking too :) still struggling with how
//  to implement serialization etc to minimize sideeffects.. 
//
//  Revision 1.5  1999/07/28 02:01:14  rkamath
//  Coded my more clearer concepts. Commit for record
//
//  Revision 1.4  1999/07/22 22:51:25  rkamath
//  Intermediate commit for safety and records. This does not compile
//  because the toString() in SOM_FormalIdentifier has not been
//  completely coded 
//  Will do that tommorrow
//
//  Revision 1.3  1999/07/16 21:51:43  rkamath
//  Some small changes. Commit for records...
//
//  Revision 1.2  1999/07/14 23:26:38  rkamath
//  Commit for today. Introduced static functions in
//  SYM_SymboltableManager and SOM_BaseClass for cleaner segregation
//  of tasks and implementation 
//
//  Revision 1.1  1999/07/13 19:33:38  rkamath
//  Intermediate commit for records..
//
//
//====================================================================
package SYM;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.util.zip.GZIPOutputStream;
import java.util.zip.GZIPInputStream;

import SOM.SOM_Declaration;
import SOM.SOM_DeclarationList;
import SOM.SOM_Identifier;
import SOM.SOM_Name;
import SOM.SOM_FunctionDeclaration;
import SOM.SOM_FunctionType;
import SOM.SOM_SetType;
import SOM.SOM_SequenceType;
import SOM.SOM_ArrayType;
import SOM.SOM_RecordType;
import SOM.SOM_OperatorDeclaration;
import SOM.SOM_TypeDeclaration;
import SOM.SOM_VariableDeclaration;

// Added by Morel, June 30, 2001
import SOM.SOM_EnumerationType;
import SOM.SOM_EnumerationItemType;

import util.ListEnumeration;

/** This is the <B>SYM_SymboltableManager</B> object
 * Read the implementation notes for clarity
 * This will keep track of all the visible symbol tables and manage
 * them 
 * Lookup should be as follows
 * Look up current and previous tables in the file scope and return
 * the more recent declaration for any identifier etc.
 * If the file scope doesn't have the required object being looked for
 * then look in all the other imported tables and retrieve ALL the
 * declarations corresponding to the identifier
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class SYM_SymboltableManager
extends Object {

	private String myName;

	// The table floating around in the environment which will store
	// the standard types..Note that no symboltable should store the
	// standard types. Rather the manager must set the standardTable
	// in the importing list of all tables that it creates
	private SYM_Symboltable standardTable;

	private SYM_Symboltable currentTable = null;

	// Whenever a new symboltable is created corresponding to a new
	// design unit etc. it is put on the scopeTables list!
	private SYM_SymboltableList scopeTables = 
		new SYM_SymboltableList(); 

	// Here are the different type declarations that may be needed
	// later 
	SOM_TypeDeclaration bunchMetaType;
	SOM_TypeDeclaration facetMetaType;
	SOM_TypeDeclaration packageMetaType;
	SOM_TypeDeclaration componentMetaType;
        SOM_TypeDeclaration interactionMetaType;
        SOM_TypeDeclaration domainMetaType;
        SOM_TypeDeclaration typeMetaType;//Added by Murthy
	
	SOM_TypeDeclaration univDecl;           // added by krishna
	SOM_SetType setDecl;                    // added by Cindy
	SOM_ArrayType arrayDecl;                // added by Cindy
	SOM_SequenceType sequenceDecl;
	SOM_RecordType recordDecl;
	SOM_TypeDeclaration tupleDecl;
        SOM_EnumerationType enumerationDecl;     /// added by Morel
        SOM_EnumerationItemType enumItemDecl;    // added by Morel
	SOM_TypeDeclaration elementDecl;        // added by Cindy
	SOM_TypeDeclaration numberDecl;         // added by krishna
	SOM_TypeDeclaration realDecl;
	SOM_TypeDeclaration rationalDecl;       // added by krishna
	SOM_TypeDeclaration intDecl;
	SOM_TypeDeclaration natDecl;
	SOM_TypeDeclaration bitDecl;            // added by krishna
	SOM_TypeDeclaration boolDecl;
	SOM_TypeDeclaration underscoreType;     // added by Cindy
	
	SOM_ArrayType vectorDecl;               // added by krishna
	SOM_TypeDeclaration charDecl;
        SOM_TypeDeclaration domainDecl;

	SOM_VariableDeclaration nullDecl;       // added by Cindy
	SOM_VariableDeclaration trueDecl;       // added by Cindy
	SOM_VariableDeclaration falseDecl;      // added by Cindy
	SOM_VariableDeclaration nilDecl;        // added by Cindy
	SOM_VariableDeclaration underscoreDecl; // added by Cindy
        SOM_VariableDeclaration logic;          // added by Cindy
        SOM_VariableDeclaration state_based;    // added by ckong
	SOM_VariableDeclaration continuous;     // added by ckong
	SOM_VariableDeclaration discrete;       // added by ckong
	
	/** The primary constructor
	 * @param name The name for this object
	 * @return Not applicable
	 */
	public SYM_SymboltableManager(String name) {
		super();
		myName = name;
		initializeDefaultTable();
		createNewCurrentSymboltable(name, null); // The filescope
		// symboltable 
	}

	/** To initialize the standard table
	 * @return void
	 */
	private void initializeDefaultTable() {
		standardTable = new SYM_Symboltable("standard", null);

		// First the METATYPES...note that all these metatypes must be
		// keywords in the actual language
		bunchMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("bunchMetaType"), null);

		facetMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("facet"), null);
		standardTable.addDeclaration(facetMetaType);
		
		interactionMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("interaction"),null);
                standardTable.addDeclaration(interactionMetaType);

		domainMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("domain"),null);
		standardTable.addDeclaration(domainMetaType);

		componentMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("component"), null);
		standardTable.addDeclaration(componentMetaType);

		packageMetaType = new SOM_TypeDeclaration
			(new SOM_Identifier("package"), null);
		standardTable.addDeclaration(packageMetaType);

		// Now the univ as an uninterpreted type
		univDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("univ"), bunchMetaType);
		standardTable.addDeclaration(univDecl);
		
		//Added by Murthy
		typeMetaType = new SOM_TypeDeclaration
		        (new SOM_Identifier("type"),bunchMetaType);
		typeMetaType.setParentType(univDecl);
		standardTable.addDeclaration(typeMetaType);



		arrayDecl = new SOM_ArrayType(bunchMetaType);
		arrayDecl.setMemberType(univDecl);
		arrayDecl.setParentType(univDecl);

		setDecl = new SOM_SetType(bunchMetaType);
		setDecl.setMemberType(univDecl);
		setDecl.setParentType(univDecl);

		sequenceDecl = new SOM_SequenceType(bunchMetaType);
		sequenceDecl.setMemberType(univDecl);
		sequenceDecl.setParentType(univDecl);

		recordDecl = new SOM_RecordType(bunchMetaType);
		sequenceDecl.setParentType(univDecl);

		tupleDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("tuple"), bunchMetaType);
		sequenceDecl.setParentType(univDecl);

		// Added by Morel, June 30, 2001
		enumerationDecl = new SOM_EnumerationType(bunchMetaType);
		enumerationDecl.setMemberType(univDecl);
		enumerationDecl.setParentType(univDecl);

		enumItemDecl = new SOM_EnumerationItemType(bunchMetaType);
      		enumItemDecl.setParentType(univDecl);

		// The element as a subtype of univ
		elementDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("element"), bunchMetaType);
		elementDecl.setParentType(univDecl);
		standardTable.addDeclaration(elementDecl);
		
		// The number as a subtype of element
		numberDecl = new SOM_TypeDeclaration
		    (new SOM_Identifier("number"), bunchMetaType);
		numberDecl.setParentType(elementDecl);
		standardTable.addDeclaration(numberDecl);

	
		// Now the real as a subtype of number
		realDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("real"), bunchMetaType);
		realDecl.setParentType(numberDecl);
		standardTable.addDeclaration(realDecl);

		// The rational as a subtype of real
		rationalDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("rational"), bunchMetaType);
		rationalDecl.setParentType(realDecl);
		standardTable.addDeclaration(rationalDecl);

		// The integer as a subtype of real
		intDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("integer"), bunchMetaType);
		intDecl.setParentType(realDecl);
		standardTable.addDeclaration(intDecl);

		// The natural as a subtype of integer
		natDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("natural"), bunchMetaType);
		natDecl.setParentType(intDecl);
		standardTable.addDeclaration(natDecl);

		// The bit as a subtype of natural
		bitDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("bit"), bunchMetaType);
		bitDecl.setParentType(natDecl);
		standardTable.addDeclaration(bitDecl);

		// The boolean as a subtype of integer
		boolDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("boolean"), bunchMetaType);
		boolDecl.setParentType(intDecl);
		standardTable.addDeclaration(boolDecl);

		// The character type as uninterpreted
		charDecl = new SOM_TypeDeclaration
			(new SOM_Identifier("character"), bunchMetaType);
		charDecl.setParentType(elementDecl);
		standardTable.addDeclaration(charDecl);


		//  //The domain type as auninterpreted
//  		domainDecl = new SOM_TypeDeclaration
//  		    (new SOM_Identifier("domain"),bunchMetaType);
//  		domainDecl.setParentType(elementDecl);
//  		standardTable.addDeclaration(domainDecl);

		// The bitvector as a subtype of array
		SOM_ArrayType bitvectorType = 
			new SOM_ArrayType(new SOM_Identifier("bitvector"), bunchMetaType);
		bitvectorType.setMemberType(bitDecl);
		bitvectorType.setParentType(arrayDecl);
		standardTable.addDeclaration(bitvectorType);

		// The underscore type
		// This is a generic type, i.e. it can be replaced by any
		// predefined or user-defined types
	    underscoreType = new SOM_TypeDeclaration
			(new SOM_Identifier("underscoreType"), bunchMetaType);

        //---------------- The predefined functions ----------------//
		// The sequence function that defines a sequence
		SOM_FunctionType f_sequenceType = new SOM_FunctionType(bunchMetaType);
		f_sequenceType.addToDomainList(univDecl);
		f_sequenceType.setRange(sequenceDecl);
		SOM_FunctionDeclaration f_sequence = new SOM_FunctionDeclaration
				(new SOM_Identifier("sequence"), f_sequenceType);
		standardTable.addFuncDeclaration(f_sequence);

        // The sel function that acts as a filter 
		SOM_FunctionType f_selType = new SOM_FunctionType(bunchMetaType);
		f_selType.addToDomainList(univDecl);
		f_selType.setRange(univDecl);
		SOM_FunctionDeclaration f_sel = new SOM_FunctionDeclaration
			(new SOM_Identifier("sel"), f_selType);
		standardTable.addFuncDeclaration(f_sel);

		// Now the default operators. 
		// Ideally they should be read from a "prelude"
		//---------------- The FACET operators       ---------------//
		SOM_FunctionType f_andType = new SOM_FunctionType(bunchMetaType);
		f_andType.addToDomainList(facetMetaType);
		f_andType.addToDomainList(facetMetaType);
		f_andType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_andDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("and"), f_andType);
		standardTable.addFuncDeclaration(f_andDecl);

		SOM_FunctionType f_orType = new SOM_FunctionType(bunchMetaType);
		f_orType.addToDomainList(facetMetaType);
		f_orType.addToDomainList(facetMetaType);
		f_orType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_orDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("or"), f_orType);
		standardTable.addFuncDeclaration(f_orDecl);

		SOM_FunctionType f_notType = new SOM_FunctionType(bunchMetaType);
		f_notType.addToDomainList(facetMetaType);
		f_notType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_notDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("not"), f_notType);
		standardTable.addFuncDeclaration(f_notDecl);

		SOM_FunctionType f_impliesType = new SOM_FunctionType(bunchMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_impliesDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("implies"), f_impliesType);
		standardTable.addFuncDeclaration(f_impliesDecl);

		f_impliesType = new SOM_FunctionType(bunchMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.setRange(facetMetaType);
		f_impliesDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("=>"), f_impliesType);
		standardTable.addFuncDeclaration(f_impliesDecl);

		f_impliesType = new SOM_FunctionType(bunchMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.addToDomainList(facetMetaType);
		f_impliesType.setRange(facetMetaType);
		f_impliesDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("=<"), f_impliesType);
		standardTable.addFuncDeclaration(f_impliesDecl);

		SOM_FunctionType f_impliedbyType = new SOM_FunctionType(bunchMetaType);
		f_impliedbyType.addToDomainList(facetMetaType);
		f_impliedbyType.addToDomainList(facetMetaType);
		f_impliedbyType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_impliedbyDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("<="), f_impliedbyType);
		standardTable.addFuncDeclaration(f_impliedbyDecl);

		SOM_FunctionType f_iffType = new SOM_FunctionType(bunchMetaType);
		f_iffType.addToDomainList(facetMetaType);
		f_iffType.addToDomainList(facetMetaType);
		f_iffType.setRange(facetMetaType);
		SOM_OperatorDeclaration f_iffDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), f_iffType);
		standardTable.addFuncDeclaration(f_iffDecl);

		//----------- The bunch composition operators --------------//
		SOM_FunctionType bunchUnionType = new SOM_FunctionType(bunchMetaType);
		bunchUnionType.addToDomainList(univDecl);
		bunchUnionType.addToDomainList(univDecl);
		bunchUnionType.setRange(univDecl);
		SOM_OperatorDeclaration bunchUnionDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("++"), bunchUnionType);
		standardTable.addFuncDeclaration(bunchUnionDecl);

		SOM_FunctionType bunchIntersectionType = 
			new SOM_FunctionType(bunchMetaType);
		bunchIntersectionType.addToDomainList(univDecl);
		bunchIntersectionType.addToDomainList(univDecl);
		bunchIntersectionType.setRange(univDecl);
		SOM_OperatorDeclaration bunchIntersectionDecl = 
			new SOM_OperatorDeclaration(new SOM_Identifier("**"), 
										bunchIntersectionType);
		standardTable.addFuncDeclaration(bunchIntersectionDecl);

		SOM_FunctionType bunchDiffType = new SOM_FunctionType(bunchMetaType);
		bunchDiffType.addToDomainList(univDecl);
		bunchDiffType.addToDomainList(univDecl);
		bunchDiffType.setRange(univDecl);
		SOM_OperatorDeclaration bunchDiffDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("--"), bunchDiffType);
		standardTable.addFuncDeclaration(bunchDiffDecl);

		SOM_FunctionType subBunchLessType = 
			new SOM_FunctionType(bunchMetaType);
		subBunchLessType.addToDomainList(univDecl);
		subBunchLessType.addToDomainList(univDecl);
		subBunchLessType.setRange(univDecl);
		SOM_OperatorDeclaration subBunchLessDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("<<"), subBunchLessType);
		standardTable.addFuncDeclaration(subBunchLessDecl);

		SOM_FunctionType subBunchGreaterType = 
			new SOM_FunctionType(bunchMetaType);
		subBunchGreaterType.addToDomainList(univDecl);
		subBunchGreaterType.addToDomainList(univDecl);
		subBunchGreaterType.setRange(univDecl);
		SOM_OperatorDeclaration subBunchGreaterDecl = 
			new SOM_OperatorDeclaration(new SOM_Identifier(">>"), 
										subBunchGreaterType);
		standardTable.addFuncDeclaration(subBunchGreaterDecl);

		//----------------  The bunch operators      ---------------//
		SOM_FunctionType bunchEqualType = new SOM_FunctionType(bunchMetaType);
		bunchEqualType.addToDomainList(univDecl);
		bunchEqualType.addToDomainList(univDecl);
		bunchEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration bunchEqualDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), bunchEqualType);
		standardTable.addFuncDeclaration(bunchEqualDecl);

		SOM_FunctionType bunchNotEqualType = 
			new SOM_FunctionType(bunchMetaType);
		bunchNotEqualType.addToDomainList(univDecl);
		bunchNotEqualType.addToDomainList(univDecl);
		bunchNotEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration bunchNotEqualDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("/="), bunchNotEqualType);
		standardTable.addFuncDeclaration(bunchNotEqualDecl);

		SOM_FunctionType bunchSizeType = new SOM_FunctionType(bunchMetaType);
		bunchSizeType.addToDomainList(univDecl);
		bunchSizeType.setRange(natDecl);
		SOM_OperatorDeclaration bunchSizeDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("$"), bunchSizeType);
		standardTable.addFuncDeclaration(bunchSizeDecl);
		
		SOM_FunctionType bunchInclusionType = 
			new SOM_FunctionType(bunchMetaType);
		bunchInclusionType.addToDomainList(univDecl);
		bunchInclusionType.addToDomainList(univDecl);
		bunchInclusionType.setRange(boolDecl);
		SOM_OperatorDeclaration bunchInclusionDecl = 
			new SOM_OperatorDeclaration(new SOM_Identifier("::"), 
										bunchInclusionType);
		standardTable.addFuncDeclaration(bunchInclusionDecl);

		SOM_FunctionType memberType = new SOM_FunctionType(bunchMetaType);
		memberType.addToDomainList(univDecl);
		memberType.addToDomainList(bunchMetaType);
		memberType.setRange(boolDecl);
		SOM_OperatorDeclaration memberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("::"), memberType);
		standardTable.addFuncDeclaration(memberDecl);

		//---------------- The arithmetic operators  ---------------//
		//*** negation operator ***//
		SOM_FunctionType negNumberType = new SOM_FunctionType(bunchMetaType);
		negNumberType.addToDomainList(numberDecl);
		negNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration negNumberDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negNumberType);
		standardTable.addFuncDeclaration(negNumberDecl);

		SOM_FunctionType negRealType = new SOM_FunctionType(bunchMetaType);
		negRealType.addToDomainList(realDecl);
		negRealType.setRange(realDecl);
		SOM_OperatorDeclaration negRealDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negRealType);
		standardTable.addFuncDeclaration(negRealDecl);

		SOM_FunctionType negRationalType = new SOM_FunctionType(bunchMetaType);
		negRationalType.addToDomainList(rationalDecl);
		negRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration negRationalDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negRationalType);
		standardTable.addFuncDeclaration(negRationalDecl);

		SOM_FunctionType negIntegerType = new SOM_FunctionType(bunchMetaType);
		negIntegerType.addToDomainList(intDecl);
		negIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration negIntegerDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negIntegerType);
		standardTable.addFuncDeclaration(negIntegerDecl);

		SOM_FunctionType negNaturalType = new SOM_FunctionType(bunchMetaType);
		negNaturalType.addToDomainList(natDecl);
		negNaturalType.setRange(intDecl);
		SOM_OperatorDeclaration negNaturalDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negNaturalType);
		standardTable.addFuncDeclaration(negNaturalDecl);

		SOM_FunctionType negBitType = new SOM_FunctionType(bunchMetaType);
		negBitType.addToDomainList(bitDecl);
		negBitType.setRange(bitDecl);
		SOM_OperatorDeclaration negBitDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negBitType);
		standardTable.addFuncDeclaration(negBitDecl);

		SOM_FunctionType negBooleanType = new SOM_FunctionType(bunchMetaType);
		negBooleanType.addToDomainList(boolDecl);
		negBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration negBooleanDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), negBooleanType);
		standardTable.addFuncDeclaration(negBooleanDecl);

		//*** min operator ***//
		SOM_FunctionType minNumberType = new SOM_FunctionType(bunchMetaType);
		minNumberType.addToDomainList(numberDecl);
		minNumberType.addToDomainList(numberDecl);
		minNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration minNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minNumberType);
		standardTable.addFuncDeclaration(minNumberDecl);

		SOM_FunctionType minRealType = new SOM_FunctionType(bunchMetaType);
		minRealType.addToDomainList(realDecl);
		minRealType.addToDomainList(realDecl);
		minRealType.setRange(realDecl);
		SOM_OperatorDeclaration minRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minRealType);
		standardTable.addFuncDeclaration(minRealDecl);

		SOM_FunctionType minRationalType = new SOM_FunctionType(bunchMetaType);
		minRationalType.addToDomainList(rationalDecl);
		minRationalType.addToDomainList(rationalDecl);
		minRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration minRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minRationalType);
		standardTable.addFuncDeclaration(minRationalDecl);

		SOM_FunctionType minIntegerType = new SOM_FunctionType(bunchMetaType);
		minIntegerType.addToDomainList(intDecl);
		minIntegerType.addToDomainList(intDecl);
		minIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration minIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minIntegerType);
		standardTable.addFuncDeclaration(minIntegerDecl);

		SOM_FunctionType minNaturalType = new SOM_FunctionType(bunchMetaType);
		minNaturalType.addToDomainList(natDecl);
		minNaturalType.addToDomainList(natDecl);
		minNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration minNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minNaturalType);
		standardTable.addFuncDeclaration(minNaturalDecl);

		SOM_FunctionType minBitType = new SOM_FunctionType(bunchMetaType);
		minBitType.addToDomainList(bitDecl);
		minBitType.addToDomainList(bitDecl);
		minBitType.setRange(bitDecl);
		SOM_OperatorDeclaration minBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minBitType);
		standardTable.addFuncDeclaration(minBitDecl);

		SOM_FunctionType minBooleanType = new SOM_FunctionType(bunchMetaType);
		minBooleanType.addToDomainList(boolDecl);
		minBooleanType.addToDomainList(boolDecl);
		minBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration minBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("min"), minBooleanType);
		standardTable.addFuncDeclaration(minBooleanDecl);

		//*** max operator ***//
		SOM_FunctionType maxNumberType = new SOM_FunctionType(bunchMetaType);
		maxNumberType.addToDomainList(numberDecl);
		maxNumberType.addToDomainList(numberDecl);
		maxNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration maxNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxNumberType);
		standardTable.addFuncDeclaration(maxNumberDecl);

		SOM_FunctionType maxRealType = new SOM_FunctionType(bunchMetaType);
		maxRealType.addToDomainList(realDecl);
		maxRealType.addToDomainList(realDecl);
		maxRealType.setRange(realDecl);
		SOM_OperatorDeclaration maxRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxRealType);
		standardTable.addFuncDeclaration(maxRealDecl);

		SOM_FunctionType maxRationalType = new SOM_FunctionType(bunchMetaType);
		maxRationalType.addToDomainList(rationalDecl);
		maxRationalType.addToDomainList(rationalDecl);
		maxRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration maxRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxRationalType);
		standardTable.addFuncDeclaration(maxRationalDecl);

		SOM_FunctionType maxIntegerType = new SOM_FunctionType(bunchMetaType);
		maxIntegerType.addToDomainList(intDecl);
		maxIntegerType.addToDomainList(intDecl);
		maxIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration maxIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxIntegerType);
		standardTable.addFuncDeclaration(maxIntegerDecl);

		SOM_FunctionType maxNaturalType = new SOM_FunctionType(bunchMetaType);
		maxNaturalType.addToDomainList(natDecl);
		maxNaturalType.addToDomainList(natDecl);
		maxNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration maxNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxNaturalType);
		standardTable.addFuncDeclaration(maxNaturalDecl);

		SOM_FunctionType maxBitType = new SOM_FunctionType(bunchMetaType);
		maxBitType.addToDomainList(bitDecl);
		maxBitType.addToDomainList(bitDecl);
		maxBitType.setRange(bitDecl);
		SOM_OperatorDeclaration maxBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxBitType);
		standardTable.addFuncDeclaration(maxBitDecl);

		SOM_FunctionType maxBooleanType = new SOM_FunctionType(bunchMetaType);
		maxBooleanType.addToDomainList(boolDecl);
		maxBooleanType.addToDomainList(boolDecl);
		maxBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration maxBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("max"), maxBooleanType);
		standardTable.addFuncDeclaration(maxBooleanDecl);

		//*** nmin operator ***//
		SOM_FunctionType nminNumberType = new SOM_FunctionType(bunchMetaType);
		nminNumberType.addToDomainList(numberDecl);
		nminNumberType.addToDomainList(numberDecl);
		nminNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration nminNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminNumberType);
		standardTable.addFuncDeclaration(nminNumberDecl);

		SOM_FunctionType nminRealType = new SOM_FunctionType(bunchMetaType);
		nminRealType.addToDomainList(realDecl);
		nminRealType.addToDomainList(realDecl);
		nminRealType.setRange(realDecl);
		SOM_OperatorDeclaration nminRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminRealType);
		standardTable.addFuncDeclaration(nminRealDecl);

		SOM_FunctionType nminRationalType = 
			new SOM_FunctionType(bunchMetaType);
		nminRationalType.addToDomainList(rationalDecl);
		nminRationalType.addToDomainList(rationalDecl);
		nminRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration nminRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminRationalType);
		standardTable.addFuncDeclaration(nminRationalDecl);

		SOM_FunctionType nminIntegerType = new SOM_FunctionType(bunchMetaType);
		nminIntegerType.addToDomainList(intDecl);
		nminIntegerType.addToDomainList(intDecl);
		nminIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration nminIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminIntegerType);
		standardTable.addFuncDeclaration(nminIntegerDecl);

		SOM_FunctionType nminNaturalType = new SOM_FunctionType(bunchMetaType);
		nminNaturalType.addToDomainList(natDecl);
		nminNaturalType.addToDomainList(natDecl);
		nminNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration nminNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminNaturalType);
		standardTable.addFuncDeclaration(nminNaturalDecl);

		SOM_FunctionType nminBitType = new SOM_FunctionType(bunchMetaType);
		nminBitType.addToDomainList(bitDecl);
		nminBitType.addToDomainList(bitDecl);
		nminBitType.setRange(bitDecl);
		SOM_OperatorDeclaration nminBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminBitType);
		standardTable.addFuncDeclaration(nminBitDecl);

		SOM_FunctionType nminBooleanType = new SOM_FunctionType(bunchMetaType);
		nminBooleanType.addToDomainList(boolDecl);
		nminBooleanType.addToDomainList(boolDecl);
		nminBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration nminBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmin"), nminBooleanType);
		standardTable.addFuncDeclaration(nminBooleanDecl);

		//*** nmax operator ***//
		SOM_FunctionType nmaxNumberType = new SOM_FunctionType(bunchMetaType);
		nmaxNumberType.addToDomainList(numberDecl);
		nmaxNumberType.addToDomainList(numberDecl);
		nmaxNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration nmaxNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxNumberType);
		standardTable.addFuncDeclaration(nmaxNumberDecl);

		SOM_FunctionType nmaxRealType = new SOM_FunctionType(bunchMetaType);
		nmaxRealType.addToDomainList(realDecl);
		nmaxRealType.addToDomainList(realDecl);
		nmaxRealType.setRange(realDecl);
		SOM_OperatorDeclaration nmaxRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxRealType);
		standardTable.addFuncDeclaration(nmaxRealDecl);

		SOM_FunctionType nmaxRationalType = 
			new SOM_FunctionType(bunchMetaType);
		nmaxRationalType.addToDomainList(rationalDecl);
		nmaxRationalType.addToDomainList(rationalDecl);
		nmaxRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration nmaxRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxRationalType);
		standardTable.addFuncDeclaration(nmaxRationalDecl);

		SOM_FunctionType nmaxIntegerType = new SOM_FunctionType(bunchMetaType);
		nmaxIntegerType.addToDomainList(intDecl);
		nmaxIntegerType.addToDomainList(intDecl);
		nmaxIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration nmaxIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxIntegerType);
		standardTable.addFuncDeclaration(nmaxIntegerDecl);

		SOM_FunctionType nmaxNaturalType = new SOM_FunctionType(bunchMetaType);
		nmaxNaturalType.addToDomainList(natDecl);
		nmaxNaturalType.addToDomainList(natDecl);
		nmaxNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration nmaxNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxNaturalType);
		standardTable.addFuncDeclaration(nmaxNaturalDecl);

		SOM_FunctionType nmaxBitType = new SOM_FunctionType(bunchMetaType);
		nmaxBitType.addToDomainList(bitDecl);
		nmaxBitType.addToDomainList(bitDecl);
		nmaxBitType.setRange(bitDecl);
		SOM_OperatorDeclaration nmaxBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxBitType);
		standardTable.addFuncDeclaration(nmaxBitDecl);

		SOM_FunctionType nmaxBooleanType = new SOM_FunctionType(bunchMetaType);
		nmaxBooleanType.addToDomainList(boolDecl);
		nmaxBooleanType.addToDomainList(boolDecl);
		nmaxBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration nmaxBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nmax"), nmaxBooleanType);
		standardTable.addFuncDeclaration(nmaxBooleanDecl);

		//*** plus operator ***//
		SOM_FunctionType plusNumberType = new SOM_FunctionType(bunchMetaType);
		plusNumberType.addToDomainList(numberDecl);
		plusNumberType.addToDomainList(numberDecl);
		plusNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration plusNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("+"), plusNumberType);
		standardTable.addFuncDeclaration(plusNumberDecl);

		SOM_FunctionType plusRealType = new SOM_FunctionType(bunchMetaType);
		plusRealType.addToDomainList(realDecl);
		plusRealType.addToDomainList(realDecl);
		plusRealType.setRange(realDecl);
		SOM_OperatorDeclaration plusRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("+"), plusRealType);
		standardTable.addFuncDeclaration(plusRealDecl);

		SOM_FunctionType plusRationalType = 
			new SOM_FunctionType(bunchMetaType);
		plusRationalType.addToDomainList(rationalDecl);
		plusRationalType.addToDomainList(rationalDecl);
		plusRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration plusRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("+"), plusRationalType);
		standardTable.addFuncDeclaration(plusRationalDecl);

		SOM_FunctionType plusIntegerType = new SOM_FunctionType(bunchMetaType);
		plusIntegerType.addToDomainList(intDecl);
		plusIntegerType.addToDomainList(intDecl);
		plusIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration plusIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("+"), plusIntegerType);
		standardTable.addFuncDeclaration(plusIntegerDecl);

		SOM_FunctionType plusNaturalType = new SOM_FunctionType(bunchMetaType);
		plusNaturalType.addToDomainList(natDecl);
		plusNaturalType.addToDomainList(natDecl);
		plusNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration plusNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("+"), plusNaturalType);
		standardTable.addFuncDeclaration(plusNaturalDecl);

		//*** minus operator ***//
		SOM_FunctionType minusNumberType = new SOM_FunctionType(bunchMetaType);
		minusNumberType.addToDomainList(numberDecl);
		minusNumberType.addToDomainList(numberDecl);
		minusNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration minusNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("-"), minusNumberType);
		standardTable.addFuncDeclaration(minusNumberDecl);

		SOM_FunctionType minusRealType = new SOM_FunctionType(bunchMetaType);
		minusRealType.addToDomainList(realDecl);
		minusRealType.addToDomainList(realDecl);
		minusRealType.setRange(realDecl);
		SOM_OperatorDeclaration minusRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("-"), minusRealType);
		standardTable.addFuncDeclaration(minusRealDecl);

		SOM_FunctionType minusRationalType = 
			new SOM_FunctionType(bunchMetaType);
		minusRationalType.addToDomainList(rationalDecl);
		minusRationalType.addToDomainList(rationalDecl);
		minusRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration minusRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("-"), minusRationalType);
		standardTable.addFuncDeclaration(minusRationalDecl);

		SOM_FunctionType minusIntegerType = 
			new	SOM_FunctionType(bunchMetaType); 
		minusIntegerType.addToDomainList(intDecl);
		minusIntegerType.addToDomainList(intDecl);
		minusIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration minusIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("-"), minusIntegerType);
		standardTable.addFuncDeclaration(minusIntegerDecl);

		SOM_FunctionType minusNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		minusNaturalType.addToDomainList(natDecl);
		minusNaturalType.addToDomainList(natDecl);
		minusNaturalType.setRange(intDecl);
		SOM_OperatorDeclaration minusNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("-"), minusNaturalType);
		standardTable.addFuncDeclaration(minusNaturalDecl);

		//*** multiply operator ***//
		SOM_FunctionType multiplyNumberType = 
			new SOM_FunctionType(bunchMetaType);
		multiplyNumberType.addToDomainList(numberDecl);
		multiplyNumberType.addToDomainList(numberDecl);
		multiplyNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration multiplyNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("*"), multiplyNumberType);
		standardTable.addFuncDeclaration(multiplyNumberDecl);

		SOM_FunctionType multiplyRealType = 
			new SOM_FunctionType(bunchMetaType);
		multiplyRealType.addToDomainList(realDecl);
		multiplyRealType.addToDomainList(realDecl);
		multiplyRealType.setRange(realDecl);
		SOM_OperatorDeclaration multiplyRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("*"), multiplyRealType);
		standardTable.addFuncDeclaration(multiplyRealDecl);

		SOM_FunctionType multiplyRationalType = 
			new SOM_FunctionType(bunchMetaType);
		multiplyRationalType.addToDomainList(rationalDecl);
		multiplyRationalType.addToDomainList(rationalDecl);
		multiplyRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration multiplyRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("*"), multiplyRationalType);
		standardTable.addFuncDeclaration(multiplyRationalDecl);

		SOM_FunctionType multiplyIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		multiplyIntegerType.addToDomainList(intDecl);
		multiplyIntegerType.addToDomainList(intDecl);
		multiplyIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration multiplyIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("*"), multiplyIntegerType);
		standardTable.addFuncDeclaration(multiplyIntegerDecl);

		SOM_FunctionType multiplyNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		multiplyNaturalType.addToDomainList(natDecl);
		multiplyNaturalType.addToDomainList(natDecl);
		multiplyNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration multiplyNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("*"), multiplyNaturalType);
		standardTable.addFuncDeclaration(multiplyNaturalDecl);

		//*** divide operator ***//
		SOM_FunctionType divideNumberType = 
			new SOM_FunctionType(bunchMetaType);
		divideNumberType.addToDomainList(numberDecl);
		divideNumberType.addToDomainList(numberDecl);
		divideNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration divideNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/"), divideNumberType);
		standardTable.addFuncDeclaration(divideNumberDecl);

		SOM_FunctionType divideRealType = new SOM_FunctionType(bunchMetaType);
		divideRealType.addToDomainList(realDecl);
		divideRealType.addToDomainList(realDecl);
		divideRealType.setRange(realDecl);
		SOM_OperatorDeclaration divideRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("/"), divideRealType);
		standardTable.addFuncDeclaration(divideRealDecl);

		SOM_FunctionType divideRationalType = 
			new SOM_FunctionType(bunchMetaType);
		divideRationalType.addToDomainList(rationalDecl);
		divideRationalType.addToDomainList(rationalDecl);
		divideRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration divideRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/"), divideRationalType);
		standardTable.addFuncDeclaration(divideRationalDecl);

		SOM_FunctionType divideIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		divideIntegerType.addToDomainList(intDecl);
		divideIntegerType.addToDomainList(intDecl);
		divideIntegerType.setRange(realDecl);
		SOM_OperatorDeclaration divideIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/"), divideIntegerType);
		standardTable.addFuncDeclaration(divideIntegerDecl);

		SOM_FunctionType divIntegerType =
			new SOM_FunctionType(bunchMetaType);
		divIntegerType.addToDomainList(intDecl);
		divIntegerType.addToDomainList(intDecl);
		divIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration divIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("div"), divIntegerType);
		standardTable.addFuncDeclaration(divIntegerDecl);

		SOM_FunctionType modIntegerType =
			new SOM_FunctionType(bunchMetaType);
		modIntegerType.addToDomainList(intDecl);
		modIntegerType.addToDomainList(intDecl);
		modIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration modIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("mod"), modIntegerType);
		standardTable.addFuncDeclaration(modIntegerDecl);

		SOM_FunctionType divideNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		divideNaturalType.addToDomainList(natDecl);
		divideNaturalType.addToDomainList(natDecl);
		divideNaturalType.setRange(realDecl);
		SOM_OperatorDeclaration divideNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/"), divideNaturalType);
		standardTable.addFuncDeclaration(divideNaturalDecl);

		//*** exponent operator ***//
		SOM_FunctionType exponentNumberType = 
			new SOM_FunctionType(bunchMetaType);
		exponentNumberType.addToDomainList(numberDecl);
		exponentNumberType.addToDomainList(numberDecl);
		exponentNumberType.setRange(numberDecl);
		SOM_OperatorDeclaration exponentNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("^"), exponentNumberType);
		standardTable.addFuncDeclaration(exponentNumberDecl);

		SOM_FunctionType exponentRealType = 
			new SOM_FunctionType(bunchMetaType);
		exponentRealType.addToDomainList(realDecl);
		exponentRealType.addToDomainList(realDecl);
		exponentRealType.setRange(realDecl);
		SOM_OperatorDeclaration exponentRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("^"), exponentRealType);
		standardTable.addFuncDeclaration(exponentRealDecl);

		SOM_FunctionType exponentRationalType = 
			new SOM_FunctionType(bunchMetaType);
		exponentRationalType.addToDomainList(rationalDecl);
		exponentRationalType.addToDomainList(rationalDecl);
		exponentRationalType.setRange(rationalDecl);
		SOM_OperatorDeclaration exponentRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("^"), exponentRationalType);
		standardTable.addFuncDeclaration(exponentRationalDecl);

		SOM_FunctionType exponentIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		exponentIntegerType.addToDomainList(intDecl);
		exponentIntegerType.addToDomainList(intDecl);
		exponentIntegerType.setRange(intDecl);
		SOM_OperatorDeclaration exponentIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("^"), exponentIntegerType);
		standardTable.addFuncDeclaration(exponentIntegerDecl);

		SOM_FunctionType exponentNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		exponentNaturalType.addToDomainList(natDecl);
		exponentNaturalType.addToDomainList(natDecl);
		exponentNaturalType.setRange(natDecl);
		SOM_OperatorDeclaration exponentNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("^"), exponentNaturalType);
		standardTable.addFuncDeclaration(exponentNaturalDecl);

		//------ The Relational operators over numbers -------------//         
		/*** equal operator ***/
		SOM_FunctionType eqNumberType = new SOM_FunctionType(bunchMetaType);
		eqNumberType.addToDomainList(numberDecl);
		eqNumberType.addToDomainList(numberDecl);
		eqNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration eqNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqNumberType);
		standardTable.addFuncDeclaration(eqNumberDecl);

		SOM_FunctionType eqRealType = new SOM_FunctionType(bunchMetaType);
		eqRealType.addToDomainList(realDecl);
		eqRealType.addToDomainList(realDecl);
		eqRealType.setRange(boolDecl);
		SOM_OperatorDeclaration eqRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqRealType);
		standardTable.addFuncDeclaration(eqRealDecl);

		SOM_FunctionType eqRationalType = new SOM_FunctionType(bunchMetaType);
		eqRationalType.addToDomainList(rationalDecl);
		eqRationalType.addToDomainList(rationalDecl);
		eqRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration eqRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqRationalType);
		standardTable.addFuncDeclaration(eqRationalDecl);

		SOM_FunctionType eqIntegerType = new SOM_FunctionType(bunchMetaType);
		eqIntegerType.addToDomainList(intDecl);
		eqIntegerType.addToDomainList(intDecl);
		eqIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration eqIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqIntegerType);
		standardTable.addFuncDeclaration(eqIntegerDecl);

		SOM_FunctionType eqNaturalType = new SOM_FunctionType(bunchMetaType);
		eqNaturalType.addToDomainList(natDecl);
		eqNaturalType.addToDomainList(natDecl);
		eqNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration eqNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqNaturalType);
		standardTable.addFuncDeclaration(eqNaturalDecl);

		SOM_FunctionType eqBitType = new SOM_FunctionType(bunchMetaType);
		eqBitType.addToDomainList(bitDecl);
		eqBitType.addToDomainList(bitDecl);
		eqBitType.setRange(boolDecl);
		SOM_OperatorDeclaration eqBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqBitType);
		standardTable.addFuncDeclaration(eqBitDecl);

		SOM_FunctionType eqBooleanType = new SOM_FunctionType(bunchMetaType);
		eqBooleanType.addToDomainList(boolDecl);
		eqBooleanType.addToDomainList(boolDecl);
		eqBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration eqBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("="), eqBooleanType);
		standardTable.addFuncDeclaration(eqBooleanDecl);

		/*** less than operator ***/
		SOM_FunctionType lessNumberType = new SOM_FunctionType(bunchMetaType);
		lessNumberType.addToDomainList(numberDecl);
		lessNumberType.addToDomainList(numberDecl);
		lessNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration lessNumberDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessNumberType);
		standardTable.addFuncDeclaration(lessNumberDecl);

		SOM_FunctionType lessRealType = new SOM_FunctionType(bunchMetaType);
		lessRealType.addToDomainList(realDecl);
		lessRealType.addToDomainList(realDecl);
		lessRealType.setRange(boolDecl);
		SOM_OperatorDeclaration lessRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessRealType);
		standardTable.addFuncDeclaration(lessRealDecl);

		SOM_FunctionType lessRationalType = 
			new SOM_FunctionType(bunchMetaType);
		lessRationalType.addToDomainList(rationalDecl);
		lessRationalType.addToDomainList(rationalDecl);
		lessRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration lessRationalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessRationalType);
		standardTable.addFuncDeclaration(lessRationalDecl);

		SOM_FunctionType lessIntegerType = new SOM_FunctionType(bunchMetaType);
		lessIntegerType.addToDomainList(intDecl);
		lessIntegerType.addToDomainList(intDecl);
		lessIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration lessIntegerDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessIntegerType);
		standardTable.addFuncDeclaration(lessIntegerDecl);

		SOM_FunctionType lessNaturalType = new SOM_FunctionType(bunchMetaType);
		lessNaturalType.addToDomainList(natDecl);
		lessNaturalType.addToDomainList(natDecl);
		lessNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration lessNaturalDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessNaturalType);
		standardTable.addFuncDeclaration(lessNaturalDecl);

		SOM_FunctionType lessBitType = new SOM_FunctionType(bunchMetaType);
		lessBitType.addToDomainList(bitDecl);
		lessBitType.addToDomainList(bitDecl);
		lessBitType.setRange(boolDecl);
		SOM_OperatorDeclaration lessBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessBitType);
		standardTable.addFuncDeclaration(lessBitDecl);

		SOM_FunctionType lessBooleanType = new SOM_FunctionType(bunchMetaType);
		lessBooleanType.addToDomainList(boolDecl);
		lessBooleanType.addToDomainList(boolDecl);
		lessBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration lessBooleanDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("<"), lessBooleanType);
		standardTable.addFuncDeclaration(lessBooleanDecl);

		/*** less than / equal operator ***/
		SOM_FunctionType lessEqualNumberType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualNumberType.addToDomainList(numberDecl);
		lessEqualNumberType.addToDomainList(numberDecl);
		lessEqualNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualNumberType);
		standardTable.addFuncDeclaration(lessEqualNumberDecl);

		SOM_FunctionType lessEqualRealType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualRealType.addToDomainList(realDecl);
		lessEqualRealType.addToDomainList(realDecl);
		lessEqualRealType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualRealDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualRealType);
		standardTable.addFuncDeclaration(lessEqualRealDecl);

		SOM_FunctionType lessEqualRationalType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualRationalType.addToDomainList(rationalDecl);
		lessEqualRationalType.addToDomainList(rationalDecl);
		lessEqualRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualRationalType);
		standardTable.addFuncDeclaration(lessEqualRationalDecl);

		SOM_FunctionType lessEqualIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualIntegerType.addToDomainList(intDecl);
		lessEqualIntegerType.addToDomainList(intDecl);
		lessEqualIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualIntegerType);
		standardTable.addFuncDeclaration(lessEqualIntegerDecl);

		SOM_FunctionType lessEqualNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualNaturalType.addToDomainList(natDecl);
		lessEqualNaturalType.addToDomainList(natDecl);
		lessEqualNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualNaturalType);
		standardTable.addFuncDeclaration(lessEqualNaturalDecl);

		SOM_FunctionType lessEqualBitType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualBitType.addToDomainList(bitDecl);
		lessEqualBitType.addToDomainList(bitDecl);
		lessEqualBitType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualBitDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualBitType);
		standardTable.addFuncDeclaration(lessEqualBitDecl);

		SOM_FunctionType lessEqualBooleanType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualBooleanType.addToDomainList(boolDecl);
		lessEqualBooleanType.addToDomainList(boolDecl);
		lessEqualBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualBooleanDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualBooleanType);
		standardTable.addFuncDeclaration(lessEqualBooleanDecl);

		/*** greater operator ***/
		SOM_FunctionType greaterNumberType = 
			new SOM_FunctionType(bunchMetaType);
		greaterNumberType.addToDomainList(numberDecl);
		greaterNumberType.addToDomainList(numberDecl);
		greaterNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterNumberType);
		standardTable.addFuncDeclaration(greaterNumberDecl);

		SOM_FunctionType greaterRealType = new SOM_FunctionType(bunchMetaType);
		greaterRealType.addToDomainList(realDecl);
		greaterRealType.addToDomainList(realDecl);
		greaterRealType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterRealDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier(">"), greaterRealType);
		standardTable.addFuncDeclaration(greaterRealDecl);

		SOM_FunctionType greaterRationalType = 
			new SOM_FunctionType(bunchMetaType);
		greaterRationalType.addToDomainList(rationalDecl);
		greaterRationalType.addToDomainList(rationalDecl);
		greaterRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterRationalType);
		standardTable.addFuncDeclaration(greaterRationalDecl);

		SOM_FunctionType greaterIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		greaterIntegerType.addToDomainList(intDecl);
		greaterIntegerType.addToDomainList(intDecl);
		greaterIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterIntegerType);
		standardTable.addFuncDeclaration(greaterIntegerDecl);

		SOM_FunctionType greaterNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		greaterNaturalType.addToDomainList(natDecl);
		greaterNaturalType.addToDomainList(natDecl);
		greaterNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterNaturalType);
		standardTable.addFuncDeclaration(greaterNaturalDecl);

		SOM_FunctionType greaterBitType = new SOM_FunctionType(bunchMetaType);
		greaterBitType.addToDomainList(bitDecl);
		greaterBitType.addToDomainList(bitDecl);
		greaterBitType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterBitDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterBitType);
		standardTable.addFuncDeclaration(greaterBitDecl);

		SOM_FunctionType greaterBooleanType = 
			new SOM_FunctionType(bunchMetaType);
		greaterBooleanType.addToDomainList(boolDecl);
		greaterBooleanType.addToDomainList(boolDecl);
		greaterBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterBooleanDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterBooleanType);
		standardTable.addFuncDeclaration(greaterBooleanDecl);

		/*** greater than / equal operator ***/
		SOM_FunctionType greaterEqualNumberType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualNumberType.addToDomainList(numberDecl);
		greaterEqualNumberType.addToDomainList(numberDecl);
		greaterEqualNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualNumberType);
		standardTable.addFuncDeclaration(greaterEqualNumberDecl);

		SOM_FunctionType greaterEqualRealType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualRealType.addToDomainList(realDecl);
		greaterEqualRealType.addToDomainList(realDecl);
		greaterEqualRealType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualRealDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualRealType);
		standardTable.addFuncDeclaration(greaterEqualRealDecl);

		SOM_FunctionType greaterEqualRationalType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualRationalType.addToDomainList(rationalDecl);
		greaterEqualRationalType.addToDomainList(rationalDecl);
		greaterEqualRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualRationalType);
		standardTable.addFuncDeclaration(greaterEqualRationalDecl);

		SOM_FunctionType greaterEqualIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualIntegerType.addToDomainList(intDecl);
		greaterEqualIntegerType.addToDomainList(intDecl);
		greaterEqualIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualIntegerType);
		standardTable.addFuncDeclaration(greaterEqualIntegerDecl);

		SOM_FunctionType greaterEqualNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualNaturalType.addToDomainList(natDecl);
		greaterEqualNaturalType.addToDomainList(natDecl);
		greaterEqualNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualNaturalType);
		standardTable.addFuncDeclaration(greaterEqualNaturalDecl);

		SOM_FunctionType greaterEqualBitType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualBitType.addToDomainList(bitDecl);
		greaterEqualBitType.addToDomainList(bitDecl);
		greaterEqualBitType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualBitDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualBitType);
		standardTable.addFuncDeclaration(greaterEqualBitDecl);

		SOM_FunctionType greaterEqualBooleanType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualBooleanType.addToDomainList(boolDecl);
		greaterEqualBooleanType.addToDomainList(boolDecl);
		greaterEqualBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualBooleanDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), greaterEqualBooleanType);
		standardTable.addFuncDeclaration(greaterEqualBooleanDecl);

		/*** equal equal operator ***/
		SOM_FunctionType equalEqualNumberType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualNumberType.addToDomainList(numberDecl);
		equalEqualNumberType.addToDomainList(numberDecl);
		equalEqualNumberType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualNumberDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualNumberType);
		standardTable.addFuncDeclaration(equalEqualNumberDecl);

		SOM_FunctionType equalEqualRealType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualRealType.addToDomainList(realDecl);
		equalEqualRealType.addToDomainList(realDecl);
		equalEqualRealType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualRealDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualRealType);
		standardTable.addFuncDeclaration(equalEqualRealDecl);

		SOM_FunctionType equalEqualRationalType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualRationalType.addToDomainList(rationalDecl);
		equalEqualRationalType.addToDomainList(rationalDecl);
		equalEqualRationalType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualRationalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualRationalType);
		standardTable.addFuncDeclaration(equalEqualRationalDecl);

		SOM_FunctionType equalEqualIntegerType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualIntegerType.addToDomainList(intDecl);
		equalEqualIntegerType.addToDomainList(intDecl);
		equalEqualIntegerType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualIntegerDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualIntegerType);
		standardTable.addFuncDeclaration(equalEqualIntegerDecl);

		SOM_FunctionType equalEqualNaturalType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualNaturalType.addToDomainList(natDecl);
		equalEqualNaturalType.addToDomainList(natDecl);
		equalEqualNaturalType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualNaturalDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualNaturalType);
		standardTable.addFuncDeclaration(equalEqualNaturalDecl);

		SOM_FunctionType equalEqualBitType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualBitType.addToDomainList(bitDecl);
		equalEqualBitType.addToDomainList(bitDecl);
		equalEqualBitType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualBitDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualBitType);
		standardTable.addFuncDeclaration(equalEqualBitDecl);

		SOM_FunctionType equalEqualBooleanType = 
			new SOM_FunctionType(bunchMetaType);
		equalEqualBooleanType.addToDomainList(boolDecl);
		equalEqualBooleanType.addToDomainList(boolDecl);
		equalEqualBooleanType.setRange(boolDecl);
		SOM_OperatorDeclaration equalEqualBooleanDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=="), equalEqualBooleanType);
		standardTable.addFuncDeclaration(equalEqualBooleanDecl);

		//----------------  The arithmetic functions ---------------//
        //*** abs function ***//
		SOM_FunctionType absType = new SOM_FunctionType(bunchMetaType);
		absType.addToDomainList(numberDecl);
		absType.setRange(numberDecl);
		SOM_FunctionDeclaration absNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("abs"), absType);
		standardTable.addFuncDeclaration(absNumberDecl);

		absType = new SOM_FunctionType(bunchMetaType);
		absType.addToDomainList(realDecl);
		absType.setRange(realDecl);
		SOM_FunctionDeclaration absRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("abs"), absType);
		standardTable.addFuncDeclaration(absRealDecl);

		absType = new SOM_FunctionType(bunchMetaType);
		absType.addToDomainList(rationalDecl);
		absType.setRange(rationalDecl);
		SOM_FunctionDeclaration absRationalDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("abs"), absType);
		standardTable.addFuncDeclaration(absRationalDecl);

		absType = new SOM_FunctionType(bunchMetaType);
		absType.addToDomainList(intDecl);
		absType.setRange(intDecl);
		SOM_FunctionDeclaration absIntegerDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("abs"), absType);
		standardTable.addFuncDeclaration(absIntegerDecl);

		absType = new SOM_FunctionType(bunchMetaType);
		absType.addToDomainList(natDecl);
		absType.setRange(natDecl);
		SOM_FunctionDeclaration absNaturalDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("abs"), absType);
		standardTable.addFuncDeclaration(absNaturalDecl);

        //*** sqrt function ***//
		SOM_FunctionType sqrtType = new	SOM_FunctionType(bunchMetaType);
		sqrtType.addToDomainList(numberDecl);
		sqrtType.setRange(numberDecl);
		SOM_FunctionDeclaration sqrtNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("sqrt"), sqrtType);
		standardTable.addFuncDeclaration(sqrtNumberDecl);

		sqrtType = new	SOM_FunctionType(bunchMetaType);
		sqrtType.addToDomainList(realDecl);
		sqrtType.setRange(realDecl);
		SOM_FunctionDeclaration sqrtRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("sqrt"), sqrtType);
		standardTable.addFuncDeclaration(sqrtRealDecl);

		sqrtType = new	SOM_FunctionType(bunchMetaType);
		sqrtType.addToDomainList(rationalDecl);
		sqrtType.setRange(rationalDecl);
		SOM_FunctionDeclaration sqrtRationalDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("sqrt"), sqrtType);
		standardTable.addFuncDeclaration(sqrtRationalDecl);

		sqrtType = new	SOM_FunctionType(bunchMetaType);
		sqrtType.addToDomainList(intDecl);
		sqrtType.setRange(intDecl);
		SOM_FunctionDeclaration sqrtIntegerDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("sqrt"), sqrtType);
		standardTable.addFuncDeclaration(sqrtIntegerDecl);

		sqrtType = new	SOM_FunctionType(bunchMetaType);
		sqrtType.addToDomainList(natDecl);
		sqrtType.setRange(natDecl);
		SOM_FunctionDeclaration sqrtNaturalDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("sqrt"), sqrtType);
		standardTable.addFuncDeclaration(sqrtNaturalDecl);

		//*** sin functions ***//
		SOM_FunctionType sinType = new SOM_FunctionType(bunchMetaType);
		sinType.addToDomainList(numberDecl);
		sinType.setRange(numberDecl);
		SOM_FunctionDeclaration sinNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("sin"), sinType);
		standardTable.addFuncDeclaration(sinNumberDecl);

		sinType = new SOM_FunctionType(bunchMetaType);
		sinType.addToDomainList(realDecl);
		sinType.setRange(realDecl);
		SOM_FunctionDeclaration sinRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("sin"), sinType);
		standardTable.addFuncDeclaration(sinRealDecl);

		//*** cos function ***//			
		SOM_FunctionType cosType = new SOM_FunctionType(bunchMetaType);
		cosType.addToDomainList(numberDecl);
		cosType.setRange(numberDecl);
		SOM_FunctionDeclaration cosNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("cos"), cosType);
		standardTable.addFuncDeclaration(cosNumberDecl);
		
		cosType = new SOM_FunctionType(bunchMetaType);
		cosType.addToDomainList(realDecl);
		cosType.setRange(realDecl);
		SOM_FunctionDeclaration cosRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("cos"), cosType);
		standardTable.addFuncDeclaration(cosRealDecl);

		//*** tan function ***///
		SOM_FunctionType tanType = new SOM_FunctionType(bunchMetaType);
		tanType.addToDomainList(numberDecl);
		tanType.setRange(numberDecl);
		SOM_FunctionDeclaration tanNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("tan"), tanType);
		standardTable.addFuncDeclaration(tanNumberDecl);

		tanType = new SOM_FunctionType(bunchMetaType);
		tanType.addToDomainList(realDecl);
		tanType.setRange(realDecl);
		SOM_FunctionDeclaration tanRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("tan"), tanType);
		standardTable.addFuncDeclaration(tanRealDecl);

		//*** exp function ***//
		SOM_FunctionType expType = new SOM_FunctionType(bunchMetaType);
		expType.addToDomainList(numberDecl);
		expType.setRange(numberDecl);
		SOM_FunctionDeclaration expNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("exp"), expType);
		standardTable.addFuncDeclaration(expNumberDecl);

		expType = new SOM_FunctionType(bunchMetaType);
		expType.addToDomainList(realDecl);
		expType.setRange(realDecl);
		SOM_FunctionDeclaration expRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("exp"), expType);
		standardTable.addFuncDeclaration(expRealDecl);

		//*** log function ***//
		SOM_FunctionType logType = new SOM_FunctionType(bunchMetaType);
		logType.addToDomainList(numberDecl);
		logType.setRange(numberDecl);
		SOM_FunctionDeclaration logNumberDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("log"), logType);
		standardTable.addFuncDeclaration(logNumberDecl);
		
		logType = new SOM_FunctionType(bunchMetaType);
		logType.addToDomainList(realDecl);
		logType.setRange(realDecl);
		SOM_FunctionDeclaration logRealDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("log"), logType);
		standardTable.addFuncDeclaration(logRealDecl);
		
		//--------- The logical operators (on boolean) -------------//
		SOM_FunctionType andType = new SOM_FunctionType(bunchMetaType);
		andType.addToDomainList(boolDecl);
		andType.addToDomainList(boolDecl);
		andType.setRange(boolDecl);
		SOM_OperatorDeclaration andDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("and"), andType);
		standardTable.addFuncDeclaration(andDecl);

		SOM_FunctionType nandType = new SOM_FunctionType(bunchMetaType);
		nandType.addToDomainList(boolDecl);
		nandType.addToDomainList(boolDecl);
		nandType.setRange(boolDecl);
		SOM_OperatorDeclaration nandDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nand"), nandType);
		standardTable.addFuncDeclaration(nandDecl);

		SOM_FunctionType orType = new SOM_FunctionType(bunchMetaType);
		orType.addToDomainList(boolDecl);
		orType.addToDomainList(boolDecl);
		orType.setRange(boolDecl);
		SOM_OperatorDeclaration orDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("or"), orType);
		standardTable.addFuncDeclaration(orDecl);

		SOM_FunctionType norType = new SOM_FunctionType(bunchMetaType);
		norType.addToDomainList(boolDecl);
		norType.addToDomainList(boolDecl);
		norType.setRange(boolDecl);
		SOM_OperatorDeclaration norDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("nor"), norType);
		standardTable.addFuncDeclaration(norDecl);

		SOM_FunctionType xorType = new SOM_FunctionType(bunchMetaType);
		xorType.addToDomainList(boolDecl);
		xorType.addToDomainList(boolDecl);
		xorType.setRange(boolDecl);
		SOM_OperatorDeclaration xorDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("xor"), xorType);
		standardTable.addFuncDeclaration(xorDecl);

		SOM_FunctionType xnorType = new SOM_FunctionType(bunchMetaType);
		xnorType.addToDomainList(boolDecl);
		xnorType.addToDomainList(boolDecl);
		xnorType.setRange(boolDecl);
		SOM_OperatorDeclaration xnorDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("xnor"), xnorType);
		standardTable.addFuncDeclaration(xnorDecl);

		SOM_FunctionType impliesType = new SOM_FunctionType(bunchMetaType);
		impliesType.addToDomainList(boolDecl);
		impliesType.addToDomainList(boolDecl);
		impliesType.setRange(boolDecl);
		SOM_OperatorDeclaration impliesDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("implies"), impliesType);
		standardTable.addFuncDeclaration(impliesDecl);

		impliesType = new SOM_FunctionType(bunchMetaType);
		impliesType.addToDomainList(boolDecl);
		impliesType.addToDomainList(boolDecl);
		impliesType.setRange(boolDecl);
		impliesDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("=>"), impliesType);
		standardTable.addFuncDeclaration(impliesDecl);

		SOM_FunctionType impliedbyType = new SOM_FunctionType(bunchMetaType);
		impliedbyType.addToDomainList(boolDecl);
		impliedbyType.addToDomainList(boolDecl);
		impliedbyType.setRange(boolDecl);
		SOM_OperatorDeclaration impliedbyDecl = 
			new SOM_OperatorDeclaration(new SOM_Identifier("<="), 
										impliedbyType);
		standardTable.addFuncDeclaration(impliedbyDecl);

		SOM_FunctionType notType = new SOM_FunctionType(bunchMetaType);
		notType.addToDomainList(boolDecl);
		notType.setRange(boolDecl);
		SOM_OperatorDeclaration notDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("not"), notType);
		standardTable.addFuncDeclaration(notDecl);

		//------------- The boolean to bit operator  ---------------//
		SOM_FunctionType convertType = new SOM_FunctionType(bunchMetaType);
		convertType.addToDomainList(boolDecl);
		convertType.setRange(bitDecl);
		SOM_OperatorDeclaration convertDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("%"), convertType);
		standardTable.addFuncDeclaration(convertDecl);

		convertType = new SOM_FunctionType(bunchMetaType);
		convertType.addToDomainList(bitDecl);
		convertType.setRange(boolDecl);
		convertDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("%"), convertType);
		standardTable.addFuncDeclaration(convertDecl);

		SOM_FunctionType andBitType = new SOM_FunctionType(bunchMetaType);
		andBitType.addToDomainList(bitDecl);
		andBitType.addToDomainList(bitDecl);
		andBitType.setRange(bitDecl);
		SOM_OperatorDeclaration andBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("and"), andBitType);
		standardTable.addFuncDeclaration(andBitDecl);		

		SOM_FunctionType orBitType = new SOM_FunctionType(bunchMetaType);
		orBitType.addToDomainList(bitDecl);
		orBitType.addToDomainList(bitDecl);
		orBitType.setRange(bitDecl);
		SOM_OperatorDeclaration orBitDecl = new SOM_OperatorDeclaration
			(new SOM_Identifier("or"), orBitType);
		standardTable.addFuncDeclaration(orBitDecl);		

		//------------- The character operators    -----------------//
		SOM_FunctionType equalCharType = new SOM_FunctionType(bunchMetaType);
		equalCharType.addToDomainList(charDecl);
		equalCharType.addToDomainList(charDecl);
		equalCharType.setRange(boolDecl);
		SOM_OperatorDeclaration equalCharDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("="), equalCharType);
		standardTable.addFuncDeclaration(equalCharDecl);

		SOM_FunctionType notEqualCharType = 
			new SOM_FunctionType(bunchMetaType);
		notEqualCharType.addToDomainList(charDecl);
		notEqualCharType.addToDomainList(charDecl);
		notEqualCharType.setRange(boolDecl);
		SOM_OperatorDeclaration notEqualCharDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/="), notEqualCharType);
		standardTable.addFuncDeclaration(notEqualCharDecl);

		SOM_FunctionType lessCharType = new SOM_FunctionType(bunchMetaType);
		lessCharType.addToDomainList(charDecl);
		lessCharType.addToDomainList(charDecl);
		lessCharType.setRange(boolDecl);
		SOM_OperatorDeclaration lessCharDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("<"), lessCharType);
		standardTable.addFuncDeclaration(lessCharDecl);

		SOM_FunctionType lessEqualCharType = 
			new SOM_FunctionType(bunchMetaType);
		lessEqualCharType.addToDomainList(charDecl);
		lessEqualCharType.addToDomainList(charDecl);
		lessEqualCharType.setRange(boolDecl);
		SOM_OperatorDeclaration lessEqualCharDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), lessEqualCharType);
		standardTable.addFuncDeclaration(lessEqualCharDecl);

		SOM_FunctionType greaterEqualCharType = 
			new SOM_FunctionType(bunchMetaType);
		greaterEqualCharType.addToDomainList(charDecl);
		greaterEqualCharType.addToDomainList(charDecl);
		greaterEqualCharType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterEqualCharDecl = 
			new SOM_OperatorDeclaration(new SOM_Identifier(">="), 
										greaterEqualCharType);
		standardTable.addFuncDeclaration(greaterEqualCharDecl);

		SOM_FunctionType greaterCharType = new SOM_FunctionType(bunchMetaType);
		greaterCharType.addToDomainList(charDecl);
		greaterCharType.addToDomainList(charDecl);
		greaterCharType.setRange(boolDecl);
		SOM_OperatorDeclaration greaterCharDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), greaterCharType);
		standardTable.addFuncDeclaration(greaterCharDecl);

		//------------- The character functions  -------------------//
		SOM_FunctionType ucType = new SOM_FunctionType(bunchMetaType);
		ucType.addToDomainList(charDecl);
		ucType.setRange(charDecl);
		SOM_FunctionDeclaration ucDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("uc"), ucType);
		standardTable.addFuncDeclaration(ucDecl);

		SOM_FunctionType dcType = new SOM_FunctionType(bunchMetaType);
		dcType.addToDomainList(charDecl);
		dcType.setRange(charDecl);
		SOM_FunctionDeclaration dcDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("dc"), dcType);
		standardTable.addFuncDeclaration(dcDecl);

		SOM_FunctionType ordType = new SOM_FunctionType(bunchMetaType);
		ordType.addToDomainList(charDecl);
		ordType.setRange(natDecl);
		SOM_FunctionDeclaration ordDecl = new SOM_FunctionDeclaration
			(new SOM_Identifier("ord"), ordType);
		standardTable.addFuncDeclaration(ordDecl);

		SOM_FunctionType charOpType = new SOM_FunctionType(bunchMetaType);
		charOpType.addToDomainList(natDecl);
		charOpType.setRange(charDecl);
		SOM_FunctionDeclaration charOpDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("char"), charOpType);
		standardTable.addFuncDeclaration(charOpDecl);
		
		//------------- The set operators     ----------------------//
		SOM_FunctionType setIntersectionType = 
			new SOM_FunctionType(bunchMetaType);
		setIntersectionType.addToDomainList(setDecl);
		setIntersectionType.addToDomainList(setDecl);
		setIntersectionType.setRange(setDecl);
		SOM_OperatorDeclaration setIntersectionDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("*"), setIntersectionType);
		standardTable.addFuncDeclaration(setIntersectionDecl);

		SOM_FunctionType setUnionType = new SOM_FunctionType(bunchMetaType);
		setUnionType.addToDomainList(setDecl);
		setUnionType.addToDomainList(setDecl);
		setUnionType.setRange(setDecl);
		SOM_OperatorDeclaration setUnionDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("+"), setUnionType);
		standardTable.addFuncDeclaration(setUnionDecl);

		SOM_FunctionType setDifferenceType = 
			new SOM_FunctionType(bunchMetaType);
		setDifferenceType.addToDomainList(setDecl);
		setDifferenceType.addToDomainList(setDecl);
		setDifferenceType.setRange(setDecl);
		SOM_OperatorDeclaration setDifferenceDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("-"), setDifferenceType);
		standardTable.addFuncDeclaration(setDifferenceDecl);
		
		SOM_FunctionType setPowerType = new SOM_FunctionType(bunchMetaType);
		setPowerType.addToDomainList(setDecl);
		setPowerType.setRange(setDecl);
		SOM_FunctionDeclaration setPowerDecl = 
			new SOM_FunctionDeclaration
				(new SOM_Identifier("power"), setPowerType);
		standardTable.addFuncDeclaration(setPowerDecl);

		SOM_FunctionType setInType = new SOM_FunctionType(bunchMetaType);
		setInType.addToDomainList(univDecl);
		setInType.addToDomainList(setDecl);
		setInType.setRange(boolDecl);
		SOM_OperatorDeclaration setInDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("in"), setInType); 
		standardTable.addFuncDeclaration(setInDecl);

		SOM_FunctionType setEqualType = new SOM_FunctionType(bunchMetaType);
		setEqualType.addToDomainList(setDecl);
		setEqualType.addToDomainList(setDecl);
		setEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration setEqualDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("="), setEqualType); 
		standardTable.addFuncDeclaration(setEqualDecl);

		SOM_FunctionType setNotEqualType = new SOM_FunctionType(bunchMetaType);
		setNotEqualType.addToDomainList(setDecl);
		setNotEqualType.addToDomainList(setDecl);
		setNotEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration setNotEqualDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("/="), setNotEqualType); 
		standardTable.addFuncDeclaration(setNotEqualDecl);

		SOM_FunctionType setLessType = new SOM_FunctionType(bunchMetaType);
		setLessType.addToDomainList(setDecl);
		setLessType.addToDomainList(setDecl);
		setLessType.setRange(boolDecl);
		SOM_OperatorDeclaration setLessDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("<"), setLessType); 
		standardTable.addFuncDeclaration(setLessDecl);

		SOM_FunctionType setLessEqualType = 
			new SOM_FunctionType(bunchMetaType);
		setLessEqualType.addToDomainList(setDecl);
		setLessEqualType.addToDomainList(setDecl);
		setLessEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration setLessEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), setLessEqualType); 
		standardTable.addFuncDeclaration(setLessEqualDecl);

		SOM_FunctionType setGreaterType = new SOM_FunctionType(bunchMetaType);
		setGreaterType.addToDomainList(setDecl);
		setGreaterType.addToDomainList(setDecl);
		setGreaterType.setRange(boolDecl);
		SOM_OperatorDeclaration setGreaterDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), setGreaterType); 
		standardTable.addFuncDeclaration(setGreaterDecl);

		SOM_FunctionType setGreaterEqualType = 
			new SOM_FunctionType(bunchMetaType);
		setGreaterEqualType.addToDomainList(setDecl);
		setGreaterEqualType.addToDomainList(setDecl);
		setGreaterEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration setGreaterEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), setGreaterEqualType); 
		standardTable.addFuncDeclaration(setGreaterEqualDecl);
		
		SOM_FunctionType setSizeType = new SOM_FunctionType(bunchMetaType);
		setSizeType.addToDomainList(setDecl);
		setSizeType.setRange(intDecl);
		SOM_OperatorDeclaration setSizeDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("#"), setSizeType); 
		standardTable.addFuncDeclaration(setSizeDecl);
		
		SOM_FunctionType setTildeType = new SOM_FunctionType(bunchMetaType);
		setTildeType.addToDomainList(setDecl);
		setTildeType.setRange(univDecl);
		SOM_OperatorDeclaration setTildeDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("~"), setTildeType); 
		standardTable.addFuncDeclaration(setTildeDecl);

		//-------------- The sequence operators   ------------------//
		SOM_FunctionType catenationType = new SOM_FunctionType(bunchMetaType);
		catenationType.addToDomainList(univDecl);
		catenationType.addToDomainList(univDecl);
		catenationType.setRange(sequenceDecl);
		SOM_OperatorDeclaration catenationDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier(";;"), catenationType);
		standardTable.addFuncDeclaration(catenationDecl);

		SOM_FunctionType subscriptionType = 
			new SOM_FunctionType(bunchMetaType);
		subscriptionType.addToDomainList(sequenceDecl);
		subscriptionType.addToDomainList(sequenceDecl);
		subscriptionType.setRange(sequenceDecl);
		SOM_OperatorDeclaration subscriptionDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("sub"), subscriptionType);
		standardTable.addFuncDeclaration(subscriptionDecl);
		
		SOM_FunctionType sequenceEqualType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceEqualType.addToDomainList(sequenceDecl);
		sequenceEqualType.addToDomainList(sequenceDecl);
		sequenceEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceEqualDecl = 
			new SOM_OperatorDeclaration
			(new SOM_Identifier("="), sequenceEqualType);
		standardTable.addFuncDeclaration(sequenceEqualDecl);

		SOM_FunctionType sequenceNotEqualType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceNotEqualType.addToDomainList(sequenceDecl);
		sequenceNotEqualType.addToDomainList(sequenceDecl);
		sequenceNotEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceNotEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/="), sequenceNotEqualType);
		standardTable.addFuncDeclaration(sequenceNotEqualDecl);

		SOM_FunctionType sequenceLessType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceLessType.addToDomainList(sequenceDecl);
		sequenceLessType.addToDomainList(sequenceDecl);
		sequenceLessType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceLessDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("<"), sequenceLessType);
		standardTable.addFuncDeclaration(sequenceLessDecl);

		SOM_FunctionType sequenceLessEqualType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceLessEqualType.addToDomainList(sequenceDecl);
		sequenceLessEqualType.addToDomainList(sequenceDecl);
		sequenceLessEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceLessEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), sequenceLessEqualType);
		standardTable.addFuncDeclaration(sequenceLessEqualDecl);

		SOM_FunctionType sequenceGreaterType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceGreaterType.addToDomainList(sequenceDecl);
		sequenceGreaterType.addToDomainList(sequenceDecl);
		sequenceGreaterType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceGreaterDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), sequenceGreaterType);
		standardTable.addFuncDeclaration(sequenceGreaterDecl);

		SOM_FunctionType sequenceGreaterEqualType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceGreaterEqualType.addToDomainList(sequenceDecl);
		sequenceGreaterEqualType.addToDomainList(sequenceDecl);
		sequenceGreaterEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration sequenceGreaterEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), sequenceGreaterEqualType);
		standardTable.addFuncDeclaration(sequenceGreaterEqualDecl);

		SOM_FunctionType sequenceSizeType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceSizeType.addToDomainList(sequenceDecl);
		sequenceSizeType.setRange(natDecl);
		SOM_OperatorDeclaration sequenceSizeDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("$"), sequenceSizeType);
		standardTable.addFuncDeclaration(sequenceSizeDecl);

		SOM_FunctionType sequenceMaxType = new SOM_FunctionType(bunchMetaType);
		sequenceMaxType.addToDomainList(sequenceDecl);
		sequenceMaxType.addToDomainList(sequenceDecl);
		sequenceMaxType.setRange(natDecl);
		SOM_OperatorDeclaration sequenceMaxDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("max"), sequenceMaxType);
		standardTable.addFuncDeclaration(sequenceMaxDecl);
		
		SOM_FunctionType sequenceMinType = new SOM_FunctionType(bunchMetaType);
		sequenceMinType.addToDomainList(sequenceDecl);
		sequenceMinType.addToDomainList(sequenceDecl);
		sequenceMinType.setRange(natDecl);
		SOM_OperatorDeclaration sequenceMinDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("min"), sequenceMinType);
		standardTable.addFuncDeclaration(sequenceMinDecl);
		
		SOM_FunctionType sequenceHeadType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceHeadType.addToDomainList(sequenceDecl);
		sequenceHeadType.setRange(univDecl);
		SOM_OperatorDeclaration sequenceHeadDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("head"), sequenceHeadType);
		standardTable.addFuncDeclaration(sequenceHeadDecl);
		
		SOM_FunctionType sequenceTailType = 
			new SOM_FunctionType(bunchMetaType);
		sequenceTailType.addToDomainList(sequenceDecl);
		sequenceTailType.setRange(univDecl);
		SOM_OperatorDeclaration sequenceTailDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("tail"), sequenceTailType);
		standardTable.addFuncDeclaration(sequenceTailDecl);
		
		//-------------- The array operators   ------------------//
		SOM_FunctionType arrayEqualType = 
			new SOM_FunctionType(bunchMetaType);
		arrayEqualType.addToDomainList(arrayDecl);
		arrayEqualType.addToDomainList(arrayDecl);
		arrayEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayEqualDecl = 
			new SOM_OperatorDeclaration
			(new SOM_Identifier("="), arrayEqualType);
		standardTable.addFuncDeclaration(arrayEqualDecl);

		SOM_FunctionType arrayNotEqualType = 
			new SOM_FunctionType(bunchMetaType);
		arrayNotEqualType.addToDomainList(arrayDecl);
		arrayNotEqualType.addToDomainList(arrayDecl);
		arrayNotEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayNotEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("/="), arrayNotEqualType);
		standardTable.addFuncDeclaration(arrayNotEqualDecl);

		SOM_FunctionType arrayCatenationType = 
			new SOM_FunctionType(bunchMetaType);
		arrayCatenationType.addToDomainList(arrayDecl);
		arrayCatenationType.addToDomainList(arrayDecl);
		arrayCatenationType.setRange(arrayDecl);
		SOM_OperatorDeclaration arrayCatenationDecl = 
			new SOM_OperatorDeclaration
			(new SOM_Identifier("^"), arrayCatenationType);
		standardTable.addFuncDeclaration(arrayCatenationDecl);

		SOM_FunctionType arrayLessType = 
			new SOM_FunctionType(bunchMetaType);
		arrayLessType.addToDomainList(arrayDecl);
		arrayLessType.addToDomainList(arrayDecl);
		arrayLessType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayLessDecl = new SOM_OperatorDeclaration
				(new SOM_Identifier("<"), arrayLessType);
		standardTable.addFuncDeclaration(arrayLessDecl);

		SOM_FunctionType arrayLessEqualType = 
			new SOM_FunctionType(bunchMetaType);
		arrayLessEqualType.addToDomainList(arrayDecl);
		arrayLessEqualType.addToDomainList(arrayDecl);
		arrayLessEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayLessEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("=<"), arrayLessEqualType);
		standardTable.addFuncDeclaration(arrayLessEqualDecl);

		SOM_FunctionType arrayGreaterType = 
			new SOM_FunctionType(bunchMetaType);
		arrayGreaterType.addToDomainList(arrayDecl);
		arrayGreaterType.addToDomainList(arrayDecl);
		arrayGreaterType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayGreaterDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">"), arrayGreaterType);
		standardTable.addFuncDeclaration(arrayGreaterDecl);

		SOM_FunctionType arrayGreaterEqualType = 
			new SOM_FunctionType(bunchMetaType);
		arrayGreaterEqualType.addToDomainList(arrayDecl);
		arrayGreaterEqualType.addToDomainList(arrayDecl);
		arrayGreaterEqualType.setRange(boolDecl);
		SOM_OperatorDeclaration arrayGreaterEqualDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier(">="), arrayGreaterEqualType);
		standardTable.addFuncDeclaration(arrayGreaterEqualDecl);

		SOM_FunctionType arraySizeType = 
			new SOM_FunctionType(bunchMetaType);
		arraySizeType.addToDomainList(arrayDecl);
		arraySizeType.setRange(natDecl);
		SOM_OperatorDeclaration arraySizeDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("#"), arraySizeType);
		standardTable.addFuncDeclaration(arraySizeDecl);

		SOM_FunctionType arrayMaxType = new SOM_FunctionType(bunchMetaType);
		arrayMaxType.addToDomainList(arrayDecl);
		arrayMaxType.addToDomainList(arrayDecl);
		arrayMaxType.setRange(natDecl);
		SOM_OperatorDeclaration arrayMaxDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("max"), arrayMaxType);
		standardTable.addFuncDeclaration(arrayMaxDecl);
		
		SOM_FunctionType arrayMinType = new SOM_FunctionType(bunchMetaType);
		arrayMinType.addToDomainList(arrayDecl);
		arrayMinType.addToDomainList(arrayDecl);
		arrayMinType.setRange(natDecl);
		SOM_OperatorDeclaration arrayMinDecl = 
			new SOM_OperatorDeclaration
				(new SOM_Identifier("min"), arrayMinType);
		standardTable.addFuncDeclaration(arrayMinDecl);

        //-------------- Domain specific operators -----------------//


		//-------------- Global constants        -------------------//
		// null, the empty bunch
		nullDecl = new SOM_VariableDeclaration
			(new SOM_Identifier("null"), univDecl);
		standardTable.addDeclaration(nullDecl);

		// true
		trueDecl = new SOM_VariableDeclaration
			(new SOM_Identifier("true"), boolDecl);
		standardTable.addDeclaration(trueDecl);

		// false
		falseDecl = new SOM_VariableDeclaration
			(new SOM_Identifier("false"), boolDecl);
		standardTable.addDeclaration(falseDecl);

		// nil, the empty sequence
		nilDecl = new SOM_VariableDeclaration
			(new SOM_Identifier("nil"), sequenceDecl);
		standardTable.addDeclaration(nilDecl);
		
		// _, the generic parameter
		// This underscore declaration can be used to replace any type
		// i.e. an underscore symbol can replace any variable without
		// violating the type of that variable
		// Therefore, underscore has most generic type, i.e. univ	
		underscoreDecl = new SOM_VariableDeclaration
			(new SOM_Identifier("_"), underscoreType);
		standardTable.addDeclaration(underscoreDecl);

		logic = new SOM_VariableDeclaration
			(new SOM_Identifier("logic"), facetMetaType);
		standardTable.addDeclaration(logic);
		//logic.setMySymboltable(standardTable);

		state_based = new SOM_VariableDeclaration
			(new SOM_Identifier("state_based"), facetMetaType);
		standardTable.addDeclaration(state_based);
		//state_based.setMySymboltable(standardTable);

		continuous = new SOM_VariableDeclaration
			(new SOM_Identifier("continuous"), facetMetaType);
		standardTable.addDeclaration(continuous);
		//continuous.setMySymboltable(standardTable);

		discrete = new SOM_VariableDeclaration
			(new SOM_Identifier("discrete"), facetMetaType);
		standardTable.addDeclaration(discrete);
		//discrete.setMySymboltable(standardTable);

		//------------- Adding the standard table  -----------------//
		scopeTables.prependElement(standardTable);
	}

	/** To retrieve the bunchMetaType declaration
	 * @return The SOM_TypeDeclaration corresponding to the
	 * bunchMetaType 
	 */
	public SOM_TypeDeclaration getBunchType() {
		return bunchMetaType;
	}

	/** To retrieve the facetMetaType declaration
	 * @return The SOM_TypeDeclaration corresponding to facetMetaType
	 */
	public SOM_TypeDeclaration getFacetType() {
		return facetMetaType;
	}

        /** To retrieve the interactionMetaType declaration
         * @return The SOM_TypeDeclaration corresponding to interactionMetaType
         */
        public SOM_TypeDeclaration getInteractionType() {
                return interactionMetaType;
        }

	public SOM_TypeDeclaration getDomainType() {
		return domainMetaType;
	}

	/** To retrieve the componentMetaType declaration
	 * @return The SOM_TypeDeclaration corresponding to
	 * componentMetaType 
	 */
	public SOM_TypeDeclaration getComponentType() {
		return componentMetaType;
	}

	/** To retrieve the packageMetaType declaration
	 * @return The SOM_TypeDeclaration corresponding to
	 * packageMetaType 
	 */
	public SOM_TypeDeclaration getPackageType() {
		return packageMetaType;
	}
         //Added by Murthy
         /** To retrieve the typeMetaType declaration
	 * @return The SOM_TypeDeclaration corresponding to
	 * typeMetaType 
	 */
        public SOM_TypeDeclaration getTypedType() {
	  return typeMetaType;
	}


	/** To retrieve the setDecl declaration
	 * @return The SOM_TypeDeclaration corresponding to setDecl
	 */
	public SOM_TypeDeclaration getSetType() {
		return setDecl;
	}

	/** To retrieve the arrayDecl declaration
	 * @return The SOM_TypeDeclaration corresponding to arrayDecl
	 */
	public SOM_TypeDeclaration getArrayType() {
		return arrayDecl;
	}
	
	/** To retrieve the sequenceDecl declaration
	 * @return The SOM_TypeDeclaration corresponding to sequenceDecl
	 */
	public SOM_TypeDeclaration getSequenceType() {
		return sequenceDecl;
	}

	/** To retrieve the recordDecl declaration
	 * @return The SOM_TypeDeclaration corresponding to recordDecl
	 */
	public SOM_TypeDeclaration getRecordType() {
		return recordDecl;
	}

	/** To retrieve the tupleDecl declaration
	 * @return The SOM_TypeDeclaration corresponding to tupleDecl
	 */
	public SOM_TypeDeclaration getTupleType() {
		return tupleDecl;
	}

    // Added by Morel, June 29, 2001
    /** To retrieve the enumerationDecl declaration
     * @return The SOM_TypeDeclaration corresponding to enumerationDecl
     */
    public SOM_TypeDeclaration getEnumerationType() {
	return enumerationDecl;
    }

    // Added by Morel, June 29, 2001
    /** To retrieve the enumItemDecl declaration
     * @return The SOM_TypeDeclaration corresponding to enumItemDecl
     */
    public SOM_TypeDeclaration getEnumItemType() {
	return enumItemDecl;
    }

    // Added by Morel, June 29, 2001
    /** To retrieve the univDecl declaration
     * @return The SOM_TypeDeclaration corresponding to univDecl
     */
    public SOM_TypeDeclaration getUnivType() {
	return univDecl;
    }

	/** To retrieve the real type declaration
	 * @return The SOM_TypeDeclaration corresponding to real
	 */
	public SOM_TypeDeclaration getRealType() {
		return realDecl;
	}

	/** To retrieve the rational type declaration
	 * @return The SOM_TypeDeclaration corresponding to rational
	 */
	public SOM_TypeDeclaration getRationalType() {
		return rationalDecl;
	}

	/** To retrieve the integer type declaration
	 * @return The SOM_TypeDeclaration corresponding to integer
	 */
	public SOM_TypeDeclaration getIntegerType() {
		return intDecl;
	}

	/** To retrieve the natural type declaration
	 * @return The SOM_TypeDeclaration corresponding to natural
	 */
	public SOM_TypeDeclaration getNaturalType() {
		return natDecl;
	}

	/** To retrieve the bit type declaration
	 * @return The SOM_TypeDeclaration corresponding to bit
	 */
	public SOM_TypeDeclaration getBitType() {
		return bitDecl;
	}

	/** To retrieve the boolean type declaration
	 * @return The SOM_TypeDeclaration corresponding to boolean
	 */
	public SOM_TypeDeclaration getBooleanType() {
		return boolDecl;
	}

	/** To retrieve the char type declaration
	 * @return The SOM_TypeDeclaration corresponding to char
	 */
	public SOM_TypeDeclaration getCharacterType() {
		return charDecl;
	}

	/** To retrieve the char type declaration
	 * @return The SOM_TypeDeclaration corresponding to char
	 */
	//public SOM_TypeDeclaration getUnderscoreType() {
	//	return underscoreDecl;
	//}

	/** To create the currentSymboltable
	 * @param name The string to identify the symboltable
	 * @param decl The SOM_Declaration for which this symbol table is
	 * being made
	 * @return void
	 */
	public void createNewCurrentSymboltable(String name,
											SOM_Declaration decl) { 
		// Every new table is created by appending the parameter name
		// to the name of the current symboltable separated by a "."
		// This way one can keep track of the nesting
		String newTableName;

		if (currentTable != null)
			newTableName = new String(currentTable.getName() + "." +
									  name); 
		else newTableName = new String(name);
		
		currentTable = new SYM_Symboltable(newTableName, decl);
		//System.out.println("Creating new current symbol table: " +
		//				   newTableName);
		scopeTables.prependElement(currentTable);
	}

	/** To retrieve the current symboltable
	 * @return The SYM_Symboltable that is the current symbol table
	 */
	public SYM_Symboltable getCurrentSymboltable() {
		return currentTable;
	}

	/** To serialize the currentSymboltable
	 * @return void
	 */
	public void serializeCurrentSymboltable(String symFileName) {
		try {
			//FileOutputStream fos =
			//	new FileOutputStream(currentTable.getName() + ".sym");
			FileOutputStream fos =
				new FileOutputStream(symFileName + ".sym");
			GZIPOutputStream gzos = new GZIPOutputStream(fos);
			ObjectOutputStream out = new ObjectOutputStream(gzos);
			out.writeObject(currentTable);
			out.flush();
			out.close();
		} catch (FileNotFoundException e) {
			throw new RuntimeException("The serialized file for " + 
									   currentTable.getName() +
									   " not found");
		} catch (IOException e) {
			throw new RuntimeException("Problem with serialization of" 
									   + currentTable.getName());
		} 
		
	}
	
	/** To deserialize the symboltable
	 * @param tableName name of symboltable to be deserialized
	 * @return void
	 */
	public void deserializeSymboltable(String tableName) {
		try {
			FileInputStream fin =
				new FileInputStream(tableName + ".sym");
			GZIPInputStream gzin = new GZIPInputStream(fin);
			ObjectInputStream in = new ObjectInputStream(gzin);
			SYM_Symboltable tableIn =
				(SYM_Symboltable)in.readObject(); 
			in.close();
			currentTable = tableIn;
			scopeTables.prependElement(tableIn);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException("Problem: deserialization of" 
									   + tableName);
		} catch (FileNotFoundException e) {
			throw new SYM_SymboltableNotFound("The serialized file for " + 
									   tableName + " not found", tableName);
		} catch (IOException e) {
			throw new RuntimeException("Problem: deserialization of" 
									   + tableName);
		}
		// Testing what tables are found in scopeTables
		//ListEnumeration scopeEnum = scopeTables.elements();
		//while (scopeEnum.hasMoreElements()) {
		//	SYM_Symboltable currSymtab =
		//		(SYM_Symboltable)scopeEnum.nextElement();
			//System.out.println("Symboltable name: " +
			//				   currSymtab.getName());
		//}
	}

	/** To be invoked after the current symbol table is no longer
	 * needed. 
	 * This removes the current element at the head of the scopeTables
	 * list and a fresh one will be then picked up from the new head
	 * of the scopeTables List
	 * @return void
	 */
	public void doneWithCurrentSymboltable() {
		scopeTables.pop();
		currentTable = (SYM_Symboltable)scopeTables.first();
	}
		
	/** To open a scope for the current symbol table
	 * @return void
	 */
	public void openScope() {
		currentTable.openScope();
	}
	
	/** To close the current scope of the current symbol table
	 * @return void
	 * @exception SYM_NullScopeException if this function is
	 * called when scope is 0
	 */
	public void closeScope() {
		currentTable.closeScope();
	}
	
	/** To add a declaration for variables to the current symbol
	 * table. Update this 
	 * declIn The SOM_Declaration to be added
	 * @return void
	 */
	public void addDeclaration(SOM_Declaration decl) {
		currentTable.addDeclaration(decl); // this also sets the
		// parentSymboltable 
	}

	/** To add a declaration for functions to the current symbol
	 * table. Update this 
	 * @param declIn The SOM_Declaration to be added
	 * @return void
	 */
	public void addFuncDeclaration(SOM_Declaration declIn) {
		currentTable.addFuncDeclaration(declIn); // this also sets the
		// parentSymboltable 
	}


	/** To lookup a name
	 * @param nameIn The SOM_Name to be lookedup
	 * @return A SOM_DeclarationList corresponding to all the
	 * declarations for nameIn
	 */
	public SOM_DeclarationList lookUp(SOM_Name nameIn) {
		SOM_DeclarationList declList = new SOM_DeclarationList();

		ListEnumeration scopeEnum = scopeTables.elements();
		while (scopeEnum.hasMoreElements()) {
			try {
				SYM_Symboltable currSymtab =
					(SYM_Symboltable)scopeEnum.nextElement(); 
				declList = currSymtab.lookUp(nameIn);
				return declList; // the moment a table returns a decl
				// list... 
				// no need to look in previous scope tables
				//System.err.println("Found decl for " +
				//nameIn.toDotSeparatedString() + " in table " +
				//currSymtab.getName()); 
			} catch (SYM_NoDeclarationException except) {
				// safely ignore this exception thrown by some
				// symboltable 
				//System.err.println(except); // for info purposes
			}
		}
		if (declList.isEmpty()) {
			throw new SYM_NoDeclarationException
				("SYM_SymboltableManager: No Declaration found for " 
				 + nameIn.toDotSeparatedString()); 
		}
		return declList;
	}
	
	/** To lookup a name with a particular kind - not to be confused
	 * with lookUpUniqueForKind
	 * @param nameIn The SOM_Name to be lookedup
	 * @param kindIn One of the constants  in SOM_Declaration ()
	 * @return A SOM_DeclarationList corresponding to the kind
	 */
	public SOM_DeclarationList lookUpForKind(SOM_Name nameIn, int
											 kindIn) { 
		ListEnumeration enum = lookUp(nameIn).elements();

		SOM_Declaration currDecl;
		SOM_DeclarationList toReturn = new SOM_DeclarationList();
		while (enum.hasMoreElements()) {
			currDecl = (SOM_Declaration)enum.nextElement();
			if (currDecl.isKind(kindIn))
				toReturn.addElement(currDecl); 
		} 
		
		return toReturn;
	}

	/** To lookup a name with a unique declaration with a particular
	 *	kind. It returns the first declaration encountered.
	 * @param nameIn The SOM_Name to be lookedup
	 * @param kindIn One of the constants  in SOM_Declaration ()
	 * @return A SOM_Declaration corresponding to the kind
	 */
	public SOM_Declaration lookUpUniqueForKind(SOM_Name nameIn, int
											   kindIn) { 
		SOM_DeclarationList declList = lookUpForKind(nameIn,kindIn);

		if (declList.countElements() != 1)
			throw new SYM_UnresolvableException
				("SYM_SymboltableManager: Empty/Multiple resolution for " 
				 + nameIn.toDotSeparatedString());

		return (SOM_Declaration)declList.first();
	}

	/** To lookup a string. It assumes that the input string can be
	 * converted to a SOM_Name
	 * @param hashString the string to be looked up
	 * @return The SOM_DeclarationList corresponding to the hashString */
	public SOM_DeclarationList lookUp(String hashString) {
		return lookUp(SOM_Name.forString(hashString));
	}

	/** To lookup a string of a particular kind . It assumes that the
	 * input string can be converted to a SOM_Name
	 * @param hashString The name to be looked up
	 * @param kindIn One of the constants  in SOM_Declaration ()
	 * @return The SOM_DeclarationList corresponding to the hashString */
	public SOM_DeclarationList lookUpForKind(String hashString, int
											 kindIn) { 
		// try to use the other overloaded version of this function
		return lookUpForKind(SOM_Name.forString(hashString),kindIn);
	}

	/** To specifically look up a string in the standard Symboltable
	 * @param hashString The string to be looked up
	 * @param kindIn One of the constants  in SOM_Declaration ()
	 * @return The SOM_DeclarationList corresponding to the hashString
	 */
	/*
	public SOM_DeclarationList lookUpForKindInStandardTable(String
	 hashString, int kindIn) { 
		ListEnumeration enum = standardTable.lookUp(nameIn).elements();

		SOM_Declaration currDecl;
		SOM_DeclarationList toReturn = new SOM_DeclarationList();
		while (enum.hasMoreElements()) {
			currDecl = (SOM_Declaration)enum.nextElement();
			if (currDecl.isKind(kindIn)) toReturn.addElement(currDecl);
		} 
		return toReturn;
	}
	*/
}
