//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: Sldl.java,v 1.18 2001/11/02 03:22:17 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Sldl.java,v $
//  Revision 1.18  2001/11/02 03:22:17  cvs
//  Modified Sldl.java to catch the new exception and recursively call itself to try and parse the subfile first and then parse the mail file.
//    --Joseph,Garrin,Wang,Murthy.
//
//  Revision 1.17  2001/10/01 18:43:56  cvs
//  I added a flag such that when false, the type info is not printed
//  out when a rosetta file is parsed. If type info is wanted for
//  debugging, open Sldl.java and change the printInfo flag to true.
//  -- Cindy
//
//  Revision 1.16  2001/08/28 23:10:59  cvs
//  Added the initial implementation of Domains & Interactions
//   --Murthy
//
//  Revision 1.15  2000/12/04 19:35:59  cvs
//  ROM is serialized and stored in packageName_ROM.ser
//  Use clause can now be used for importing packages in same directory.
//  -- Cindy
//
//  Revision 1.14  2000/11/13 21:56:06  cvs
//  Calling the method buildROM from the main method
//  --srinivas
//
//  Revision 1.13  2000/11/06 18:29:12  cvs
//  Added the statement to build the ROM.
//  -- Cindy
//
//  Revision 1.12  2000/04/21 00:40:54  cvs
//  changed the name of serialzed output file to <filename>.ser
//
//  Revision 1.11  2000/03/16 01:04:19  cvs
//  shifted the "parsed completely" message before serialization - makarand
//
//  Revision 1.10  2000/03/03 18:38:31  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.9  2000/02/10 03:51:13  cvs
//  Updated the Copyright information in some of the files which were
//  omitted in the previous commit 
//  - krishna
//
//  Revision 1.8  2000/02/10 00:37:39  cvs
//  Changed Copyright information to University of Kansas
//  - by makarand
//
//  Revision 1.7  2000/02/08 18:42:46  cvs
//  Completed populating the object model and publishing the object model.
//  Added 'publishSOM(PrintWriter)' method to all necessary classes
//  (with the help 
//  of Makarand and Srini). Removed some unnecessary classes (due to changes
//  brought to the grammar). Renamed SOM_EqualityOperator to
//  SOM_EqualEqualOperator. Modified SldlParser.jj.h to populate the
//  object model. 
//  -- Cindy.
//
//  Revision 1.6  1999/12/22 01:58:59  makarand
//  Added System.exit(0) for the regression test script which assumes
//  return value 
//  of 0 for successful completion
//  appended the classpath to include proper directories
//
//  Revision 1.5  1999/11/15 03:16:33  cvs
//  Many changes - especially with unfolding of overloading. Everything
//  compiles now. Some features are yet to be coded...but it is now again
//  a rudimentary parser (as it was two weeks ago :)
//
//  Revision 1.4  1999/10/27 20:43:58  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.4  1999/10/20 21:52:40  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.5  1999/10/20 21:20:55  rkamath
//  Commit before the archive moves over to Kansas. This is not complete - and
//  may not compile...
//
//  Revision 1.4  1999/07/06 21:03:28  rkamath
//  Wth the new type expression syntax....
//
//  Revision 1.3  1999/06/30 02:20:14  rkamath
//  An intermediate commit for safey purposes. sldl.jj REALLY needs it :)
//
//  Revision 1.2  1999/06/20 03:06:32  rkamath
//  Moved the Makefile.config and made corresponding changes in the
//  other makefiles 
//  Also some changes to the jj and Sldl.java file
//
//  Revision 1.1  1999/06/14 22:51:48  rkamath
//  Added these fresh files as the start of a rudimentary parser.
//  Currently includes the facet composition expression. Other things
//  will be added gradually
//
//
//=============================================================================
package src;

import java.io.FileReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.Exception;

// added by ckong
import SOM.*;
import java.io.*;
import ROM.Universe;
import SYM.SYM_SymboltableNotFound;

public class Sldl extends Object {

  // Flag that prints type info when set to true
  public static boolean printInfo = false;

  public static void main(String args[]) throws Exception {
    System.out.println("\nRosetta Parser, Rudimentary Prototype");
    System.out.println("Copyright(C) 2000 The University of Kansas");
    System.out.println("All rights reserved.\n\n");
    //Added on 30 oct 2001	
    parseFile(args[0]);
  }

  public static void parseFile(String filename) 
    throws ParseException, IOException, FileNotFoundException {
      
    boolean serializeSOM = true; 

    System.out.println("Parsing file: " + filename);
    try {
      FileReader fileReader = new FileReader(filename);
      String serialFile = filename.substring(0,filename.indexOf(".sld"));
      //SldlParser parser = new SldlParser(fileReader,args[0]);
      SldlParser parser = new SldlParser(fileReader,serialFile);
      SOM_DesignFile dFile = parser.designFile();

      System.out.println("\nFile parsed completely\n");
      
      //added by ckong					
      FileOutputStream fileOutput = new FileOutputStream("ObjectModel.txt");
      PrintWriter printWriter = new PrintWriter(fileOutput, true);
      dFile.publishSOM(printWriter);
      
      // To build the loader
      //     System.out.println("Begin Building ROM");
      Universe loaderRoot = dFile.buildROM();
      //System.out.println("Building ROM -- Completed");
      FileOutputStream romOutput = 
	new FileOutputStream("ROMObjectModel.txt");
      PrintWriter pw = new PrintWriter(romOutput, true);
      //System.out.println("Start printing ROMObjectModel.txt");
      loaderRoot.publishROM(pw);
      //System.out.println("Finished printing ROMObjectModel.txt");
      
      // added by makarand for Serialization of Object model
      if (serializeSOM){
	//System.out.println("Begin Serialization .gr. ");
	//String serialfile = args[0].substring(0,args[0].indexOf(".sld"));
	FileOutputStream fileout =
	  new FileOutputStream(serialFile + ".ser");
	ObjectOutputStream out = new ObjectOutputStream(fileout);
	out.writeObject(dFile);
	out.flush();
	out.close();
	//System.out.println("Object model serialized");
      }
      // ******* end changes - mak ************
      
      printWriter.flush();					
      fileReader.close();
      printWriter.close();
      fileOutput.close();
      
      
    } catch (FileNotFoundException e) {
      System.err.println("Serious Error: File " + filename + 
			 " is not available\n");
      throw e;
    } catch (IOException except) {   // added by ckong
      System.err.println(except);
    }catch (SYM_SymboltableNotFound e) {
      parseFile(e.getFilename()+".sld");
      parseFile(filename);
    } // end of catch
    
   }
}









