//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: AssociatedList.java,v 1.1 2001/05/24 20:09:33 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: AssociatedList.java,v $
//  Revision 1.1  2001/05/24 20:09:33  cvs
//  Wrote my own version of pairs and associated list, i.e. list of
//  pairs. Similar to Hashtable, except that we can have two pairs of keys
//  and values with the same keys.
//  -- Cindy
//
//
//
//=============================================================================

package util;

import java.io.*;
import java.util.*;
import util.*;
import SOM.*;
import ROM.*;

/** This represents the <B>AssociatedList</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */

public class AssociatedList {

	LinkedList assocList;
	
	/**
	 * An associated list is a list of pairs. It may contain several
	 * pairs with the same key. 
	 * @return Not applicable
	 */
	public AssociatedList() {
		assocList = new LinkedList();
	}

	/**
	 * A second constructor
	 * @param pairIn a first pair for the list
	 * @return Not applicable
	 */
	public AssociatedList(Pair pairIn) {
		assocList = new LinkedList();
		assocList.add(pairIn);
	}

	/**
	 * A third constructor.
	 * @param keyIn, key of the pair
	 * @param valueIn value of the pair
	 * @return Not applicable
	 */
	public AssociatedList (Object keyIn, Object valueIn) {
		assocList = new LinkedList();
		assocList.add(new Pair(keyIn, valueIn));
	}
	
	/**
	 * To add a pair.
	 * @param pairIn The pair to add
	 * @return void
	 */
	public void addPair(Pair pairIn) {
		assocList.add(pairIn);
	}

	/**
	 * To add a pair given a key and a value
	 * @param keyIn Key of the pair to add
	 * @param valueIn Value of the pair to add
	 * @return void
	 */
	public void addPair(Object keyIn, Object valueIn) {
		assocList.add(new Pair(keyIn, valueIn));
	}

	/**
	 * To add the members of another list to this list
	 * @param listIn the list whose members is to be added
	 * @return void
	 */
	public void addList(AssociatedList listIn) {
		assocList.addAll((Collection)listIn.getList());
	}

	/**
	 * To get the list
	 * @return The list of the associated list
	 */
	public LinkedList getList() {
		return assocList;
	}

	/**
	 * To get the pairs in the list
	 * @return A list iterator of pairs
	 */
	public ListIterator getPairs() {
		return assocList.listIterator();
	}

	/** 
	 * To get the keys of all pairs in the list
	 * @return A list of keys
	 */
	public LinkedList getKeys() { 
		LinkedList toReturn = new LinkedList();
		ListIterator iter = assocList.listIterator();
		Pair aPair;
		while (iter.hasNext()) {
			aPair = (Pair)iter.next();
			toReturn.add(aPair.getKey());
		}
		return toReturn;
	}

	/** 
	 * To get the values of all pairs in the list
	 * @return A list of values
	 */
	public LinkedList getValues() { 
		LinkedList toReturn = new LinkedList();
		ListIterator iter = assocList.listIterator();
		Pair aPair;
		while (iter.hasNext()) {
			aPair = (Pair)iter.next();
			toReturn.add(aPair.getValue());
		}
		return toReturn;
	}

	/** 
	 * To get the values associated with a key
	 * @param keyIn the key being looked up
	 * @return A list of values 
	 */ 
	public LinkedList getValuesForKey(Object keyIn) { 
		LinkedList toReturn = new LinkedList();
		ListIterator iter= assocList.listIterator();
		Pair aPair;
		while (iter.hasNext()) {
			aPair = (Pair)iter.next();
			if (aPair.getKey().equals(keyIn))
				toReturn.add(aPair.getValue());
		}
		return toReturn;
	}

}
