//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Roshan rkamath@ececs.uc.edu
//
//  $Id: ListEnumeration.java,v 1.4 2000/05/18 18:04:23 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: ListEnumeration.java,v $
//  Revision 1.4  2000/05/18 18:04:23  cvs
//  Due to /projects directory failing and the latest version available is old, I
//  am committing the file again.
//  -- Cindy
//
//  Revision 1.4  2000/05/05 19:36:58  cvs
//  List.java is now inheriting from LinkedList (java1.2).
//  ListEnumeration inherits from ListIterator.
//  This is done so that all predefined methods are available.
//  The methods in List.java and ListEnumeration.java now simply call the
//  corresponding methods from their super class.
//  -- Cindy
//
//  Revision 1.3  2000/03/03 18:38:33  cvs
//  Seriliazing and deserializing the object model - makarand
//
//  Revision 1.2  2000/02/10 03:51:12  cvs
//  Updated the Copyright information in some of the files which were omitted in the previous commit
//  - krishna
//
//  Revision 1.1  1999/10/27 20:43:59  cvs
//  The huge commit - shifting the sldl-lrm archive from uc to kansas. The
//  SldlParser.jj still has holes - but committing it anyway.
//
//  Revision 1.1  1999/10/20 21:52:51  rkamath
//  The HUGE commit (moving from my personal archive to the "kbse" archive)
//  before it moves to Kansas...
//  These files will have the log-info from my archive; they can be ignored.
//  Work on this can start afresh once all the files are in place
//  The src/SldlParser.jj.pre has "holes" - but committing it anyway
//
//  Revision 1.2  1999/07/24 23:21:08  rkamath
//  Added a copy constructor for ListEnumeration and modified concat in List
//  to concatenate a copy of the second argument.....
//
//  Revision 1.1  1999/07/13 19:29:56  rkamath
//  The new util directory with the basic data structures
//
//
//=============================================================================
package util;

import java.io.*;
import java.util.ListIterator;

/** This represents the <B>ListEnumeration</B> object
 * @author <I>Roshan Kamath</I> (rkamath@ececs.uc.edu)
 */
public class ListEnumeration
extends Object
implements Serializable {

	ListIterator listIter;

	/** Constructor
	 * @param listIterIn, the ListIterator to traverse
	 * @return Not applicable
	 */
	public ListEnumeration(ListIterator listIterIn) {
		listIter = listIterIn;
	}

	/** To check if the more elements exist
	 * @return true if there are still elements left; false otherwise
	 */
	public boolean hasMoreElements() {
		if (listIter == null)
			return false;
		return listIter.hasNext();
	}

	/** To retrieve the current element. Throws NullPointerException if 
	 * hasMoreElements would have returned false.
	 * @return The object currently pointed to
	 */
	public Object nextElement(){
		if (listIter == null)
			return null;
		return listIter.next();
	}

	/** To delete the element just returned by next.
	 * @return void
	 */
	public void remove(){
		if (listIter != null)
			listIter.remove();
	}
}
