//============================================================================
//  Copyright (c) 2000 University of Kansas 
//  All rights reserved.
//
//  KU MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF 
//  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
//  PARTICULAR PURPOSE, OR NON-INFRINGEMENT.  KU SHALL NOT BE LIABLE
//  FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
//  MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
//
//
//  By using or copying this Software, Licensee agrees to abide by the
//  intellectual property laws, and all other applicable laws of the
//  U.S., and the terms of this license.
//
//	authors: Cindy Kong ckong@ittc.ukans.edu
//
//  $Id: Pair.java,v 1.1 2001/05/24 20:09:33 cvs Exp $
//
//  Revision History:
//  ----------------
//  $Log: Pair.java,v $
//  Revision 1.1  2001/05/24 20:09:33  cvs
//  Wrote my own version of pairs and associated list, i.e. list of
//  pairs. Similar to Hashtable, except that we can have two pairs of keys
//  and values with the same keys.
//  -- Cindy
//
//
//
//=============================================================================

package util;

import java.io.*;
import java.util.*;
import util.*;
import SOM.*;
import ROM.*;

/** This represents the <B>Pair</B> object
 * @author <I>Cindy Kong</I> (ckong@ittc.ukans.edu)
 */

public class Pair {

	private Object key;
	private Object value;
	
	/**
	 * A pair is of the form (value1, value2). However, we consider a pair
	 * as a 2-tuple with a key and a value, i.e. (key, value)
	 * @param keyIn, key of the pair
	 * @param valueIn value of the pair
	 * @return Not applicable
	 */
	public Pair (Object keyIn, Object valueIn) {
		key = keyIn;
		value = valueIn;
	}
	
	/** 
	 * To get the key
	 * @return The key of the pair
	 */
	public Object getKey() { 
		return key;
	}

	/** 
	 * To set the key
	 * @param keyIn the new key to be set
	 * @return void
	 */
	public void setKey(Object keyIn) { 
		key = keyIn;
	}

	/** 
	 * To get the value
	 * @return The value of the pair
	 */
	public Object getValue() { 
		return value;
	}

	/** 
	 * To set the value
	 * @param valueIn the new value to be set
	 * @return void
	 */
	public void setValue(Object valueIn) { 
		value = valueIn;
	}

	/**
	 * To know if two pairs are equal
	 * @param otherPair The pair to compare with
	 * @return boolean, true if pair equal
	 */
	public boolean equals(Pair otherPair) {
		if (getKey().equals(otherPair.getKey())
			&& getValue().equals(otherPair.getValue()))
			return true;
		else
			return false;
	}
}
